<?php

namespace Application\Controller;

use Application\SessionStorage;
use Zend\View\Model\ViewModel;

use Zend\Uri\Uri;

use ZendServer\Exception;

use ZendServer\Validator\UriPath;

use Users\Identity;

use Audit\Db\ProgressMapper;

use Audit\Db\Mapper;

use ZendServer\Configuration\Manager;

use ZendServer\Mvc\Controller\ActionController,
	Application\Module,
	ZendServer\Log\Log,
	Application\Forms\Login;
	 

class LoginController extends ActionController
{
	protected $version;
	protected $build;
	protected $licenseType;
	protected $licenseExpiryDate;
	protected $authenticationSource;
	protected $licenseEvaluation;
	protected $licenseIsOk;
	protected $daysToExpired;
	protected $licenseNeverExpires;

	/**
	 * @var \Application\Forms\Login
	 */
	protected $loginForm;	
	
	
	public function indexAction() {

		$params = $this->getParameters(array('redirectTo' => ''));
		$redirectTo = $this->validateRedirectTo($params['redirectTo']);
		if ($this->Authentication()->hasIdentity()) { // User has already authenticated
			$this->Redirect()->toUrl($redirectTo);
		}

		$this->authenticationSource = Module::config('authentication', 'simple') ? 'simple' : 'extended';
		
		$this->version = Module::config('package', 'version');//get the current version of server
		$this->build = Module::config('package', 'build');//get the current build of server
		$utilsWrapper = new \Configuration\License\ZemUtilsWrapper(); 
		list($this->licenseType, $this->licenseExpiryDate, $this->licenseEvaluation) = $this->getLicenseDetails();// get the license details
		
		$this->Layout('layout/login');
		$request = $this->getRequest(); /* @var $request \Zend\Http\PhpEnvironment\Request */		
		if (!$request->isPost()) {
			/// this page must explicitly not be loaded as an ajax call
			return $this->getPageComponents(null, $redirectTo) + array(
						'isAjaxRequest' => $this->getRequest()->isXmlHttpRequest(),
					);
		}
		
		$this->loginForm = $this->getLocator('Application\Forms\Login');
		$parameters = $request->getPost()->toArray();
		$this->loginForm->setData($parameters);
		
		$this->auditMessage()->setIdentity(new Identity($parameters['username']));
		
		if (!$this->loginForm->isValid()) {			
			$nonValidElements = '';
			foreach ($this->loginForm->getMessages() as $field => $errors) {
				if (!$errors) continue;
				$nonValidElements .= $field . ',';
			}
			
			$errorMessage = 'Invalid input in field(s): ' . rtrim($nonValidElements, ',');
			$this->auditMessage(Mapper::AUDIT_GUI_AUTHENTICATION, ProgressMapper::AUDIT_PROGRESS_ENDED_FAILED, array(array($errorMessage)));
			return $this->getPageComponents($errorMessage);
		}
		
		if ($parameters['username'] != Module::config('user', 'zend_gui', 'adminUser') && (! $this->isAclAllowedEdition('data:useMultipleUsers'))) {
			$errorMessage = _t('Only the \'admin\' user is allowed to use this edition of Zend Server');
			$this->auditMessage(Mapper::AUDIT_GUI_AUTHENTICATION, ProgressMapper::AUDIT_PROGRESS_ENDED_FAILED, array(array($errorMessage)));
			return $this->getPageComponents($errorMessage);
		}
		
		if (!$this->Authentication()->authenticate($parameters['username'], $parameters['password'])){
			$errorMessage = _t('Username or password incorrect');
			$this->auditMessage(Mapper::AUDIT_GUI_AUTHENTICATION, ProgressMapper::AUDIT_PROGRESS_ENDED_FAILED, array(array($errorMessage)));
			return $this->getPageComponents($errorMessage);
		}

        // re-randomize the session id
        $manager = $this->getLocator()->get('Zend\Session\SessionManager'); /* @var $manager \Zend\Session\SessionManager */
        $manager->regenerateId();

		$this->auditMessage(Mapper::AUDIT_GUI_AUTHENTICATION, ProgressMapper::AUDIT_PROGRESS_ENDED_SUCCESFULLY);
		Log::info("authentication completed successfully");

		$this->setLibrariesUpdateCookie();
		
		if ($redirectTo) {
			$decoded = urldecode($redirectTo);
			if ($decoded == '/') {
				$decoded = '/ZendServer';
			}
			Log::debug("Redirect to" . $decoded);
			$this->Redirect()->toUrl($decoded);
		} else {
			$this->Redirect()->toRoute('home');
		}
		return $this->getResponse();
	}
	
	public function logoutAction() {
		$this->Authentication()->getAuthService()->clearIdentity();
		$routeMatch  = $this->getEvent()->getRouteMatch();
		$collectCurrentUrl = $routeMatch->getParam('collectCurrentUrl', false);

        /// clean up session
        $manager = $this->getLocator()->get('Zend\Session\SessionManager'); /* @var $manager \Zend\Session\SessionManager */
        $manager->destroy();
        $manager->regenerateId();

		$viewModel = new ViewModel(array('loginUrl' => Module::config('loginUrl')));
		$viewModel->setVariable('collectCurrentUrl', $collectCurrentUrl && ($routeMatch->getMatchedRouteName() != 'home'));
		$viewModel->setTerminal(true);
		return $viewModel;
	}


	/**
	 * @param string $redirectTo
	 * @return string
	 */
	private function validateRedirectTo($redirectTo) {
		$uri = new Uri($redirectTo);

		$validator = new UriPath();
		if (! $validator->isValid($uri->getPath())) {
			Log::notice('URL to redirect to was malformed, redirect to home');
			return '';
		}
		$redirectTo = $uri->getPath();
		$redirectTo = is_null($redirectTo) ? '/' : $redirectTo;
		if ($uri->getQuery()) {
			$redirectTo .= "?{$uri->getQuery()}";
		}
		
		if ($uri->getFragment()) {
			$redirectTo .= "#{$uri->getFragment()}";
		}
		
		return $redirectTo;
	}
	
	private function getPageComponents($error='', $redirectTo = '') {
		if (!$this->loginForm) {
			$this->loginForm = new Login();
		}
		
		if ($redirectTo) {
			$uri = new Uri($redirectTo);
			$redirectTo = $uri->getPath();
			if ($uri->getQuery()) {
				$redirectTo .= '?'.Uri::encodeQueryFragment($uri->getQuery());
			}
			if ($uri->getFragment()) {
				$redirectTo .= '#'.Uri::encodeQueryFragment($uri->getFragment());
			}
			$this->loginForm->setData(array('redirectTo' => urlencode($redirectTo)));
		}
		
		$manager = new Manager();
		$pageComponents = array(
					'form' 	=> $this->loginForm,
					'version' => $this->version,
					'build' => $this->build,
					'licenseType' => $this->licenseType,
					'licenseExpiryDate' => $this->licenseExpiryDate,
					'licenseEvaluation' => $this->licenseEvaluation,
					'authenticationSource' => $this->authenticationSource,
					'licenseIsOk' => $this->licenseIsOk,
					'licenseNeverExpires' => $this->licenseNeverExpires,
					'daysToExpired' => $this->daysToExpired,
					'osType' => $manager->getOsType(),
				);

		if ($error) {
			$pageComponents += array('error' => _t($error));
		}
		
		return $pageComponents;		
	}
	
	private function getLicenseDetails() {
		$utilsWrapper = new \Configuration\License\ZemUtilsWrapper();		
		$this->licenseNeverExpires = $utilsWrapper->getLicenseInfo()->isNeverExpires();
		$this->licenseIsOk = $utilsWrapper->getLicenseInfo()->isLicenseOk();
		$this->daysToExpired = $utilsWrapper->getLicenseExpirationDaysNum();
		return array($utilsWrapper->getLicenseType(), $utilsWrapper->getLicenseFormattedExpiryDate(), $utilsWrapper->getLicenseEvaluation());
	}
}
