<?php

namespace Application\Controller;

use Zend\View\Model\ViewModel;

use ZendServer\Mvc\Controller\ActionController,
	Application\Module,
	Application\Forms\Settings,
	ZendServer\Configuration\Ui\Directives\Container,
	Audit\Db\ProgressMapper,
	Audit\Db\Mapper as auditMapper,
	Zsd\Db\TasksMapper,
	ZendServer\Configuration\Manager,
	ZendServer\Log\Log;

class SettingsController extends ActionController
{
    public function indexAction() {
    	// Get the restart strategy settings
    	$directivesMapper = $this->getLocator()->get('Configuration\MapperDirectives'); /* @var $directivesMapper \Configuration\MapperDirectives */
    	$restartStrategy = $directivesMapper->selectSpecificDirectives(array('zend_server_daemon.restart_strategy'));
    	$restartStrategyValue = $directivesMapper->selectSpecificDirectives(array('zend_server_daemon.restart_strategy_value'));
    	$guiHostName = $directivesMapper->selectSpecificDirectives(array('zend_monitor.gui_host_name'));
    	
    	$generalSettingsForm = new \Application\Forms\Settings\General(array('guiHostName' => $guiHostName->current()));

    	$notificationAction = $this->getNotificationsActionMapper()->getNotification('restartRequired'); // get email of the restartRequired type as representative
    	$notificationAction = $notificationAction[0];
    	$notificationCenterForm = new \Application\Forms\Settings\NotificationCenter(array('notificationAction' => $notificationAction));
    	
    	$auditEmail = $this->getAuditSettingsMapper()->getEmail();
    	$auditScriptUrl = $this->getAuditSettingsMapper()->getScriptUrl();
    	$auditForm = new \Application\Forms\Settings\Audit(array('auditEmail' => $auditEmail, 'auditScriptUrl' => $auditScriptUrl));
    	
    	$mailForm = new \Application\Forms\Settings\Mail();
    	
    	$allowToEditSettings = $this->isAclAllowed('data:editSettings');
    	$allowToEditRestart = $this->isAclAllowed('data:editRestartSettings');
    	
    	if (! $allowToEditSettings) {
	    	$mailForm->disableForm();
	    	$this->setDisabledByEditionMessage($mailForm);
    	}
    	
    	if (! $allowToEditSettings) {
    		$auditForm->disableForm();
    		$this->setDisabledByEditionMessage($auditForm);
    	} elseif (! $this->isAclAllowed('data:editAuditSettings')) {
    		$auditForm->disableFormElement($auditForm->get('auditCustomAction'));
    		$this->setDisabledByEditionMessage($auditForm);
    	}
    	
    	if (! $allowToEditSettings) {
    		$notificationCenterForm->disableForm();
    		$this->setDisabledByEditionMessage($notificationCenterForm);
    	} elseif (! $this->isAclAllowed('data:editNotificationsSettings')) {
    		$notificationCenterForm->disableFormElement($notificationCenterForm->get('notificationsCustomAction'));
    		$this->setDisabledByEditionMessage($notificationCenterForm);
    	}
    	
    	$licenseType = $this->getLocator()->get('Configuration\License\ZemUtilsWrapper')->getLicenseType();
    	$licenseType = ucfirst(strtolower($licenseType));
    	
    	//INSERT OR IGNORE INTO GUI_ACL_RESOURCES VALUES(79,'data:editRestartSettings');
    	$manager = new Manager();
		return array('allowToEditRestart' => $allowToEditRestart,
					 'allowToEditSettings' => $allowToEditSettings,
				 	 'mailForm' => $mailForm, 
					 'auditForm' => $auditForm, 
					 'notificationCenterForm' => $notificationCenterForm, 
					 'generalSettingsForm' => $generalSettingsForm, 
					 'restartStrategy' => ($manager->getOsType() == Manager::OS_TYPE_IBMI) ? false : $restartStrategy->current(), 
					 'restartStrategyValue' => $restartStrategyValue->current(),
					 'licenseType' => $licenseType);
	}
	
	
	public function generalAction() {
		$result = array('success' => true);
		
		$directivesMapper = $this->getLocator()->get('Configuration\MapperDirectives'); /* @var $directivesMapper \Configuration\MapperDirectives */
		$guiHostName = $directivesMapper->selectSpecificDirectives(array('zend_monitor.gui_host_name'));
		
		$request = $this->getRequest(); /* @var $request \Zend\Http\PhpEnvironment\Request */
		if ($request->isPost()) {
			$form = new \Application\Forms\Settings\General(array('guiHostName' => $guiHostName->current()));
			$originalForm = clone $form;
			
			$params = $this->getParameters();
			$form->setData($params);
			if ($form->isValid()){
				$parameters = $params->toArray();
				$directivesToSave = array();
				$externalDirectivesToSave = array();
				foreach ($parameters as $key => $parameter) {
					$formValue = $originalForm->get($key)->getValue();
					if ($formValue != $parameter) {
						$section = $originalForm->get($key)->getAttribute('section');
						if (empty($section)) { // no section - go to external directives
							$externalDirectivesToSave[$key] = $parameter;
						} else {
							$directivesToSave[$key] = $parameter;
						}
					}
				}
				
				if (count($directivesToSave) > 0) {
					if (isset($directivesToSave['defaultServer']) && !$directivesToSave['defaultServer']) { // was set, but empty - server to <default-server>
						$directivesToSave['defaultServer'] = '<default-server>';
					}
					try { // TODO - use special audit type rather than AUDIT_DIRECTIVES_MODIFIED
						$auditMessage = $this->auditMessage(auditMapper::AUDIT_DIRECTIVES_MODIFIED,	ProgressMapper::AUDIT_PROGRESS_REQUESTED, array($directivesToSave)); /* @var $auditMessage \Audit\Container */
						$this->getTasksMapper()->setGuiDirectives($directivesToSave, $auditMessage->getAuditId());
						$result = array('success' => true);
					} catch (\Exception $e) {
						$this->auditMessageProgress($auditMessage->getAuditId(), ProgressMapper::AUDIT_PROGRESS_ENDED_FAILED, array(array('errorMessage' => $e->getMessage())));
						Log::err("Set UI directives failed: " . $e->getMessage());
						$result = array('success' => false, 'error' => $e->getMessage());
					}
				} else {
					$result = array('success' => true);
				}
				
				if (count($externalDirectivesToSave) > 0) {
					if (isset($externalDirectivesToSave['externalUrl'])) {
						// update the directive
						try { // TODO - use special audit type rather than AUDIT_DIRECTIVES_MODIFIED
							$auditMessage = $this->auditMessage(auditMapper::AUDIT_DIRECTIVES_MODIFIED,	ProgressMapper::AUDIT_PROGRESS_REQUESTED, array($directivesToSave)); /* @var $auditMessage \Audit\Container */
							$directivesMapper->setDirectives(array('zend_monitor.gui_host_name' => $externalDirectivesToSave['externalUrl']), $auditMessage->getAuditId());
						} catch (\Exception $e) {
							$this->auditMessageProgress($auditMessage->getAuditId(), ProgressMapper::AUDIT_PROGRESS_ENDED_FAILED, array(array('errorMessage' => $e->getMessage())));
							Log::err("Set directives failed: " . $e->getMessage());
							$result = array('success' => false, 'error' => $e->getMessage());
						}
							
						// call reload configuration ZSRV-9601
						try { // TODO - use special audit type rather than AUDIT_DIRECTIVES_MODIFIED
							$serversIds = $this->getServersIds();
							$this->getTasksMapper()->insertTasksServers($serversIds, TasksMapper::COMMAND_RELOAD_CONFIGURATION, $auditMessage->getAuditId());
						} catch (\Exception $e) {
							$this->auditMessageProgress($auditMessage->getAuditId(), ProgressMapper::AUDIT_PROGRESS_ENDED_FAILED, array(array('errorMessage' => $e->getMessage())));
							Log::err("Reload configuration failed: " . $e->getMessage());
							$result = array('success' => false, 'error' => $e->getMessage());
						}
					}
					
					$result = array('success' => true);
				}
			} else {
				$error = $this->getFormErrorMessage($form);
				$result = array('success' => false, 'error' => $error);
			}			
		} else {
			$result = array('success' => false, 'error' => 'Must be post request');
		}
		
		
		$viewModel = new ViewModel();
		$viewModel->setTerminal (true);
		$viewModel->setTemplate('application/settings/settings-response');
		$viewModel->setVariable('result', $result);
		
		return $viewModel;
	}
	
	public function mailAction() {
		$result = array('success' => true);
	
		$request = $this->getRequest(); /* @var $request \Zend\Http\PhpEnvironment\Request */
		if ($request->isPost()) {
			$form = new \Application\Forms\Settings\Mail();
			$originalForm = clone $form;
				
			$params = $this->getParameters();
			$form->setData($params);
			if ($form->isValid()){
				$parameters = $params->toArray();
				$directivesToSave = array();
				foreach ($parameters as $key => $parameter) {
					$formValue = $originalForm->get($key)->getValue();
					if ($formValue != $parameter) {
						$directivesToSave[$key] = $parameter;
					}
				}
	
				$directivesToSaveAudit = $directivesToSave;
				/// scrub the mail password from the audit entry, if more entries need to be scrubbed, look into creating a generalized solution in auditMessage
				if (isset($directivesToSaveAudit['mail_password'])) {
					unset($directivesToSaveAudit['mail_password']);
				}
				
				$auditMessage = $this->auditMessage(auditMapper::AUDIT_DIRECTIVES_MODIFIED,	ProgressMapper::AUDIT_PROGRESS_REQUESTED,  array($directivesToSaveAudit)); /* @var $auditMessage \Audit\Container */
				if (count($directivesToSave) > 0) {
					try { // TODO - use special audit type rather than AUDIT_DIRECTIVES_MODIFIED
						$this->getTasksMapper()->setGuiDirectives($directivesToSave, $auditMessage->getAuditId());
						$result = array('success' => true);
					} catch (\Exception $e) {
						$this->auditMessageProgress($auditMessage->getAuditId(), ProgressMapper::AUDIT_PROGRESS_ENDED_FAILED, array(array('errorMessage' => $e->getMessage())));
						Log::err("Set UI directives failed: " . $e->getMessage());
						$result = array('success' => false, 'error' => $e->getMessage());
					}
				} else {
					$result = array('success' => true);
					$this->auditMessageProgress($auditMessage->getAuditId(), ProgressMapper::AUDIT_PROGRESS_ENDED_SUCCESFULLY);
				}
			} else {
				$error = $this->getFormErrorMessage($form);
				$result = array('success' => false, 'error' => $error);
			}
		} else {
			$result = array('success' => false, 'error' => 'Must be post request');
		}
	
		$viewModel = new ViewModel();
		$viewModel->setTerminal (true);
		$viewModel->setTemplate('application/settings/settings-response');
		$viewModel->setVariable('result', $result);
	
		return $viewModel;
	}
	
	public function notificationCenterAction() {
		$result = array('success' => true);
		
		$request = $this->getRequest(); /* @var $request \Zend\Http\PhpEnvironment\Request */
		if ($request->isPost()) {
			$notificationAction = $this->getNotificationsActionMapper()->getNotification('restartRequired'); // get email of the restartRequired type as representative
			$notificationAction = $notificationAction[0];
			
			$form = new \Application\Forms\Settings\NotificationCenter(array('notificationAction' => $notificationAction));
			$originalForm = clone $form;
				
			$params = $this->getParameters();
			$form->setData($params);
			if ($form->isValid()){
				$parameters = $params->toArray();
				$directivesToSave = array();				
				foreach ($parameters as $key => $parameter) {
					$formValue = $originalForm->get($key)->getValue();
					if ($formValue != $parameter) {
						$directivesToSave[$key] = $parameter;
					}
				}
				
				if (count($directivesToSave) > 0) {
					$auditMessage = $this->auditMessage(auditMapper::AUDIT_DIRECTIVES_MODIFIED,	ProgressMapper::AUDIT_PROGRESS_REQUESTED,  array($directivesToSave)); /* @var $auditMessage \Audit\Container */
					if (isset($directivesToSave['notificationsEmail'])) {
						$this->getNotificationsActionMapper()->updateTypesEmail($directivesToSave['notificationsEmail']);
					}
					if (isset($directivesToSave['notificationsCustomAction'])) {
						$this->getNotificationsActionMapper()->updateTypesCustomAction($directivesToSave['notificationsCustomAction']);
					}
				}
				
				$result = array('success' => true);
				$this->auditMessageProgress($auditMessage->getAuditId(), ProgressMapper::AUDIT_PROGRESS_ENDED_SUCCESFULLY, array($directivesToSave));
			} else {
				$error = $this->getFormErrorMessage($form);
				$result = array('success' => false, 'error' => $error);
			}
		} else {
			$result = array('success' => false, 'error' => 'Must be post request');
		}
	
		$viewModel = new ViewModel();
		$viewModel->setTerminal (true);
		$viewModel->setTemplate('application/settings/settings-response');
		$viewModel->setVariable('result', $result);
		
		return $viewModel;
	}
	
	public function auditAction() {
		$result = array('success' => true);
	
		$request = $this->getRequest(); /* @var $request \Zend\Http\PhpEnvironment\Request */
		if ($request->isPost()) {
			$auditEmail = $this->getAuditSettingsMapper()->getEmail();
    		$auditScriptUrl = $this->getAuditSettingsMapper()->getScriptUrl();
    		$form = new \Application\Forms\Settings\Audit(array('auditEmail' => $auditEmail, 'auditScriptUrl' => $auditScriptUrl));
    	
			$originalForm = clone $form;
	
			$params = $this->getParameters();
			$form->setData($params);
			if ($form->isValid()){
				$parameters = $params->toArray();
				$directivesToSave = array();
				foreach ($parameters as $key => $parameter) {
					$formValue = $originalForm->get($key)->getValue();
					if ($formValue != $parameter) {
						$directivesToSave[$key] = $parameter;
					}
				}
	
				if (count($directivesToSave) > 0) {
					$auditMessage = $this->auditMessage(auditMapper::AUDIT_DIRECTIVES_MODIFIED,	ProgressMapper::AUDIT_PROGRESS_REQUESTED,  array($directivesToSave)); /* @var $auditMessage \Audit\Container */
					
					if (isset($directivesToSave['auditEmail'])) {
						$this->getAuditSettingsMapper()->setEmail($directivesToSave['auditEmail']);
					}
					if (isset($directivesToSave['auditCustomAction'])) {
						$this->getAuditSettingsMapper()->setURL($directivesToSave['auditCustomAction']);
					}
				}
	
				$result = array('success' => true);
				$this->auditMessageProgress($auditMessage->getAuditId(), ProgressMapper::AUDIT_PROGRESS_ENDED_SUCCESFULLY, array($directivesToSave));
			} else {
				$error = $this->getFormErrorMessage($form);
				$result = array('success' => false, 'error' => $error);
			}
		} else {
			$result = array('success' => false, 'error' => 'Must be post request');
		}
	
		$viewModel = new ViewModel();
		$viewModel->setTerminal (true);
		$viewModel->setTemplate('application/settings/settings-response');
		$viewModel->setVariable('result', $result);
	
		return $viewModel;
	}
	
	/**
	 * @param Form\Form $form
	 * @return string
	 */
	private function getFormErrorMessage($form) {
		$errors = array();
		foreach ($form->getMessages() as $key => $errorMessages) {
			if (!$errorMessages) continue;
			foreach ($errorMessages as $errorMessage) {
				return $form->get($key)->getLabel() . ': ' . $errorMessage;
			}
		}
	
		return '';
	}
	
	private function setDisabledByEditionMessage(\Zend\Form\Form $form) {
		// check if all elements in the form are disabled
		$allDisabled = true;
		foreach ($form->getElements() as $element) { /* @var $element Element */
			if ($element->getAttribute('type') != 'submit' && ! $element->hasAttribute('disabled')) {
				$allDisabled = false;
			}
		}
		 
		$licenseType = $this->getLocator()->get('Configuration\License\ZemUtilsWrapper')->getLicenseType();
		$licenseType = ucwords(strtolower($licenseType));
		 
		// all form elements are disabled - show general message
		if ($allDisabled) {
			// add global message
			$description = '';
			if ($form->hasAttribute('description')) {
				$description = $form->getAttribute('description');
			}
			$description .= '<div class="restart-server-note global-restart-server-note">Not available in this edition</div>';
			$form->setAttribute('description', $description);
	
			// remove the submit button
			foreach ($form->getElements() as $element) { /* @var $element Element */
				if ($element->getAttribute('type') == 'submit') {
					$form->remove($element->getName());
				}
			}
		} else { // not all form element disabled - show specific element message
			foreach ($form->getElements() as $element) { /* @var $element Element */
				if ($element->hasAttribute('disabled')) {
					$element->setAttribute('description', '<div class="restart-server-note">Not available in this edition</div>');
				}
			}
		}
		 
		return $form;
	}
	
	/**
	 * @return array
	 */
	private function getServersIds() {	
		$servers = $this->getServersMapper()->findAllServers();
		return array_map(function($server) {return $server['NODE_ID'];}, $servers->toArray());
	}
}