<?php
namespace Application;

use \Zend\Session\SessionManager,
	\ZendServer\Exception;

class SessionStorage {
	
	/**
	 * @var string
	 */
	const NS = 'ZEND_SERVER_SETTINGS';
	
	/**
	 * @var SessionManager
	 */
	private $manager = null;
	
	public function __construct() {	
		$this->manager = new SessionManager();
		$this->manager->start();
	}
	
	/**
	 * @param string $tz
	 */
	public function setTimezone($tz) {
		$this->setValue('timezone', $tz);
	}
	
	/**
	 * @return string 
	 */
	public function getTimezone() {
		return $this->getValue('timezone');
	}
	
	/**
	 * @return bool
	 */
	public function hasTimezone() {
		return $this->hasValue('timezone');
	}
	
	public function isExists() {
		return ($this->manager->getStorage()->offsetExists(self::NS) &&	! is_null($this->getStorage()));
	}
	
	public function clear() {
		$this->manager->getStorage()->clear(self::NS);
	}
	
	/**
	 * @param string $key
	 * @param string $value
	 */
	private function setValue($key, $value) {
		$data = $this->getStorage();
		$data[$key] = $value;
		$this->setData($data);
	}
	
	/**
	 * @param string $key
	 * @return string
	 * @throws Exception
	 */
	private function getValue($key) {
		$data = $this->getStorage();
		
		if (! isset($data[$key])) {
			throw new Exception('Could not retrieve value from key ' . $key);
// 					Zwas_Translate::_('Could not retrieve value from key %s'),
// 					array((string)$key)
// 			));
		}
	
		return $data[$key];
	}
	
	/**
	 * @param string $key
	 * @return string
	 * @throws Zwas_Exception
	 */
	private function hasValue($key) {
		$data = $this->getStorage();
		
		return isset($data[$key]);
	}
	
	/**
	 * @param array $data
	 */
	private function setData($data) {
		$namespace = self::NS;
		$this->manager->getStorage()->$namespace = $data;
	}
	
	/**
	 * @return array
	 */
	public function getStorage() {
		$namespace = self::NS;
		$data = isset($this->manager->getStorage()->$namespace) ? $this->manager->getStorage()->$namespace : null;
		
		if (! is_array($data)) {
			return array();
		}
		
		return $data;
	}
}