<?php
namespace Bootstrap\Controller;

use WebAPI\Exception;

use Users\Identity;

use Audit\Db\ProgressMapper;

use Audit\Db\Mapper;

use Application\Module;

use ZendServer\Mvc\Controller\WebAPIActionController,
	ZendServer\Set,
	Zend\Mvc\Controller\ActionController,
	ZendServer\Log\Log,
	\Audit\Container,
	WebAPI;

use Zend\View\Model\ViewModel;

class WebAPIController extends WebAPIActionController {
	
	public function bootstrapSingleServerAction() {
		try {
				
			$this->isMethodPost();
			
			if (Module::isBootstrapCompleted()) {
				throw new WebAPI\Exception('Zend Server bootstrap was completed already', WebAPI\Exception::MALFORMED_REQUEST);
			}
			
			$params = $this->getParameters(array(
					'production' 		=> 'TRUE',
					'applicationUrl' 	=> '',
					'adminEmail' 		=> '',
					'developerPassword' => '',
					'orderNumber' 		=> '',
					'licenseKey' 		=> '',
			));
				
			$this->validateMandatoryParameters($params, array(
					'adminPassword',
			));
			
			$adminPassword = $this->validateStringNonEmpty($params['adminPassword'], 'adminPassword');
			if ($params['adminEmail']) {
				$this->validateEmailAddress($params['adminEmail'], 'adminEmail');
			}
			
			if ($params['applicationUrl']) {
				$this->validateStringNonEmpty($params['applicationUrl'], 'applicationUrl');
			}
			
			if ($params['developerPassword']) {
				$this->validateStringNonEmpty($params['developerPassword'], 'developerPassword');
			}
			$orderNumber = $params['orderNumber'];
			$licenseKey = $params['licenseKey'];
			
			$production = $this->validateBoolean($params['production'], 'production') ? 'production' : 'development';
			
		} catch (WebAPI\Exception $ex) {
			$this->handleException($ex, 'Input validation failed');
		}
		
		$this->auditMessage()->setIdentity(new Identity(_t('Unknown')));
		$audit = $this->auditMessage(Mapper::AUDIT_GUI_BOOTSTRAP_SAVELICENSE);
		try {
			$bootstrapModel = $this->getLocator('Bootstrap\Mapper'); /* @var $bootstrapModel \Bootstrap\Mapper */
			$bootstrapModel->setAdminPassword($adminPassword);
			$bootstrapModel->setApplicationUrl($params['applicationUrl']);
			$bootstrapModel->setAdminEmail($params['adminEmail']);
			$bootstrapModel->setDeveloperPassword($params['developerPassword']);
			$bootstrapModel->setLicenseKey($licenseKey);
			$bootstrapModel->setLicenseUser($orderNumber);
			$bootstrapModel->setProduction(Module::isCluster() ? 'cluster' : $params['mode']);
			 
			$bootstrapResult = $bootstrapModel->bootstrapSingleServer();
			$this->getTasksMapper()->waitForTasksComplete(); // we would like ensure that a flag such as bootstrap['zend_gui_completed] will be surely captured  
			$this->createConfigurationSnapshot();
		} catch (\Exception $e) {
			$this->auditMessageProgress($audit->getAuditId(), ProgressMapper::AUDIT_PROGRESS_ENDED_FAILED);
			Log::err("bootstrap failed with the following error: " . $e->getMessage());
			Log::debug($e);
			throw new Exception($e->getMessage(), Exception::INTERNAL_SERVER_ERROR, $e);
		}
		$this->auditMessageProgress($audit->getAuditId(), ProgressMapper::AUDIT_PROGRESS_ENDED_SUCCESFULLY);
		$this->layout('layout/layout');
		$viewModel = new ViewModel();
		$viewModel->setTemplate('bootstrap/web-api/1x3/bootstrap-single-server');
		$viewModel->setVariable('newKey', $bootstrapResult['key']);
		return $viewModel;
	}
}
