<?php
namespace Notifications\Controller;

use Notifications\Db\NotificationsMapper,
	Notifications\NotificationContainer;

use Application\Module;

use Zend\Stdlib\Parameters;

use ZendServer\Mvc\Controller\WebAPIActionController,
	ZendServer\Set,
	Zend\Mvc\Controller\ActionController,
	ZendServer\Log\Log,
	WebAPI;

use Servers\View\Helper\ServerStatus;
use Notifications\NotificationActionContainer;

class WebApiController extends WebAPIActionController {
	
	public function getNotificationsAction() {
		$this->isMethodGet();
		
		$params = $this->getParameters(array(
			'hash' => '',
		));
		
		$this->validateString($params['hash'], 'hash');
		
		$this->checkZsdHealth();
		
		$mapper = $this->getNotificationsMapper();
		
	    $serversMapper = $this->getServersMapper();
        
	    // add restarting notification in case some servers in restarting state
	    $restartingServers = $serversMapper->findRestartingServers();
        if ((is_array($restartingServers) && count($restartingServers) > 0)
        	|| (($restartingServers instanceof Set) && ($restartingServers->count() > 0))) {
        	$mapper->insertNotification(NotificationContainer::TYPE_SERVER_RESTARTING);
        } else {
        	$mapper->deleteByType(NotificationContainer::TYPE_SERVER_RESTARTING);
        }
        
        // add restart required notification in case some servers in restart required state
        $restartRequiredServers = $serversMapper->findRestartRequiredServers();
        if ((is_array($restartRequiredServers) && count($restartRequiredServers) > 0)
        	|| (($restartRequiredServers instanceof Set) && ($restartRequiredServers->count() > 0))) {
        	$mapper->insertNotification(NotificationContainer::TYPE_RESTART_REQUIRED);
        } else {
        	$mapper->deleteByType(NotificationContainer::TYPE_RESTART_REQUIRED);
        }
        
		$notifications = $mapper->findAllNotificationsWithNames();
		
		$notificationsSet = new Set($notifications->toArray());
		$notificationsSet->setHydrateClass('\Notifications\NotificationContainer');
		
		// remove restarting notification in case no restart required is present
		$notificationTypes = array();
		foreach ($notificationsSet as $key => $notification) { /* @var $notification NotificationContainer */
			$notificationTypes[$notification->getType()] = $key;
		}
		
		// Remove expiration notices if license is not going to expire
		/// this may happen if a trial license was replaced "under the hood" with a new unexpired license
		if (isset($notificationTypes[NotificationContainer::TYPE_LICENSE_ABOUT_TO_EXPIRE_45])
		 || isset($notificationTypes[NotificationContainer::TYPE_LICENSE_ABOUT_TO_EXPIRE_15])
		 	|| isset($notificationTypes[NotificationContainer::TYPE_LICENSE_ABOUT_TO_EXPIRE])) {
			
			$licenseExpiryDays = $this->getZemUtilsWrapper()->getLicenseExpirationDaysNum(true);
			$licenseNeverExpires = $this->getZemUtilsWrapper()->getLicenseInfo()->isNeverExpires();
			if ($licenseExpiryDays > 60 || $licenseNeverExpires) {
				if (isset($notificationTypes[NotificationContainer::TYPE_LICENSE_ABOUT_TO_EXPIRE_45])) {
					unset($notificationsSet[$notificationTypes[NotificationContainer::TYPE_LICENSE_ABOUT_TO_EXPIRE_45]]);
					$mapper->deleteByType(NotificationContainer::TYPE_LICENSE_ABOUT_TO_EXPIRE_45);
				}
				if (isset($notificationTypes[NotificationContainer::TYPE_LICENSE_ABOUT_TO_EXPIRE_15])) {
					unset($notificationsSet[$notificationTypes[NotificationContainer::TYPE_LICENSE_ABOUT_TO_EXPIRE_15]]);
					$mapper->deleteByType(NotificationContainer::TYPE_LICENSE_ABOUT_TO_EXPIRE_15);
				}
				if (isset($notificationTypes[NotificationContainer::TYPE_LICENSE_ABOUT_TO_EXPIRE])) {
					unset($notificationsSet[$notificationTypes[NotificationContainer::TYPE_LICENSE_ABOUT_TO_EXPIRE]]);
					$mapper->deleteByType(NotificationContainer::TYPE_LICENSE_ABOUT_TO_EXPIRE);
				}
			}
		}
		// Remove 45 days expiry if there's either a 15/7 days notification
		if (isset($notificationTypes[NotificationContainer::TYPE_LICENSE_ABOUT_TO_EXPIRE_45]) && (isset($notificationTypes[NotificationContainer::TYPE_LICENSE_ABOUT_TO_EXPIRE_15]) || isset($notificationTypes[NotificationContainer::TYPE_LICENSE_ABOUT_TO_EXPIRE]))) {
			unset($notificationsSet[$notificationTypes[NotificationContainer::TYPE_LICENSE_ABOUT_TO_EXPIRE_45]]);
			$mapper->deleteByType(NotificationContainer::TYPE_LICENSE_ABOUT_TO_EXPIRE_45);
		}	
		// Remove 15 days expiry if there's 7 days notification
		if (isset($notificationTypes[NotificationContainer::TYPE_LICENSE_ABOUT_TO_EXPIRE_15]) && isset($notificationTypes[NotificationContainer::TYPE_LICENSE_ABOUT_TO_EXPIRE])) {
			unset($notificationsSet[$notificationTypes[NotificationContainer::TYPE_LICENSE_ABOUT_TO_EXPIRE_15]]);
			$mapper->deleteByType(NotificationContainer::TYPE_LICENSE_ABOUT_TO_EXPIRE_15);
		}
		if (isset($notificationTypes[NotificationContainer::TYPE_DATABASE_CONNECTION_RESTORED])) {
			$notificationsArray = $notificationsSet->toArray();
			foreach ($notificationsArray as $key => $value) {
				if ($value['TYPE'] == NotificationContainer::TYPE_DATABASE_CONNECTION_RESTORED) {
					$serversMapper = $this->getLocator()->get('Servers\Db\Mapper'); /* @var $serversMapper \Servers\Db\Mapper */
					$serversSet = $serversMapper->findServerById($value['NODE_ID']);
					
					$notificationsArray[$key]['EXTRA_DATA'] = json_encode(array($serversSet->getNodeName()));
				}
			}
			$notificationsSet = new Set($notificationsArray);
			$notificationsSet->setHydrateClass('\Notifications\NotificationContainer'); 
		}
				
		$hash = md5(serialize($notificationsSet));
		if ($hash == $params['hash']) {
			$notificationsSet = array();
		}
		
		return array('notifications' => $notificationsSet, 'hash' => $hash);
	}
	
	private function checkZsdHealth() {
		$zsdHealthChecker = $this->getLocator()->get('Zsd\ZsdHealthChecker');
		$notificationContainer = $this->getNotificationsMapper()->getNotificationByType(NotificationContainer::TYPE_ZSD_OFFLINE)->current();
		$zsdIsDown = (is_object($notificationContainer) && $notificationContainer->getId() > 0);
		
		if (($healthCheckesult = $zsdHealthChecker->checkZsdHealth($zsdIsDown)) === false) {		
			$this->getNotificationsMapper()->insertNotification(NotificationContainer::TYPE_ZSD_OFFLINE);
		} elseif ($healthCheckesult === true) {
			$this->getNotificationsMapper()->deleteByType(NotificationContainer::TYPE_ZSD_OFFLINE);
		}
	}
	
	public function deleteNotificationAction() {
		$this->isMethodPost();
		
		$params = $this->getParameters(array(
			'type' => '',
		));
		
		$this->validateMandatoryParameters($params, array('type'));
		$type = $this->validateNotificationType($params['type'], 'type');
		
				
		$mapper = $this->getNotificationsMapper();
		$mapper->deleteByType($type);
		
		return array();
	}
	
	public function updateNotificationAction() {
		$this->isMethodPost();
		
		$params = $this->getParameters(array(
				'type' => '',
				'repeat' => ''
		));
		
		$this->validateMandatoryParameters($params, array('type'));
		$this->validateString($params['type'], 'type');
		$this->validateInteger($params['repeat'], 'repeat');
		
		$mapper = $this->getNotificationsMapper();
		$type = $this->convertNotificationNameToType($params['type']);
		$repeat = $params['repeat'];
		
		if ($type == \Notifications\NotificationContainer::TYPE_MAX_SERVERS_IN_CLUSTER) { // maxServersInCluster will be delayed for maximum 1 week 
			$repeat = min(10080, $repeat); // 1 week = 60 * 24 * 7
		} elseif ($type == \Notifications\NotificationContainer::TYPE_NO_SUPPORT) { // noSupport will be delayed for maximum 1 month
			$repeat = min(40320, $repeat); // 1 month = 60 * 24 * 7 * 4
		}
		
		$mapper->insertFilter($type, $repeat);
		
		$notifications = $mapper->getNotificationByType($type);
		$notificationsSet = new Set($notifications->toArray());
		$notificationsSet->setHydrateClass('\Notifications\NotificationContainer');
		
		return array('notifications' => $notificationsSet);
	}
		
	public function sendNotificationAction() {
		$this->isMethodPost();
		
		$params = $this->getParameters(array(
			'type' => '',
			'ip' => '127.0.0.1'
		));
		
		if (empty($params['ip'])) {
			$params['ip'] = '127.0.0.1';
		}
		
		$this->validateMandatoryParameters($params, array('type', 'ip'));
		$this->validateString($params['type'], 'type');
		$this->validateString($params['ip'], 'ip');

		
		$notificationAction = $this->getNotificationsActionMapper()->getNotification($params['type']);

		if ($notificationAction->count() == 0) {
			return array();
		}
		
		$notificationAction = $notificationAction->current();
		
		$notification = new NotificationContainer(array('TYPE' => $notificationAction->getType()));
		
		$notificationEmail = $notificationAction->getEmail();
		$notificationCustomAction = $notificationAction->getCustomAction();
		
		$listView = array();
		
		$licenseError = false;
		// send email
		if (! empty($notificationEmail)) {
			if ($this->isAclAllowed('data:useEmailNotification')) {
				$templateParams = array('title' => $notification->getTitle(),
										'description' => $notification->getDescription(),
										'url' => 'http://' . $params['ip'] . ':' . Module::config('installation', 'defaultPort') . Module::config()->baseUrl . $notification->getUrl());
				
				$this->getRequest()->getPost()->set('to', $notificationEmail);
				$this->getRequest()->getPost()->set('from', 'noreply@zend.com');
				$this->getRequest()->getPost()->set('subject', 'Zend Server Notification Alert');
				$this->getRequest()->getPost()->set('templateName', 'notification');
				$this->getRequest()->getPost()->set('templateParams', $templateParams);
				
				$listView = $this->forward()->dispatch('EmailWebAPI-1_3', array('action' => 'emailSend')); /* @var $emailListView \Zend\View\Model\ViewModel */
				$listView->setTemplate('notifications/web-api/1x3/send-notification');
			} else {
				$licenseError = true;
			}
		}
		
		// call script
		if (! empty($notificationCustomAction)) {
			if ($this->isAclAllowed('data:useCustomAction')) {
				// need to replace with cUrl
				$ctx = stream_context_create(array(
						'http' => array(
								'timeout' => 1
						)
					)
				);
				
				$uri = new \Zend\Uri\Http($notificationCustomAction);
				$uriParams = $uri->getQueryAsArray();
				$uri->setQuery(array_merge($uriParams, array('type' => $params['type'])));
				
				file_get_contents($uri, 0, $ctx);
			} else {
				$licenseError = true;
			}
		}
		
		if ($licenseError) {
			throw new WebAPI\Exception(_t('Not supported by edition'), WebAPI\Exception::NOT_SUPPORTED_BY_EDITION);
		}
		
		return $listView;
	}	
	
	private function validateNotificationType($type) {
		$notificationsActionsMapper = $this->getNotificationsActionMapper();
		$notifications = $notificationsActionsMapper->findAll();
		$dictionary = array();
		foreach ($notifications as $notificationType) { /* @var $notificationType NotificationActionContainer */
			$dictionary[$notificationType->getName()] = $notificationType->getType();
		}
		Log::debug($dictionary);
		if (! isset($dictionary[$type])) {
			throw new WebAPI\Exception(_t('type parameter must be one of (%s)', array(implode(', ', array_keys($dictionary)))), WebAPI\Exception::INVALID_PARAMETER);
		}
		return $dictionary[$type];
	}
	
	/**
	 * @param string $name
	 * @return integer
	 */
	private function convertNotificationNameToType($name) {
		$notificationsActionsMapper = $this->getNotificationsActionMapper();
		$notificationAction = $notificationsActionsMapper->getNotification($name);
		if (count($notificationAction) == 0) {
			return -1;
		}
		$notificationAction = $notificationAction->current();
		return $notificationAction->getType();
	}
}