<?php

namespace Notifications\Db;

use Notifications\NotificationContainer;

use ZendServer\Log\Log,
ZendServer\Set,
Configuration\MapperAbstract,
Zend\Db\TableGateway\TableGateway,
Messages\MessageContainer,
Servers\Db\Mapper,
\Zend\Db\Sql\Select;

class NotificationsMapper extends MapperAbstract {
	
	protected $setClass = '\Notifications\NotificationContainer';
	
	/**
	 * @var \Zend\Db\TableGateway\TableGateway
	 */
	protected $notificationsActionsGateway;

	/**
	 * @return Set[MessageContainer]
	 */
	public function findAllNotifications() {
		$date = new \DateTime();
		$showAt = $date->getTimestamp();
		
		return $this->select('SHOW_AT <= "' . $showAt . '"');
	}
	
	/**
	 * @return Set[MessageContainer]
	 */
	public function findAllNotificationsWithNames() {
		$date = new \DateTime();
		$showAt = $date->getTimestamp();
		$notificationsTable = $this->getTableName();
		$notificationsActionsTable = $this->getNotificationsActionsGateway()->getTable(); 
		
		$select = new Select($notificationsTable);
		$select->join($notificationsActionsTable, "{$notificationsTable}.TYPE = {$notificationsActionsTable}.TYPE", array('NAME'), Select::JOIN_LEFT);
		$select->where('SHOW_AT <= "' . $showAt . '"');
		$select->order('ID desc');
		return $this->getTableGateway()->selectWith($select);
	}
	
	/**
	 * @return Set[MessageContainer]
	 */
	public function getNotificationByType($type) {
		return $this->select('type = "' . $type . '"');
	}
	
	/**
	 * @param integer $type
	 */
	public function deleteByType($type) {
		return $this->delete('TYPE = "' . $type . '"');
	}

	/**
	 * @param array $types
	 */
	public function deleteByTypes(array $types) {
		$where = $this->getSqlInStatement('TYPE', $types);
		return $this->delete($where);
	}
		
	/**
	 * @param integer $type
	 */
	public function insertNotification($type, $extraData = null) {
		$notifications = $this->getNotificationByType($type);
		if (count($notifications) > 0) { // insert only if not exists
			return;
		}
		
		$date = new \DateTime();
		$showAt = $date->getTimestamp();
		
		$data = array('SEVERITY' => 2, 'CREATION_TIME' => $showAt, 'TYPE' => $type, 'REPEATS' => 0, 'SHOW_AT' => $showAt);
		if (! is_null($extraData)) {
			$data['EXTRA_DATA'] = json_encode($extraData);
		}
		$row = $this->getTableGateway()->insert($data);
	}
	
	/**
	 * @param integer $type
	 * @param integer $minutesDelta
	 * @return number
	 */
	public function insertFilter($type, $minutesDelta) {
		$date = new \DateTime();
		$date->modify('+' . $minutesDelta . ' minutes');
		$showAt = $date->getTimestamp();
	
		return $this->update(array('SHOW_AT' => $showAt), array('TYPE' => $type));
	}
	
	/**
	 * @return \Zend\Db\TableGateway\TableGateway
	 */
	public function getNotificationsActionsGateway() {
		return $this->notificationsActionsGateway;
	}
	
	/**
	 * @param \Zend\Db\TableGateway\TableGateway $notificationsActionsGateway
	 */
	public function setNotificationsActionsGateway($notificationsActionsGateway) {
		$this->notificationsActionsGateway = $notificationsActionsGateway;
	}
}
