<?php

namespace Notifications;

class NotificationContainer {
	// SEVERITY
	const SEVERITY_INFO     = 0;
	const SEVERITY_WARNING	= 1;
	const SEVERITY_ERROR	= 2;
	
	// TYPE
	const TYPE_UNKNOWN 							= -1;
	const TYPE_SERVER_OFFLINE 					= 0;
	const TYPE_RESTART_REQUIRED 				= 1;
	const TYPE_DAEMON_OFFLINE 					= 2;
	const TYPE_PHP_EXT_DIRECTIVE_MISSMATCH 		= 3;
	const TYPE_PHP_EXT_NOT_LOADED 				= 4;
	const TYPE_PHP_EXT_NOT_INSTALLED 			= 5;
	const TYPE_ZEND_EXT_DIRECTIVE_MISSMATCH 	= 6;
	const TYPE_ZEND_EXT_NOT_LOADED 				= 7;
	const TYPE_ZEND_EXT_NOT_INSTALLED 			= 8;
	const TYPE_DEPLOYMENT_FAILURE 				= 9;
	const TYPE_DEPLOYMENT_UPDATE_FAILURE 		= 10;
	const TYPE_DEPLOYMENT_REDEPLOY_FAILURE 		= 11;
	const TYPE_DEPLOYMENT_HEALTHCHECK_FAILURE	= 12;
	const TYPE_DEPLOYMENT_REMOVE_FAILURE 		= 13;
	const TYPE_DEPLOYMENT_ROLLBACK_FAILURE 		= 14;
	const TYPE_DEPLOYMENT_DEFINE_APP_FAILURE 	= 15;
	const TYPE_SERVER_ADD_ERROR 				= 16;
	const TYPE_SERVER_REMOVE_ERROR 				= 17;
	const TYPE_SERVER_ENABLE_ERROR 				= 18;
	const TYPE_SERVER_DISABLE_ERROR 			= 19;
	const TYPE_SERVER_FORCE_REMOVE_ERROR 		= 20;
	const TYPE_JOBQUEUE_HIGH_CONCURRENCY 		= 21;
	const TYPE_SERVER_RESTARTING				= 22;
	const TYPE_SC_SESSION_HANDLER_FILES			= 23;
	const TYPE_SC_NO_BACKUP						= 24;
	const TYPE_LICENSE_INVALID					= 26;
	const TYPE_LICENSE_ABOUT_TO_EXPIRE			= 27;
	const TYPE_WEBSERVER_NOT_RESPONDING			= 28;
	const TYPE_MAX_SERVERS_IN_CLUSTER			= 29;
	const TYPE_LICENSE_ABOUT_TO_EXPIRE_45		= 30;
	const TYPE_LICENSE_ABOUT_TO_EXPIRE_15		= 31;
	const TYPE_DATABASE_CONNECTION_RESTORED		= 32;
	const TYPE_SCD_STDBY_MODE					= 33;
	const TYPE_ZSD_OFFLINE						= 34;
	const TYPE_SCD_ERROR						= 35;
	const TYPE_MAIL_SETTINGS_NOT_SET			= 36;
	const TYPE_SCD_SHUTDOWN_ERROR				= 37;
	const TYPE_NO_SUPPORT						= 38;
	
	
	//// internal types
	const TYPE_LICENSE_EXPIRES_TODAY			= -10;
	const TYPE_LICENSE_EXPIRES_TOMORROW			= -11;
	
	const TYPE_LIBRARY_UPDATE_AVAILABLE			= 500;
	
	private $titles = array(
			self::TYPE_SERVER_OFFLINE => 'Offline server',
			self::TYPE_RESTART_REQUIRED => 'Restart required',
			self::TYPE_DAEMON_OFFLINE => 'Offline daemon',
			self::TYPE_PHP_EXT_DIRECTIVE_MISSMATCH => 'Mismatched extension directive',
			self::TYPE_PHP_EXT_NOT_LOADED => 'PHP extension not loaded',
			self::TYPE_PHP_EXT_NOT_INSTALLED => 'PHP extension not installed',
			self::TYPE_ZEND_EXT_DIRECTIVE_MISSMATCH => 'Mismatched component directive',
			self::TYPE_ZEND_EXT_NOT_LOADED => 'Component not loaded',
			self::TYPE_ZEND_EXT_NOT_INSTALLED => 'Component not installed',
			self::TYPE_DEPLOYMENT_FAILURE => 'Failed to deploy',
            self::TYPE_DEPLOYMENT_UPDATE_FAILURE => 'Failed to update',
			self::TYPE_DEPLOYMENT_REDEPLOY_FAILURE => 'Failed to redeploy',
            self::TYPE_DEPLOYMENT_HEALTHCHECK_FAILURE => 'Failed to health check',
			self::TYPE_DEPLOYMENT_REMOVE_FAILURE => 'Failed to remove',
			self::TYPE_DEPLOYMENT_ROLLBACK_FAILURE => 'Failed to rollback',
			self::TYPE_DEPLOYMENT_DEFINE_APP_FAILURE => 'Failed to define',
			self::TYPE_SERVER_ADD_ERROR => 'Failed to add server',
			self::TYPE_SERVER_REMOVE_ERROR => 'Failed to remove server',
			self::TYPE_SERVER_ENABLE_ERROR => 'Failed to enable server',
			self::TYPE_SERVER_DISABLE_ERROR => 'Failed to disable server',
			self::TYPE_SERVER_FORCE_REMOVE_ERROR => 'Failed to force remove server',
			self::TYPE_JOBQUEUE_HIGH_CONCURRENCY => 'Job Queue reached a high concurrency level',
			self::TYPE_SC_SESSION_HANDLER_FILES => 'Session Clustering is disabled',
			self::TYPE_SC_NO_BACKUP => 'Session Clustering failed to locate backup servers',
			self::TYPE_LICENSE_INVALID => 'Invalid license',
			self::TYPE_LICENSE_ABOUT_TO_EXPIRE => 'License is about to expire',
			self::TYPE_WEBSERVER_NOT_RESPONDING => 'Webserver is not responding',
			self::TYPE_MAX_SERVERS_IN_CLUSTER => 'You have reached the maximum amount of allowed servers for this license',
			self::TYPE_LICENSE_ABOUT_TO_EXPIRE_45 => 'License is about to expire',
			self::TYPE_LICENSE_ABOUT_TO_EXPIRE_15 => 'License is about to expire',
			self::TYPE_LICENSE_EXPIRES_TODAY => 'License is about to expire',
			self::TYPE_LICENSE_EXPIRES_TOMORROW => 'License is about to expire',
			self::TYPE_DATABASE_CONNECTION_RESTORED => 'Database Connection Restored',
			self::TYPE_SCD_STDBY_MODE => 'Sessions may not be tracked',
			self::TYPE_ZSD_OFFLINE => 'Zend server daemon offline',
			self::TYPE_MAIL_SETTINGS_NOT_SET => 'Mail server settings are needed',
			self::TYPE_NO_SUPPORT => 'Your license does not include support',
			self::TYPE_LIBRARY_UPDATE_AVAILABLE => 'Library update available',
			self::TYPE_SCD_SHUTDOWN_ERROR => 'Graceful Shutdown failed',
	);

	private $descriptions = array(
			self::TYPE_SERVER_OFFLINE => 'The server is offline.',
			self::TYPE_RESTART_REQUIRED => 'To apply changes, you must perform a restart.',
			self::TYPE_DAEMON_OFFLINE => 'The daemon is offline.',
			self::TYPE_PHP_EXT_DIRECTIVE_MISSMATCH => 'The values for this extension directive contradict corresponding values in the cluster.',
			self::TYPE_PHP_EXT_NOT_LOADED => 'To configure the directives for this PHP extension and begin working with its functionality, the extension needs to be enabled.',
			self::TYPE_PHP_EXT_NOT_INSTALLED => 'To configure the directives for this PHP extension and begin working with its functionality, the extension needs to be installed and enabled.',
			self::TYPE_ZEND_EXT_DIRECTIVE_MISSMATCH => 'The values for this component directive contradict corresponding values in the cluster.',
			self::TYPE_ZEND_EXT_NOT_LOADED => 'To configure the directives for this component and begin working with its functionality, the component needs to be enabled.',
			self::TYPE_ZEND_EXT_NOT_INSTALLED => 'To configure the directives for this component and begin working with its functionality, the component needs to be installed and enabled.',
			self::TYPE_DEPLOYMENT_FAILURE => 'The application package you attempted to deploy could not be deployed.',
			self::TYPE_DEPLOYMENT_UPDATE_FAILURE => 'The application you attempted to update could not be updated.',
			self::TYPE_DEPLOYMENT_REDEPLOY_FAILURE => 'The application you attempted to redeploy could not be redeployed.',
			self::TYPE_DEPLOYMENT_HEALTHCHECK_FAILURE => 'The application you attempted to health check could not be health checked.',
			self::TYPE_DEPLOYMENT_REMOVE_FAILURE => 'The application you attempted to remove could not be removed.',
			self::TYPE_DEPLOYMENT_ROLLBACK_FAILURE => 'The deployment you attempted to rollback failed to roll back.',
			self::TYPE_DEPLOYMENT_DEFINE_APP_FAILURE => 'Your new application could not be defined.',
			self::TYPE_SERVER_ADD_ERROR => 'The server you attempted to add to the cluster could not be added.',
			self::TYPE_SERVER_REMOVE_ERROR => 'The server you attempted to remove from the cluster could not be removed',
			self::TYPE_SERVER_ENABLE_ERROR => 'The server you attempted to enable could not be enabled.',
			self::TYPE_SERVER_DISABLE_ERROR => 'The server you attempted to enable could not be enabled.',
			self::TYPE_SERVER_FORCE_REMOVE_ERROR => 'The server you attempted to force remove could not be removed',
			self::TYPE_JOBQUEUE_HIGH_CONCURRENCY => 'The Job Queue daemon has reached a high concurrency level',
			self::TYPE_SC_SESSION_HANDLER_FILES => 'Session Clustering is disabled since the \'session.save_handler\' directive is not set to \'cluster\'',
			self::TYPE_SC_NO_BACKUP => 'Session Clustering failed to locate a backup server for sessions in server/s %s',
			self::TYPE_LICENSE_INVALID => 'The Zend Server license you are currently using has expired or is invalid',
			self::TYPE_LICENSE_ABOUT_TO_EXPIRE => 'The Zend Server license you are currently using is about to expire in %s day(s)',
			self::TYPE_WEBSERVER_NOT_RESPONDING => 'The webserver is not responding',
			self::TYPE_MAX_SERVERS_IN_CLUSTER => 'The number of servers in the cluster has reached the limit defined in the license',
			self::TYPE_LICENSE_ABOUT_TO_EXPIRE_45 => 'The Zend Server license you are currently using is about to expire in %s day(s)',
			self::TYPE_LICENSE_ABOUT_TO_EXPIRE_15 => 'The Zend Server license you are currently using is about to expire in %s day(s)',
			self::TYPE_LICENSE_EXPIRES_TODAY => 'The Zend Server license you are currently using will expire today',
			self::TYPE_LICENSE_EXPIRES_TOMORROW => 'The Zend Server license you are currently using will expire tomorrow',
			self::TYPE_DATABASE_CONNECTION_RESTORED => 'Zend Server has restored the database connection for server \'%s\'',
			self::TYPE_SCD_STDBY_MODE => 'The \'session.save_handler\' directive is set to \'cluster\', but the Session Clustering daemon is inactive. PHP might not be able to track user sessions.',
			self::TYPE_ZSD_OFFLINE => 'The zend server daemon is offline.',
			self::TYPE_MAIL_SETTINGS_NOT_SET => 'Zend Server mail settings have not been defined yet. To receive email notifications from Zend Server, configure your email server on the Settings page.', 
			self::TYPE_NO_SUPPORT => 'The Zend Server license you are currently using does not include support',
			self::TYPE_LIBRARY_UPDATE_AVAILABLE => 'There is a new library update available and ready for download',
			self::TYPE_SCD_SHUTDOWN_ERROR => 'The Session Clustering daemon (SCD) has failed to perform a graceful shutdown. Restart it to restore session data',
	);
	
	private $singleDescriptions = array(
			self::TYPE_PHP_EXT_DIRECTIVE_MISSMATCH => 'The values for this extension directive have been changed, and contradict corresponding values in the database.',
			self::TYPE_ZEND_EXT_DIRECTIVE_MISSMATCH => 'The values for this component directive have been changed, and contradict corresponding values in the database.',
			self::TYPE_SERVER_ADD_ERROR => 'The server you attempted to add to the cluster could not be added.',
	);

	/**
	 * @var array
	 */
	protected $notification;
	
	/**
	 * @param array $eventsGroup
	 */
	public function __construct(array $notification) {
		$this->notification = $notification;
		$this->setTitle($this->getType());
		$this->setUrl($this->getType());		
	}
	
	public function toArray() {
		return $this->notification;
	}
	
	/**
	 * @return integer
	 */
	public function getId() {
		return isset($this->notification['ID']) ? (integer) $this->notification['ID'] : 0;
	}
	
	/**
	 * @return integer
	 */
	public function getSeverity() {
		return (integer) $this->notification['SEVERITY'];
	}
	
	/**
	 * @return integer
	 */
	public function getCreationTime() {
		return (integer) $this->notification['CREATION_TIME'];
	}
	
	/**
	 * @return integer
	 */
	public function getType() {
		return isset($this->notification['TYPE']) ? (integer) $this->notification['TYPE'] : 0;
	}
	
	/**
	 * @return integer
	 */
	public function getRepeats() {
		return (integer) $this->notification['REPEATS'];
	}
	
	public function getTitle() {
		return $this->notification['TITLE'];
	}
	
	public function getDescription() {
		$type = $this->getType();
		
		if (isset($this->descriptions[$type])) {
			$extraData = $this->getExtraData();
			if ($type == self::TYPE_LICENSE_ABOUT_TO_EXPIRE || $type == self::TYPE_LICENSE_ABOUT_TO_EXPIRE_15 || $type == self::TYPE_LICENSE_ABOUT_TO_EXPIRE_45) {
				$numofDays = 0;
				
				$utilsWrapper = new \Configuration\License\ZemUtilsWrapper();
				if (! $utilsWrapper->isLicenseValid()) {
					return array('Non Valid License!', '');
				}
		
				$numofDays = $utilsWrapper->getLicenseExpirationDaysNum(true);
				$extraData = array($numofDays);
				
				if ($numofDays == 1) {
					$type = self::TYPE_LICENSE_EXPIRES_TOMORROW;
				} elseif ($numofDays == 0) {
					$type = self::TYPE_LICENSE_EXPIRES_TODAY;
				}
			}
				
			// single server with specific message
			if (\Application\Module::isSingleServer() && isset($this->singleDescriptions[$type])) {
				$this->notification['DESCRIPTION'] = sprintf(_t($this->singleDescriptions[$type], $extraData));
			} else {
				$this->notification['DESCRIPTION'] = sprintf(_t($this->descriptions[$type], $extraData));
			}
		} else {
			$this->notification['DESCRIPTION'] = '';
		}
		
		return $this->notification['DESCRIPTION'];
	}
	
	public function getUrl() {
		return $this->notification['URL'];
	}
	
	public function getName() {
		return $this->notification['NAME'];
	}
	
	public function getNodeId() {
		return $this->notification['NODE_ID'];
	}
	
	public function getExtraData() {
		if (isset($this->notification['EXTRA_DATA']) && $this->notification['EXTRA_DATA']) {
			return json_decode($this->notification['EXTRA_DATA']);
		} else {
			return array();
		}
	}
	
	private function setTitle($type) {
		if (isset($this->titles[$type])) {
			$this->notification['TITLE'] = _t($this->titles[$type]);
		} else {		
			$this->notification['TITLE'] = '';
		}
	}
	
	private function setUrl($type) {
		switch ($type) {
			case (self::TYPE_DEPLOYMENT_FAILURE):
			case (self::TYPE_DEPLOYMENT_UPDATE_FAILURE):
			case (self::TYPE_DEPLOYMENT_REDEPLOY_FAILURE):
			case (self::TYPE_DEPLOYMENT_HEALTHCHECK_FAILURE):
			case (self::TYPE_DEPLOYMENT_REMOVE_FAILURE):
			case (self::TYPE_DEPLOYMENT_ROLLBACK_FAILURE):
			case (self::TYPE_DEPLOYMENT_DEFINE_APP_FAILURE):
				$this->notification['URL'] = '/Deployment';
				break;
			case (self::TYPE_LIBRARY_UPDATE_AVAILABLE):
				$this->notification['URL'] = '/DeploymentLibrary';
				break;
			case (self::TYPE_SERVER_ADD_ERROR):
			case (self::TYPE_SERVER_REMOVE_ERROR):
			case (self::TYPE_SERVER_ENABLE_ERROR):
			case (self::TYPE_SERVER_DISABLE_ERROR):
			case (self::TYPE_SERVER_FORCE_REMOVE_ERROR):
			case (self::TYPE_SERVER_OFFLINE):
			case (self::TYPE_PHP_EXT_DIRECTIVE_MISSMATCH):
			case (self::TYPE_ZEND_EXT_DIRECTIVE_MISSMATCH):
			case (self::TYPE_WEBSERVER_NOT_RESPONDING):				
				$this->notification['URL'] = '/Servers';
				break;
			case (self::TYPE_PHP_EXT_NOT_LOADED):
			case (self::TYPE_PHP_EXT_NOT_INSTALLED):
			case (self::TYPE_SCD_STDBY_MODE):
				$this->notification['URL'] = '/Extensions/phpExtensions';
				break;
			case (self::TYPE_ZEND_EXT_NOT_LOADED):
			case (self::TYPE_ZEND_EXT_NOT_INSTALLED):
			case (self::TYPE_DAEMON_OFFLINE):
			case (self::TYPE_SCD_SHUTDOWN_ERROR):
				$this->notification['URL'] = '/ZendComponents';
				break;
			case (self::TYPE_JOBQUEUE_HIGH_CONCURRENCY):
				$this->notification['URL'] = '/ZendComponents#grid=Zend--Job--Queue';
				break;
			case (self::TYPE_SC_SESSION_HANDLER_FILES):
				$this->notification['URL'] = '/SessionClustering';
				break;				
			case (self::TYPE_MAX_SERVERS_IN_CLUSTER):
			case (self::TYPE_LICENSE_INVALID):
			case (self::TYPE_LICENSE_ABOUT_TO_EXPIRE):
			case (self::TYPE_LICENSE_ABOUT_TO_EXPIRE_45):
			case (self::TYPE_LICENSE_ABOUT_TO_EXPIRE_15):
			case (self::TYPE_NO_SUPPORT):
				$this->notification['URL'] = '/License';
				break;
			case (self::TYPE_MAIL_SETTINGS_NOT_SET):
				$this->notification['URL'] = '/Settings';
				break;
			default:
				$this->notification['URL'] = '';
				break;
		}
	}
}