<?php
namespace Servers\Db;

use Zsd\Db\TasksMapperAwareInterface;
use Zsd\Db\TasksMapper;
use ZendServer\Log\Log;
class Tasks implements TasksMapperAwareInterface {
	/**
	 * @var TasksMapper
	 */
	private $tasksMapper;
	
	/**
	 * @param string $serverName
	 * @param string $dbHost
	 * @param string $dbUsername
	 * @param string $dbPassword
	 * @param string $dbName
	 * @param integer $auditId - the auditId to be inserted
	 */
	public function serverAddToCluster($serverName, $nodeIp, $dbHost, $dbPort, $dbUsername, $dbPassword, $dbName, $auditId, $extraAuditDetails=array()) {
		Log::info("adding server '$serverName' to the cluster");
		
		$this->tasksMapper->insertTask(0, TasksMapper::COMMAND_JOIN_CLUSTER, $auditId, array(
					'node_name' => $serverName,
					'node_ip' => $nodeIp,
					'database_host' => $dbHost,
                    'database_port' => (string)$dbPort,
					'database_name' => $dbName,
					'user' => $dbUsername,
					'password' => $dbPassword
			) + $extraAuditDetails);
	}
	
	/**
	 * @param integer $serverId
	 * @param integer $auditId
	 */
	public function serverEnable($serverId, $auditId) {
		Log::info("enable server '$serverId'");
		$this->tasksMapper->insertTask($serverId, TasksMapper::COMMAND_ENABLE_SERVER, $auditId);
	}
	
	/**
	 * @param integer $serverId
	 * @param integer $auditId
	 */
	public function serverDisable($serverId, $auditId) {
		Log::info("disable server '$serverId'");
		$this->tasksMapper->insertTask($serverId, TasksMapper::COMMAND_DISABLE_SERVER, $auditId);
	}
	
	/**
	 * @param integer $serverId
	 * @param integer $auditId
	 */
	public function serverRemove($serverId, $auditId) {
		Log::info("remove server '$serverId'");
		$this->tasksMapper->insertTask($serverId, TasksMapper::COMMAND_DISCONNECT_FROM_CLUSTER, $auditId);
	}
	
	/**
	 * @param integer $serverId
	 * @param integer $auditId
	 */
	public function serverForceRemove($serverId, $auditId) {
		Log::info("remove server '$serverId'");
		$this->tasksMapper->removeServerTasks($serverId);
		$this->tasksMapper->insertTask(TasksMapper::DUMMY_NODE_ID, TasksMapper::COMMAND_FORCE_DISCONNECT_FROM_CLUSTER, $auditId, array('node_id' => $serverId));
	}
	
	/**
	 * @param array $serversIds
	 * @param integer $auditId
	 */
	public function restartPhpSelective($serversIds, $auditId) {
		Log::info('Starting selective restart');
		$this->tasksMapper->insertTasksServers($serversIds, TasksMapper::COMMAND_RESTART_SERVER, $auditId, array(TasksMapper::RESTART_TYPE_SELECTIVE));
	}
	
	/**
	 * @param array $serversIds
	 * @param integer $auditId
	 */
	public function restartPhpFull($serversIds, $auditId) {
		Log::info('Starting full restart');
		$this->tasksMapper->insertTasksServers($serversIds, TasksMapper::COMMAND_RESTART_SERVER, $auditId, array(TasksMapper::RESTART_TYPE_FULL));
	}
	
	/**
	 * @param array $serversIds
	 * @param integer $auditId
	 */
	public function restartDaemon($serversIds, $daemon, $auditId) {
		$task = $this->getTasksMapper()->getTaskDaemonName($daemon);
		$this->tasksMapper->insertTasksServers($serversIds, $task, $auditId, array(TasksMapper::RESTART_TYPE_FULL));
	}
	
	/**
	 * @return TasksMapper
	 */
	public function getTasksMapper() {
		return $this->tasksMapper;
	}
	
	/* (non-PHPdoc)
	 * @see \Zsd\Db\TasksMapperAwareInterface::setTasksMapper()
	 */
	public function setTasksMapper($tasksMapper) {
		$this->tasksMapper = $tasksMapper;
	}
	

}