<?php

namespace Zsd;
use Bootstrap\Mapper;

use Application\Db\SqliteDbCreator;

use ZendServer\FS\FS;

use Zend\Db\Adapter\Adapter;

use ZendServer\Log\Log;

use ZendServer\Ini\IniReader;
use PDO;

class DbConnector {
	
	/**
	 * @var \PDO
	 */
	private static $db;
	public static function factory($context) {
		$config = new IniReader();
		$iniFile = get_cfg_var('zend.conf_dir').DIRECTORY_SEPARATOR.'zend_database.ini';
		$dbDirectives = $config->fromFile($iniFile, false);// flat reading, important for windows
		
		if ($dbDirectives['zend.database.type'] == 'SQLITE') {
			Log::info("Create sqlite db connection to {$context}");
			if ($context == 'monitor') {
				$dsn = 'sqlite:' . get_cfg_var('zend.data_dir') . '/db/monitor.db';
			} elseif ($context == 'statistics') {
				$dsn = 'sqlite:' . get_cfg_var('zend.data_dir') . '/db/statistics.db';
			} elseif ($context == 'codetracing') {
				$dsn = 'sqlite:' . get_cfg_var('zend.data_dir') . '/db/codetracing.db';
			} elseif ($context == 'gui') {
				$dsn = 'sqlite:' . get_cfg_var('zend.data_dir') . '/db/gui.db';
			} elseif ($context == 'jobqueue') {
				$dsn = 'sqlite:' . get_cfg_var('zend.data_dir') . '/db/jobqueue.db';
			} else {
				$dsn = 'sqlite:' . get_cfg_var('zend.data_dir') . '/db/zsd.db';
			}
			
			$pdo = new PDO($dsn, '', '');
			//todo - enable excetpion error mode and catch exceptions 
			//$pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
			if ($context == 'gui') {
				if (! $pdo->query('SELECT * FROM GUI_METADATA where NAME = \'gui_schema_version\'')) {
					Log::warn('gui db looks empty, attempt to rebuild');
					$creator = new SqliteDbCreator($pdo);
					$creator->createGuiDb();
					Log::info('gui db rebuild completed');
					$bootstrap = new Mapper();
					$bootstrap->resetBootstrap();
				}
			}
			return $pdo;
		} else {
			if(is_null(static::$db)) {
				$conn = strtolower($dbDirectives['zend.database.type']). ":host={$dbDirectives['zend.database.host_name']};port={$dbDirectives['zend.database.port']}; dbname={$dbDirectives['zend.database.name']}";
				Log::info("Create MySQL connection using: '{$conn}'");
				static::$db = new PDO($conn, $dbDirectives['zend.database.user'], $dbDirectives['zend.database.password']);
			}
			return static::$db;
		}
	}
}
