<?php

namespace Zsd;

use ZendServer\Log\Log;

use ZendServer\Edition;
use ZendServer\EditionAwareInterface;


class ZsdHealthChecker implements EditionAwareInterface {
	
	const THRESHOLD_DELTA = 10; // how long in seconds we allow the zsd timestmap to be stale, before reporting that ZSD is down. ZSD updates the timestamp at least once every 1 seconds
	
	/**
	 * @var Servers\Db\Mapper
	 */
	private $serversMapper;
	
	/**
	 * @var SessionStorage
	 */
	private $sessionStorage = null;
	
	/**
	 * @var Edition
	 */
	private $edition;
	
	public function checkZsdHealth($zsdIsDown=false) {
		$edition = $this->getEdition();
		if (! $edition->isSingleServer()) {
			return true;
		}

		$currentZsdTimeStamp = $this->getServersMapper()->getZsdLastUpdated($edition->getServerId());
		if (is_null($currentZsdTimeStamp)) {
			Log::warn("ZSD HealthCheck - ZSD seems to be down");
			return false;
		}
		
		$currentPhpTimeStamp = time();
		
		$previousZsdTimeStamp = $this->getSessionStorage()->getZsdTimeStamp();
		$previousPhpTimeStamp = $this->getSessionStorage()->getPhpTimeStamp();
		
		if (! $previousZsdTimeStamp) {
			log::info("ZSD HealthCheck - timestamps are missing - newly created system?");
			$this->getSessionStorage()->setZsdTimeStamp($currentZsdTimeStamp);
			$this->getSessionStorage()->setPhpTimeStamp($currentPhpTimeStamp);
			return;
		}
		
		if ($zsdIsDown === false && ($currentPhpTimeStamp - $previousPhpTimeStamp) <= self::THRESHOLD_DELTA) {
			Log::debug("ZSD HealthCheck - will not check ZSD timestamps as within the threshold");
			return;
		}
		
		$this->getSessionStorage()->setPhpTimeStamp($currentPhpTimeStamp);
		if ($currentZsdTimeStamp === $previousZsdTimeStamp) {
			Log::warn("ZSD HealthCheck - ZSD seems to be down");
			return false;
		} else {
			$this->getSessionStorage()->setZsdTimeStamp($currentZsdTimeStamp);
			Log::debug("ZSD HealthCheck - ZSD is up and running");
			return true;
		}
	}
	
	/**
	 * @return \Servers\Db\Mapper
	 */
	public function getServersMapper() {
		return $this->serversMapper;
	}
	/**
	 * @param \Zsd\Servers\Db\Mapper $serversMapper
	 */
	public function setServersMapper($serversMapper) {
		$this->serversMapper = $serversMapper;
		return $this;
	}

	/**
	 * @return \Zsd\SessionStorage
	 */
	public function getSessionStorage() {
		if ($this->sessionStorage) return $this->sessionStorage;
				
		return $this->sessionStorage = new SessionStorage;
	}
	
	/**
	 * @param \Zsd\SessionStorage $sessionStorage
	 */
	public function setSessionStorage($sessionStorage) {
		$this->sessionStorage = $sessionStorage;
	}
	/* (non-PHPdoc)
	 * @see \ZendServer\EditionAwareInterface::setEdition()
	 */
	public function setEdition($edition) {
		$this->edition = $edition;
	}


	/**
	 * @return \ZendServer\Edition
	 */
	public function getEdition() {
		return $this->edition;
	}

	
}
