<?php
/**
 * Zend Framework (http://framework.zend.com/)
 *
 * @link      http://github.com/zendframework/Audit for the canonical source repository
 * @copyright Copyright (c) 2005-2012 Zend Technologies USA Inc. (http://www.zend.com)
 * @license   http://framework.zend.com/license/new-bsd New BSD License
 */

namespace Audit;

use Zend\ModuleManager\Feature\AutoloaderProviderInterface;
use Zend\Mvc\ModuleRouteListener;
use Zend\ModuleManager\Feature\ViewHelperProviderInterface;
use Zend\ModuleManager\Feature\ControllerPluginProviderInterface;
use Zend\ModuleManager\Feature\ServiceProviderInterface;
use Audit\Controller\Plugin\AuditMessage;
use Audit\Controller\Plugin\AuditMessageProgress;
use Audit\Db\Mapper;
use Zend\Db\TableGateway\TableGateway;
use Audit\Db\ProgressMapper;
use Audit\Db\SettingsMapper;
use Audit\View\Helper\auditType;
use Audit\View\Helper\auditTypeById;

class Module implements AutoloaderProviderInterface, ViewHelperProviderInterface, ControllerPluginProviderInterface, ServiceProviderInterface
{
    public function getAutoloaderConfig()
    {
        return array(
            'Zend\Loader\StandardAutoloader' => array(
                'namespaces' => array(
                    __NAMESPACE__ => __DIR__ . '/src/' . str_replace('\\', '/' , __NAMESPACE__),
                ),
            ),
        );
    }

    public function getConfig()
    {
        return include __DIR__ . '/config/module.config.php';
    }

    public function onBootstrap($e)
    {
    	
    }
    
    public function getViewHelperConfig() {
    	return array(
    			'factories' => array(
    				'auditType' => function($sm) {
    					$auditType = new auditType();
    					$auditType->setDictionary($sm->getServiceLocator()->get('Audit\Dictionary'));
    					return $auditType;
    				},
    				'auditTypeById' => function($sm) {
    					$auditType = new auditTypeById();
    					$auditType->setDictionary($sm->getServiceLocator()->get('Audit\Dictionary'));
    					return $auditType;
    				},
    			));
    }
    
    public function getControllerPluginConfig() {
    	$module = $this;
    	return array(
    		'factories' => array(
    			'AuditMessage' => function($sm) {
					return $sm->getServiceLocator()->get('Audit\Controller\Plugin\AuditMessage');
				},
				'AuditMessageProgress' => function($sm) {
					return $sm->getServiceLocator()->get('Audit\Controller\Plugin\AuditMessageProgress');
				},
    		));
    }
    
    public function getServiceConfig() {
    	$module = $this;
    	return array(
    			'factories' => array(
    				'Audit\Controller\Plugin\AuditMessage' => function ($sm) {
	    				$auditMessage = new AuditMessage();
	    				$auditMessage->setAuditMapper($sm->get('Audit\Db\Mapper'));
	    				return $auditMessage;
	    			},
					'Audit\Controller\Plugin\AuditMessageProgress' => function($sm) {
						$auditMessage = new AuditMessageProgress();
						$auditMessage->setServersMapper($sm->get('Servers\Configuration\Mapper'));
						$auditMessage->setServersDbMapper($sm->get('Servers\Db\Mapper'));
						$auditMessage->setAuditProgressMapper($sm->get('Audit\Db\ProgressMapper'));
						return $auditMessage;
					},
					'Audit\Db\Mapper' => function($sm) {
						$auditMapper = new Mapper();
						$auditMapper->setTableGateway(new TableGateway('ZSD_AUDIT', $sm->get('zsdDbAdapter')));
						$auditMapper->setProgressMapper($sm->get('Audit\Db\ProgressMapper'));
						return $auditMapper;
					},
					'Audit\Db\ProgressMapper' => function($sm) {
						$auditMapper = new ProgressMapper();
						$auditMapper->setTableGateway(new TableGateway('ZSD_AUDIT_PROGRESS', $sm->get('zsdDbAdapter')));
						return $auditMapper;
					},
					'Audit\Db\SettingsMapper' => function($sm) {
						$auditMapper = new SettingsMapper();
						$auditMapper->setPropertiesTable(new TableGateway('ZSD_AUDIT_PROPERTIES', $sm->get('zsdDbAdapter')));
						return $auditMapper;
					},
    			)
    		);
    }
}
