<?php

namespace Audit\Controller\Plugin;
use ZendServer\Exception;

use ZendServer\Edition;

use ZendServer\Log\Log;

use Audit\Db\Mapper;

use Application\Module;

use Audit\ProgressContainer;

use Zend\Json\Json;

use Audit\Container;
use Servers;

use Zend\Mvc\Controller\Plugin\AbstractPlugin;

class AuditMessageProgress extends AbstractPlugin
{
	/**
	 * @var \Audit\Db\ProgressMapper
	 */
	private $auditProgressMapper;
	/**
	 * @var \Servers\Configuration\Mapper
	 */
	private $serversMapper;
	/**
	 * @var \Servers\Db\Mapper
	 */
	private $serversDbMapper;
	
	public function __invoke($auditId, $progress, $extraData = array()) {

		try {
			try {
				$nodeId = $this->serversMapper->getServerNodeId();
				$serverData = $this->serversDbMapper->findServerById($nodeId)->toArray();
				$server = new Servers\Container($serverData, $nodeId);
			} catch (Exception $e) {
				$edition = new Edition();
				if ($edition->isClusterManager()) {
					$server = new Servers\Container(array(
								'NODE_ID' => 0,
								'NODE_IP' => $this->getController()->getRequest()->getServer('REMOTE_ADDR'),
								'NODE_NAME' => _t('Cluster Manager'),
							), 0);
				} else {
					throw $e;
				}
			}
			
			$progressMessage = new ProgressContainer();
			$progressMessage->setAuditId($auditId);
			$progressMessage->setProgress($progress);
			$progressMessage->setNodeId($server->getNodeId());
			$progressMessage->setNodeIp($server->getNodeIp());
			$progressMessage->setNodeName($server->getNodeName());
			$progressMessage->setCreationTime(time());
			$progressMessage->setExtraData($extraData);
			$this->auditProgressMapper->addAuditProgress($progressMessage);
			
		} catch (\Exception $e) {
			Log::warn("Failed writing progress entry for auditId {$auditId}, {$progress}");
			Log::debug($e);
			return new ProgressContainer();
		}
		
		return $progressMessage;
	}
	
	/**
	 * @param \Servers\Mapper $serversMapper
	 * @return AuditMessage
	 */
	public function setServersMapper($serversMapper) {
		$this->serversMapper = $serversMapper;
		return $this;
	}
	
	/**
	 * @param \Servers\Db\Mapper $serversDbMapper
	 * @return AuditMessage
	 */
	public function setServersDbMapper($serversDbMapper) {
		$this->serversDbMapper = $serversDbMapper;
		return $this;
	}
	/**
	 * @param \Audit\Db\ProgressMapper $auditProgressMapper
	 * @return AuditMessage
	 */
	public function setAuditProgressMapper($auditProgressMapper) {
		$this->auditProgressMapper = $auditProgressMapper;
		return $this;
	}




}