<?php

namespace Cache\Controller;

use ZendServer\Mvc\Controller\WebAPIActionController,
	Configuration\DdMapper,
	ZendServer\Set,
	ZendServer\Log\Log,
	ZendServer\Exception as ZSException,
	Audit\Db\Mapper as auditMapper,
	Audit\Db\ProgressMapper;
use Zsd\Db\TasksMapper;
use Configuration\Controller\ZendComponentsController;
use WebAPI\Exception;

class WebAPIController extends WebAPIActionController
{
	
	/**
	 * @throws WebAPI\Exception
	 */
	public function cacheClearAction() {
		$this->isMethodPost();

		$auditTypeArray = array(	ZendComponentsController::COMPONENT_ZEND_OPTIMIZER	=> auditMapper::AUDIT_CLEAR_OPTIMIZER_PLUS_CACHE,
									ZendComponentsController::COMPONENT_ZEND_DATA_CACHE 	=> auditMapper::AUDIT_CLEAR_DATA_CACHE_CACHE,
									ZendComponentsController::COMPONENT_ZEND_PAGE_CACHE 	=> auditMapper::AUDIT_CLEAR_PAGE_CACHE_CACHE);
		
		
		$params = $this->getParameters();
		$this->validateMandatoryParameters($params, array('component'));
		$this->validateAllowedValues($params['component'], 'component', array(ZendComponentsController::COMPONENT_ZEND_OPTIMIZER, ZendComponentsController::COMPONENT_ZEND_DATA_CACHE, ZendComponentsController::COMPONENT_ZEND_PAGE_CACHE));
		$audit = $this->auditMessage($auditTypeArray[$params['component']], ProgressMapper::AUDIT_PROGRESS_REQUESTED); /* @var $audit \Audit\Container */
		$tasksMapper = $this->getLocator()->get('Zsd\Db\TasksMapper'); /* @var $tasksMapper \Zsd\Db\TasksMapper */
		
		$taskId = '';
		
		if ($params['component'] == ZendComponentsController::COMPONENT_ZEND_PAGE_CACHE) {
			$tasks = $this->getLocator()->get('PageCache\Model\Tasks');
			$tasks->clearCache($audit->getAuditId());
		} else {
			switch ($params['component']) {
				case ZendComponentsController::COMPONENT_ZEND_OPTIMIZER:
					$taskId = TasksMapper::COMMAND_CLEAR_OPTIMIZER_PLUS_CACHE;
					break;
				case ZendComponentsController::COMPONENT_ZEND_DATA_CACHE:
					$taskId = array(TasksMapper::COMMAND_CLEAR_DATACACHE_DISK_CACHE, TasksMapper::COMMAND_CLEAR_DATACACHE_SHM_CACHE);
					break;
				default:
					throw new Exception("No clear action exists for this component {$params['component']}");
					
			}
			
			if (is_array($taskId)) {
				foreach ($taskId as $task) {
					$tasksMapper->insertTask(TasksMapper::DUMMY_NODE_ID, $task, $audit->getAuditId());
				}
			} else {
				$tasksMapper->insertTask(TasksMapper::DUMMY_NODE_ID, $taskId, $audit->getAuditId());
			}
		}
		
	
		return array();
	}
}