<?php

namespace Configuration;

use Zend\ModuleManager\Feature\ServiceProviderInterface;

use Zend\ModuleManager\Feature\AutoloaderProviderInterface as AutoloaderProvider;
use Configuration\License\ChangeListener;
use Configuration\License\LicenseChangeAnalyzer;
use Zend\EventManager\EventManager;

class Module implements AutoloaderProvider, ServiceProviderInterface
{
	/*
	 * (non-PHPdoc)
	 * @see \Zend\ModuleManager\Feature\ServiceProviderInterface::getServiceConfig()
	 */
	public function getServiceConfig() {
		return array(
					'factories' => array(
						'Configuration\License\ChangeListener' => function($sm) {
							$listener = new ChangeListener();
							$listener->setAcl($sm->get('ZendServerLicenseAcl'));
							$listener->setUsersMapper($sm->get('Users\Db\Mapper'));
							$listener->setTasksMapper($sm->get('Zsd\Db\TasksMapper'));
							$listener->setProfileMapper($sm->get('Snapshots\Mapper\Profile'));
							$listener->setlicenseChangeAnalyzer($sm->get('Configuration\License\LicenseChangeAnalyzer'));
							return $listener;
						},
						'Configuration\License\LicenseChangeAnalyzer' => function($sm) {
							$analyzer = new LicenseChangeAnalyzer();
							$analyzer->setZendServerUtils($sm->get('Configuration\License\ZemUtilsWrapper'));
							return $analyzer;
						},
						'Configuration\MapperReplies' => function($sm) {
							$mapper = new MapperReplies($sm->get('zsdReplies_tg'));
							$mapper->setTasksMapper($sm->get('Zsd\Db\TasksMapper'));
							return $mapper;
						},
						'Configuration\MapperDirectives' => function($sm) {
							$mapper = new MapperDirectives();
							$mapper->setEventManager(new EventManager());
							$mapper->setTableGateway($sm->get('zsdDirectives_tg'));
							$mapper->setTasksMapper($sm->get('Zsd\Db\TasksMapper'));
							$changeListener = $sm->get('Configuration\License\ChangeListener');
							$mapper->getEventManager()->attach($changeListener);
							return $mapper;
						},
						'Configuration\DbImport' => function($sm) {
							$mapper = new DbImport();
							$mapper->setAdapter($sm->get('zsdDbAdapter'));
							$mapper->setTasksMapper($sm->get('Zsd\Db\TasksMapper'));
							$mapper->setServersMapper($sm->get('Servers\Db\Mapper'));
							return $mapper;
						}
					)
				);
	}
	/*
	 * (non-PHPdoc)
	 * @see \Zend\ModuleManager\Feature\AutoloaderProviderInterface::getAutoloaderConfig()
	 */
	public function getAutoloaderConfig()
	{
		return array(
				'Zend\Loader\StandardAutoloader' => array(
						'namespaces' => array(
								__NAMESPACE__ => __DIR__ . '/src/' . __NAMESPACE__,
						),
				),
		);
	}
	
	public function getConfig()
	{
		return include __DIR__ . '/config/module.config.php';
	}
}
