<?php

namespace Configuration;

use ZendServer\Exception;

use ZendServer\Log\Log;

class DbImport {
	/**
	 * @var \Zend\Db\Adapter\Adapter
	 */
	private $adapter;
	/**
	 * @var Zsd\Db\TasksMapper
	 */
	private $tasksMapper;
	/**
	 * @var Servers\Db\Mapper
	 */
	private $serversMapper;
	
	/**
	 * @return \Zend\Db\Adapter\Adapter
	 */
	public function getAdapter() {
		return $this->adapter;
	}

	/**
	 * @return \Zsd\Db\TasksMapper $tasksMapper
	 */
	public function getTasksMapper() {
		return $this->tasksMapper;
	}

	/**
	 * @return \Servers\Db\Mapper $serversMapper
	 */
	public function getServersMapper() {
		return $this->serversMapper;
	}

	/**
	 * @param \Servers\Db\Mapper $serversMapper
	 * @return DbImport
	 */
	public function setServersMapper($serversMapper) {
		$this->serversMapper = $serversMapper;
		return $this;
	}

	/**
	 * @param \Zsd\Db\TasksMapper $tasksMapper
	 * @return DbImport
	 */
	public function setTasksMapper($tasksMapper) {
		$this->tasksMapper = $tasksMapper;
		return $this;
	}

	/**
	 * @param \Zend\Db\Adapter\Adapter $adapter
	 */
	public function setAdapter($adapter) {
		$this->adapter = $adapter;
	}
	
	public function importDatabase($sql) {
		$connection = $this->adapter->getDriver()->getConnection(); /* @var $connection \PDO */
		try{
			$connection->beginTransaction();
			$replaces = explode("REPLACE INTO", $sql);
			foreach ($replaces as $replace) {
				if ($replace) {
					$connection->execute("REPLACE INTO {$replace}");
				}
			}
			$this->getTasksMapper()->applyBlueprintToServers($this->getServersMapper()->findAllServersIds());
			$connection->commit();
		} catch (\Exception $e) {
			$connection->rollback();
			throw new Exception($e->getMessage(), Exception::ERROR, $e);
		}
	}
	
}
