<?php

namespace Configuration\License;

use ZendServer\Log\Log,
ZendServer\Exception as ZSException;
use ZendServer\FS\FS;
use Zend\Config\Reader\Ini;
use Configuration\License\License;

class ZemUtilsWrapper {
	
	/**
	 * @var License
	 */
	private $license;
	/**
	 * @param string $serialNumber
	 * @param string $userName
	 * @return License
	 * @throws ZendServer\Exception
	 */
	public function getSerialNumberInfo($serialNumber, $userName) {
		$method = 'zem_serial_number_info';
		
		try {
			$this->validateMethod($method);
			$licenseInfo = $method($serialNumber, $userName);
			if (!is_array($licenseInfo)) {				
				throw new ZSException('unexpected response received');				
			}
		} catch (\Exception $e) {
			Log::err("method {$method} invocation failed with the following error: " . $e->getMessage());
			throw new ZSException("method {$method} invocation failed with the following error: " . $e->getMessage());
		}
		
		return new License($licenseInfo);
	}
	
	/**
	 * @return \Configuration\License\License
	 */
	public function getLicenseInfo() {
		if ($this->license) return $this->license;
		try {
			$licenseDetails = \Application\Module::serviceManager()->get('\Configuration\MapperDirectives')->getDirectivesValues(array('zend.serial_number', 'zend.user_name'));
		} catch (\Exception $ex) {
			$config = new Ini();
			$ZGconfig = $config->fromFile(FS::getGlobalDirectivesFile());
			if (isset($ZGconfig['Zend']['zend'])) {
				$licenseDetails = array($ZGconfig['Zend']['zend']['serial_number'], $ZGconfig['Zend']['zend']['user_name']);
			} else {
				$licenseDetails = array($ZGconfig['zend']['serial_number'], $ZGconfig['zend']['user_name']);
			}
			Log::notice('License information retrieved from file');
			Log::debug($ex);
		}

		if (!isset($licenseDetails[1])) {
			Log::err("Failed retrieving license directives from ZS blueprint");
			throw new ZSException("Failed retrieving license directives from ZS blueprint");
		}
		
		list($serialNumber, $userName) = $licenseDetails;
		return $this->license = $this->getSerialNumberInfo($serialNumber, $userName);
	}

	public function isLicenseValid() {
		$licenseInfo = $this->getLicenseInfo();
	
		return $licenseInfo->isLicenseOk() && $licenseInfo->isSignatureValid() && $licenseInfo->isLicenseKnown();
	}	
	
	public function getLicenseFormattedExpiryDate() {
		$licenseInfo = $this->getLicenseInfo();
		
		return strftime("%d/%m/%Y", $licenseInfo->getExpiration());
	}
	
	public function getLicenseEvaluation() {
		return $this->getLicenseInfo()->isEvaluation();
	}
	
	public function getLicenseExpirationDaysNum($ignoreNotificationTime = false) {
		$trialExtended = \Application\Module::config('package', 'trialExtended');
		if ($trialExtended) {
			$notificationTime = \Application\Module::config('notifications', 'zend_gui', 'longNotificationTime');
		} else {
			$notificationTime = \Application\Module::config('notifications', 'zend_gui', 'shortNotificationTime');
		}		
		$daysToExpired = ceil(($this->getLicenseInfo()->getExpiration() - time()) / (60*60*24)); // using ceil, as license should be calculated till the end of the last day (rather than start)
		
		if ($daysToExpired <= $notificationTime || $ignoreNotificationTime) {
			return intval($daysToExpired);
		}
		
		return false; // not in the notification time range
	}
	
	public function getLicenseType() {
		$licenseInfo = $this->getLicenseInfo();
		return $licenseInfo->getEdition();
	}
	
	private function validateMethod($method) {
		if (!function_exists($method)) {
			throw new ZSException("method does not exist - 'zend utils' not loaded?");
		}
	}
}