<?php

namespace Configuration;

use Zend\Db\Sql\Select;

class MapperExtensions extends MapperAbstract {

	protected $setClass='\Configuration\ExtensionContainer';
	
	const NAME = 'NAME';
	const IS_ZEND_COMPONENT = 'IS_ZEND_COMPONENT';
	const IS_INSTALLED = 'IS_INSTALLED';
	const IS_LOADED = 'IS_LOADED';	

	
	/**
	 * @return \ZendServer\Set[\Configuration\ExtensionContainer]
	 */
	public function selectAllExtensions() {
		return $this->select();		
	}	


	/**
	 * @return array sql queries
	 */
	public function getExportData() {		
		$data = array();		
		$table = $this->getTableGateway()->getTable();
		$select = new Select($table);
		$res = $this->selectWith($select)->toArray();		
		$intVals =  array("IS_INSTALLED", "IS_LOADED");
		
		foreach ($res as $row) {			
			foreach ($row as $key => $val) {
				if (in_array($key, $intVals)) {
					$row[$key] = intval($row[$key]);
				} else {
					$row[$key] = "'" . $row[$key] . "'";
				}
			}		
			$line = "REPLACE INTO " . (string) $this->getTableGateway()->getTable() . " (" . implode(",", array_keys($row)) . ') VALUES (' . implode(",", array_values($row)) . ");";
			$data[] = $line; 
		}		
		
		return $data; 
	}
	

	/**
	 * @return \ZendServer\Set[\Configuration\ExtensionContainer]
	 */
	public function selectAllZendExtensions() {
		$table = $this->getTableGateway()->getTable();
		$select = new Select($table);
		$select->where(array(self::IS_ZEND_COMPONENT => 1));
		$select->order(array(self::NAME => 'ASC')); // A-Z
		return $this->selectWith($select);
	}
	
	/**
	 * @return \Zend\Db\ResultSet\ResultSet
	 */
	public function selectAllPHPExtensions() {
		return $this->select(array(self::IS_ZEND_COMPONENT => 0));
	}
	
	/**
	 * @return \ZendServer\Set[\Configuration\ExtensionContainer]
	 */
	public function selectAllPHPExtensionsInstalled() {		
		$predicate1 = self::IS_ZEND_COMPONENT . '=0';
		$predicate2 = self::IS_LOADED . '=1';
		$predicate3 = self::IS_INSTALLED . '=1';
		
		return $this->select("{$predicate1} AND ({$predicate2} OR {$predicate3})"); // BuiltIn extensions have IS_INSTALLED = "false" && IS_LOADED = "true"
	}
	
	/**
	 * @return \ZendServer\Set[\Configuration\ExtensionContainer]
	 */
	public function selectExtensions(array $extensions) {		
		$predicate = $this->getSqlInStatement(self::NAME, $extensions);

		return $this->select(array($predicate)); // in values, should look like: ("bcmath","bz2")
	}	
	/**
	 * @return \Configuration\ExtensionContainer
	 */
	public function selectExtension($extension) {		
		return $this->select(array(self::NAME => $extension))->current(); // in values, should look like: ("bcmath","bz2")
	}
	
	/**
	 * @param string $extension
	 * @return boolean
	 */
	public function isExtensionLoaded($extension) {
		$res = $this->select(array(self::NAME => $extension, self::IS_LOADED => 1), false); // simply get back an array
		return (Boolean) $res; // whether the array is empty or not
	}
}
