<?php

namespace Configuration;

use Zend\Db\Sql\Select;
use Zend\Db\TableGateway\TableGateway;
use Zsd\Db\TasksMapper;
use ZendServer\Set;
use ZendServer\Log\Log;

class MapperReplies extends MapperAbstract {
	protected $setClass='\Configuration\ServerInfoReplyContainer';
	
	const ZSD_TASKS_SEQUENCE = 'ZSD_TASKS_SEQUENCE';
	
	/**
	 * @var TasksMapper
	 */
	protected $tasksMapper;

	/**
	 *
	 * @param integer $task
	 * @return boolean
	 */
	public function isTaskServerInfoComplete($task) {
		$resultSet = $this->tableGateway->select(array(self::ZSD_TASKS_SEQUENCE => $task));
	
		/* @var $resultSet \Zend\Db\ResultSet\ResultSet */
		return ($resultSet && count($resultSet->toArray()) > 0) ? true : false;
	}
	
	/**
	 *
	 * @param integer $task
	 * @return Set
	 */
	public function getServerInfo($taskId) {
		return $this->select(array(self::ZSD_TASKS_SEQUENCE => $taskId));
	}

	public function getServerInfoWithRetry($serverId) {
		$taskId = $this->getTasksMapper()->insertTask($serverId, TasksMapper::COMMAND_GET_PHPINFO);
		try {
			$maxSleep = 0;			
			while(! $this->isTaskServerInfoComplete($taskId) && $maxSleep < 6) { // sleeping of maximum 6 secs
				$maxSleep++;
				Log::debug('sleep tick');
				sleep(1);
			}
			
			$serverInfo = $this->getServerInfo($taskId);
			if ((! ($serverInfo instanceof Set)) || (0 == $serverInfo->count()) || (! $serverInfo->current()->hasReply())) {
				throw new \Exception(_t("Webserver took an unusually long time to respond. Plese try again")); 
			}
			
			$serverData = $serverInfo->current()->toArray();
		} catch (\Exception $e) {
			throw new \WebAPI\Exception($e->getMessage(), \WebAPI\Exception::INTERNAL_SERVER_ERROR, $e);
		}
		
		return $serverData;
	}
	
	/**
	 * @return \Zend\Db\TableGateway\TableGateway $tableGateway
	 */
	public function getTableGateway() {
		return $this->tableGateway;
	}
	
	/**
	 * @return TasksMapper
	 */
	public function getTasksMapper() {
		return $this->tasksMapper;
	}

	/**
	 * @param TasksMapper $tasksMapper
	 */
	public function setTasksMapper($tasksMapper) {
		$this->tasksMapper = $tasksMapper;
		return $this;
	}

	/**
	 * @param \Zend\Db\TableGateway\TableGateway $tableGateway
	 * @return \Servers\Db\Mapper
	 */
	public function setTableGateway($tableGateway) {
		$this->tableGateway = $tableGateway;
		return $this;
	}
	
}