<?php

namespace Deployment;

use Deployment\View\Helper\ApplicationUrl;

use Zend\ModuleManager\Feature\ViewHelperProviderInterface;

use Zend\EventManager\EventInterface;

use Application\Module as appModule;

use Zend\EventManager\Event;

use Servers\Db\Mapper;

use ZendDeployment_Manager;

use DeploymentLibrary\Mapper as DeploymentLibraryMapper;

use DeploymentLibrary\Db\Mapper as DeploymentLibraryUpdateMapper;

use Deployment\Validator\ApplicationNameNotExists;
use Deployment\Forms\SetInstallation,
	Deployment\Forms\DefineApplicationForm;

use Zend\ModuleManager\Feature\ConfigProviderInterface;

use Zend\ModuleManager\Feature\ServiceProviderInterface;

use Zend\ModuleManager\Feature\BootstrapListenerInterface;

use Zend\Crypt\PublicKey\Rsa\PublicKey;

use Zend\Mvc\MvcEvent;

use ZendServer\Log\Log;

use Zend\ModuleManager\Feature\InitProviderInterface;

use Zend\ModuleManager\ModuleManager,
	Zend\EventManager\Event as ManagerEvent,
    Zend\EventManager\StaticEventManager,
    Zend\ModuleManager\Feature\AutoloaderProviderInterface as AutoloaderProvider,
	Application\View,
	Zend\Http\Header\Accept;
use DeploymentLibrary\Controller\Plugin\SetUpdateCookie;

class Module implements ViewHelperProviderInterface, AutoloaderProvider, BootstrapListenerInterface, ServiceProviderInterface, ConfigProviderInterface
{
	/**
	 * @var boolean
	 */
	private $supportedByWebserver;
	
	public function onBootstrap(EventInterface $e) {
		$this->initializeFeatures($e);
		$serviceManager = $e->getApplication()->getServiceManager();
		$serviceManager->addInitializer(function ($instance) use ($serviceManager) {
			if ($instance instanceof IdentityApplicationsAwareInterface) {
				$instance->setIdentityFilter($serviceManager->get('Deployment\IdentityFilter'));
			}
		});
	}
	/*
	 * (non-PHPdoc)
	 * @see \Zend\ModuleManager\Feature\ViewHelperProviderInterface::getViewHelperConfig()
	 */
	public function getViewHelperConfig() {
		return array(
				'factories' => array(
					'ApplicationUrl' => function($sm) {
						$helper = new ApplicationUrl();
						$helper->setDefaultServer(appModule::config('deployment', 'defaultServer'));
						return $helper;
					}
				)
			);
	}
	
	public function getServiceConfig() {
		$module = $this;
		return array(
			'factories' => array(
				'DeploymentLibrary\Mapper' => function ($sm) use ($module) {
					$mapper = new DeploymentLibraryMapper();
					$mapper->setDeploySupportedByWebserver($module->getSupportedByWebserver());
					$mapper->setServersMapper($sm->get('Servers\Db\Mapper'));
					return $mapper;
				},
				'Deployment\Model' => function ($sm) use ($module) {
					$model = new Model();
					$model->setServersMapper($sm->get('Servers\Db\Mapper'));
					$model->setManager(new ZendDeployment_Manager());
					$model->setDeploySupportedByWebserver($module->getSupportedByWebserver());
					if (! appModule::config('authentication', 'simple')) {
						$model->getEventManager()->attach('preRemove', function(Event $e) use ($sm) {
							$application = $e->getTarget(); /* @var $application \Deployment\Application\Container */
							$mapperGroups = $sm->get('Acl\Db\MapperGroups'); /* @var $mapperGroups \Acl\Db\MapperGroups */
							$mapperGroups->deleteMapping($application->getApplicationId());
						});
					}
					if (appModule::config('deployment', 'demoAppId')) {
						$model->getEventManager()->attach('postRemove', function(Event $e) use ($sm) {
							$application = $e->getTarget(); /* @var $application \Deployment\Application\Container */
							if ($application->getApplicationId() == appModule::config('deployment', 'demoAppId')) {
								Log::info('Remove demo app marker');
								$mapper = $sm->get('Zsd\Db\TasksMapper'); /* @var $mapper \Zsd\Db\TasksMapper */
								$mapper->setGuiDirectives(array('demoAppId' => 0));
							}
						});
					}
					if (appModule::config('deployment', 'samplesAppId')) {
						$model->getEventManager()->attach('postRemove', function(Event $e) use ($sm) {
							$application = $e->getTarget(); /* @var $application \Deployment\Application\Container */
							if ($application->getApplicationId() == appModule::config('deployment', 'samplesAppId')) {
								Log::info('Remove samples app marker');
								$mapper = $sm->get('Zsd\Db\TasksMapper'); /* @var $mapper \Zsd\Db\TasksMapper */
								$mapper->setGuiDirectives(array('samplesAppId' => 0));
							}
						});
					}
					return $model;
				},
				'Deployment\FilteredAccessMapper' => function ($sm) use ($module) {
					$mapper = new FilteredAccessMapper();
					$mapper->setModel($sm->get('Deployment\Model'));
					return $mapper;
				},
				'Deployment\IdentityFilter' => function($sm) {
					if (! appModule::config('authentication', 'simple')) {
						$mapperGroups = $sm->get('Acl\Db\MapperGroups');
						$filter = new IdentityFilterGroups();
						$filter->setMapperGroups($mapperGroups);
					} else {
						$filter = new IdentityFilterSimple();
					}
					$filter->setDeploymentMapper($sm->get('Deployment\Model'));
					return $filter;
				},
				'Deployment\Form\SetInstallation' => function ($sm) use ($module) {
					$model = $sm->get('Deployment\Model');
					$form = new SetInstallation(null, $model);
					$form->fillWithGeneralDeploymentData ($model->getVirtualHosts ());
					return $form;
				},
				'Deployment\Form\DefineApplicationForm' => function ($sm) use ($module) {
					$form = new DefineApplicationForm(null, $sm->get('Deployment\Model'));
					return $form;
				},
			)
		);
	}
	
	public function getControllerPluginConfig() {
		$module = $this;
		return array(
				'factories' => array(
						'SetLibrariesUpdateCookie' => function($sm) { /* @var $sm \Zend\Mvc\Controller\PluginManager */
							$plugin = new SetUpdateCookie();
							$plugin->setAcl($sm->getServiceLocator()->get('ZendServerAcl'));
							$plugin->setLibrariesMapper($sm->getServiceLocator()->get('DeploymentLibrary\Mapper'));
							$plugin->setUpdatesMapper($sm->getServiceLocator()->get('DeploymentLibrary\Db\Mapper'));
							return $plugin;
						},
				));
	}
	
	/**
	 * @param MvcEvent $e
	 */
	public function initializeFeatures(MvcEvent $e) {
		$request = $e->getRequest(); /* @var $request \Zend\Http\PhpEnvironment\Request */
		$webserver = $request->getServer('SERVER_SOFTWARE', '');
		if (strpos($webserver, 'Apache') !== false || strpos($webserver, 'lighttpd') !== false) {
			$this->supportedByWebserver = true;
		} else {
			$this->supportedByWebserver = false;
		}
	}
	
	public function getAutoloaderConfig() {
		return array(
				'Zend\Loader\StandardAutoloader' => array(
						'namespaces' => array(
								__NAMESPACE__ => __DIR__ . '/src/' . __NAMESPACE__,
								'Prerequisites' => __DIR__ . '/src/Prerequisites',
								'DeploymentLibrary' => __DIR__ . '/src/DeploymentLibrary',
						),
				),
		);
	}
	
	public function getConfig()
	{
		return include __DIR__ . '/config/module.config.php';
	}
	/**
	 * @return boolean $supportedByWebserver
	 */
	public function getSupportedByWebserver() {
		return $this->supportedByWebserver;
	}

}
