<?php

namespace DeploymentLibrary\Controller;

use ZendDeployment_Manager,
	Audit\Db\Mapper as auditMapper,
	Audit\Db\ProgressMapper,
	Deployment\Controller\WizardController as baseWizardController,
	Deployment\SessionStorage,
 	ZendServer\Text, 
	ZendServer\Exception, 
	ZendServer\Log\Log;
use Zend\View\Model\ViewModel;

class WizardController extends baseWizardController {
	
	public function wizardAction() {
		$sessionStorage = new SessionStorage();
		$sessionStorage->clear();
		
		$viewModel = new \Zend\View\Model\ViewModel ();
		$viewModel->setTerminal (true);
		
		return $viewModel;
	}
	
	public function uploaderAction() {
		$this->Layout ( 'layout/login' );
		$viewModel = new ViewModel();
		$viewModel->setTemplate('deployment/wizard/uploader');
		$viewModel->setVariable('controller', 'LibraryWizard');
		$viewModel->setVariable('action', 'Upload');
		return $viewModel;
	}
	
	public function summaryAction() {
		$viewModel = new \Zend\View\Model\ViewModel();
		$viewModel->setTerminal (true);
		$viewModel->setVariable ('success', true);
		
		$sessionStorage = new SessionStorage();
		$package = $sessionStorage->getStoredPackage();
		
		$viewModel->setVariable('logo', $package->getLogo());
		$viewModel->setVariable('libraryName', $package->getName());
		$viewModel->setVariable('version', $package->getVersion());
		return $viewModel;
	}
	
	public function deployAction() {
		$viewModel = new \Zend\View\Model\ViewModel();
		$viewModel->setTerminal(true);
	
		try {
			$sessionStorage = new SessionStorage();
			$packagePath = $sessionStorage->getPackageFilePath();
			$package = $sessionStorage->getStoredPackage();
			$packageName = $package->getName();
			$model = $this->getLocator('Deployment\Model'); /* @var $model \Deployment\Model */
		} catch ( Exception $e ) {
			Log::err ( 'Could not retrieve the library package by base URL: '. $e->getMessage() );
			throw new Exception ( 'Library package is missing.  Retry your last action and if this problem persists, go to the Zend Support Center at: http://www.zend.com/support-center' );
		}
	
		$auditMessage = $this->auditMessage(auditMapper::AUDIT_LIBRARY_DEPLOY, ProgressMapper::AUDIT_PROGRESS_REQUESTED, array(array(_t('Library name: %s (%s)', array($packageName, $package->getVersion())))), $packagePath); /* @var $auditMessage \Audit\Container */
		$library = $this->getDeploymentLibraryMapper()->deployLibrary($packagePath, $auditMessage->getAuditId());
		
		$viewModel->setVariable('libraryInfo', $library);
		
		return $viewModel;
	}
	
	public function cancelAction() {
		
		$sessionStorage = new SessionStorage();
		$sessionStorage->clear();
		
		return $this->getResponse();
	}
	
	protected function validateUploadedPackage($packagePath) {
		$this->getDeploymentLibraryMapper()->validatePackage($packagePath);
	}
	
	/**
	 * Get EULA text
	 *
	 * @todo implement
	 * @return string
	 */
	private function getEulaContents() {
		$sessionStorage = new SessionStorage ();
		$package = $sessionStorage->getStoredPackage();
		return $package->getEula ();
	}
	
	/**
	 * @param array $params
	 * @return boolean
	 */
	private function isWizardRequest($params) {
		return isset($params['wizardAjax']);
	}
}
