<?php

namespace JobQueue\Controller;

use Zend\View\Model\ViewModel;
use ZendServer\Log\Log;
use ZendServer\Edition;
use ZendServer\Mvc\Controller\ActionController,
	Application\Module;

class IndexController extends ActionController
{
    public function ListAction() {
    	$output = $this->forward()->dispatch('JobQueue', array('action' => 'Index'));
    	$output->setTemplate('job-queue/index/index');
    	return $output;
    }
    
    public function IndexAction() {
    	if (! $this->isAclAllowedEdition('route:JobQueueWebApi')) {
    		$viewModel = new ViewModel();
    		$viewModel->setTemplate('job-queue/index/index-marketing');
    		return $viewModel;
    	}

    	$jobqueueView = $this->forward()->dispatch('JobQueueWebApi-1_3', array('action' => 'jobqueueJobsList')); /* @var $jobqueueView \Zend\View\Model\ViewModel */
    	$jobqueueView->setTemplate('job-queue/index/index');// Restoring original route
    	$jobqueueView->setVariable('perPage', Module::config('list', 'resultsPerPage'));
    	
    	$filterDictionary = $this->getLocator()->get('JobQueue\Filter\Dictionary'); /* @var $filterDictionary \JobQueue\Filter\Dictionary */
    	
    	$deploymentModel = $this->getLocator()->get('Deployment\Model'); /* @var $deploymentModel \Deployment\Model */
    	$applications = $deploymentModel->getMasterApplications();
    	$applications->setHydrateClass('\Deployment\Application\Container');
    	$applicationsDictionary = array();
    	foreach ($applications as $app) {
    		$applicationsDictionary[$app->getApplicationId()] = $app->getUserApplicationName();
    	}
    	
    	$mapper = $this->getLocator()->get('JobQueue\Db\Mapper'); /* @var $mapper \JobQueue\Db\Mapper */
    	$schedulingRules = array();
    	try {
    		foreach ($mapper->getSchedulingRules() as $rule) {
    			$schedulingRules[$rule['id']] = $rule['name'];
    		}   		
    	} catch (\ZendJobQueueException $e) {
    		throw new \ZendServer\Exception("No Job Queue data available: " . $e->getMessage()); // will most probably never reach here
    	}

    	$jobqueueView->setVariable(	'internalFilters', array(	'priority' =>	array('name' => 'priority', 'label' => _t('Priority'), 'options' => $filterDictionary->getPriorities()),
												    			'status' => array('name' => 'status', 'label' => _t('Status'), 'options' => $filterDictionary->getStatusDictionaryForFiltering()),
												    			'app_ids'=> array('name' => 'app_ids', 'label' => _t('Application'), 'options' => $applicationsDictionary, 'noOptionsError' => _t('Currently no deployed/defined applications')),
    															'rule_ids' => array('name' => 'rule_ids', 'label' => _t('Scheduling Rule'), 'options' => $schedulingRules, 'noOptionsError' => _t('Currently no defined recurring jobs'))));
    	
    	$jobqueueView->setVariable('externalFilters', array(	array('name' => 'timeRange', 'label' => '', 'options' => $filterDictionary->getJQTimeRange(), 'extra' => $filterDictionary->getTimeRanges())));
    	$mapper = $this->getLocator('ZendServer\Filter\Mapper');
    	$existingFilters = array();
    	foreach ($mapper->getByType('job') as $filter) { /* @var $filter \ZendServer\Filter\Filter */
    		$existingFilters[$filter->getName()] = array('id' => $filter->getId(),
    				'name' => $filter->getName(), 'custom' => $filter->getCustom(), 'data' => $filter->getData());
    	}
    	
    	$jobqueueView->setVariable('existingFilters', $existingFilters);
    	$edition = new Edition();
    	$jobqueueView->setVariable('singleServer', $edition->isSingleServer());
    	
    	return $jobqueueView;
    }
    
    public function JobInfoAction() {
    	$viewModel = new \Zend\View\Model\ViewModel();
    	$viewModel->setTerminal(true);
    	
    	$mapper = $this->getLocator()->get('JobQueue\Model\Mapper'); /* @var $mapper \JobQueue\Model\Mapper */
    	$dictionary = $this->getLocator()->get('jq_dictionary'); /* @var $dictionary \JobQueue\Filter\Dictionary */
    	
    	$params = $this->getParameters(array(
					'id' => ''));
		$this->validateMandatoryParameters($params, array('id'));
    	
		$deploymentModel = $this->getLocator()->get('Deployment\FilteredAccessMapper'); /* @var $deploymentModel \Deployment\FilteredAccessMapper */
		$applicationsSet = $deploymentModel->getAllApplicationsInfo();
		$applicationsSet->setHydrateClass('\Deployment\Application\InfoContainer');
		
		$job = $mapper->getJob($params['id']);
		$viewModel->setVariable('job', $job);
		$viewModel->setVariable('priorities', $dictionary->getDbPriorityDictionary());
		$viewModel->setVariable('applications', $applicationsSet);
		
		// by default set to false the job rule 
		$viewModel->setVariable('jobRule', 0);
		if (isset($job['schedule_id']) && $job['schedule_id'] != 1) {
			
			try {
				$rule = $mapper->getSchedulingRule($job['schedule_id']);
			} catch (\Exception $e) {
				// no such rule with $job['schedule_id']
			}
			if (is_array($rule) && !empty($rule)) {
				$viewModel->setVariable('jobRule', 1);
			}
		}
    	return $viewModel;
    }
    
	public function RuleInfoAction() {
    	$viewModel = new \Zend\View\Model\ViewModel();
    	$viewModel->setTerminal(true);
    	
    	$mapper = $this->getLocator()->get('JobQueue\Model\Mapper'); /* @var $mapper \JobQueue\Model\Mapper */
    	$dictionary = $this->getLocator()->get('jq_dictionary'); /* @var $dictionary \JobQueue\Filter\Dictionary */
    	
    	$params = $this->getParameters(array(
					'id' => ''));
		$this->validateMandatoryParameters($params, array('id'));
    	
		$deploymentModel = $this->getLocator()->get('Deployment\FilteredAccessMapper'); /* @var $deploymentModel \Deployment\FilteredAccessMapper */
		$applicationsSet = $deploymentModel->getAllApplicationsInfo();
		$applicationsSet->setHydrateClass('\Deployment\Application\InfoContainer');
		
		$rule = $mapper->getSchedulingRule($params['id']);
		$viewModel->setVariable('rule', $rule);
		$viewModel->setVariable('priorities', $dictionary->getPriorityDictionary());
		$viewModel->setVariable('applications', $applicationsSet);
		
    	return $viewModel;
    }
}
