<?php
namespace JobQueue\Controller;

use Zend\View\Model\ViewModel;
use JobQueue\Model\RecurringJobSchedule;

use Zend\Json\Json;

use JobQueue\Model\RecurringJob;

use ZendServer\Log\Log;
use JobQueue\Model\RulesSet;
use ZendServer\Mvc\Controller\ActionController,
Application\Module;

class RecurringJobsController extends ActionController
{
	public function IndexAction() {
		if (! $this->isAclAllowedEdition('route:JobQueueWebApi')) {
			$viewModel = new ViewModel();
			$viewModel->setTemplate('job-queue/recurring-jobs/index-marketing');
			return $viewModel;
		}
		
		$mapper = $this->getLocator()->get('JobQueue\Db\Mapper'); /* @var $mapper \JobQueue\Db\Mapper */
		$dictionary = $this->getLocator('jq_dictionary'); /* @var $dictionary \JobQueue\Filter\Dictionary */

		$params = $this->getParameters(array(
				'limit' => Module::config('list', 'resultsPerPage'),
				'offset' => 0,
				'order' => 'creationTime',
				'direction' => 'DESC'
			));
		$rules = $mapper->getSchedulingRules();
		
		$deploymentModel = $this->getLocator()->get('Deployment\FilteredAccessMapper'); /* @var $deploymentModel \Deployment\FilteredAccessMapper */
		$applicationsSet = $deploymentModel->getAllApplicationsInfo();
		$applicationsSet->setHydrateClass('\Deployment\Application\InfoContainer');
		
		return array(
					'rules' => new RulesSet($rules),
					'perPage' => 200,
					'statuses' => $dictionary->getRuleStatusDictionary(),
					'priorities' => $dictionary->getPriorityDictionary(),
					'applications' => $applicationsSet,
		);
	}
	
	public function createAction() {
		$this->getEvent()->getRouteMatch()->setParam('action', 'form');
		$deploymentModel = $this->getLocator()->get('Deployment\FilteredAccessMapper'); /* @var $deploymentModel \Deployment\FilteredAccessMapper */
		$applicationsSet = $deploymentModel->getAllApplicationsInfo();
		$applicationsSet->setHydrateClass('\Deployment\Application\InfoContainer');
		$dictionary = $this->getLocator('jq_dictionary'); /* @var $dictionary \JobQueue\Filter\Dictionary */
		return array('rule' => null, 'parsedSchedule' => new RecurringJobSchedule(''),
					'statuses' => $dictionary->getRuleStatusDictionary(),
					'priorities' => $dictionary->getPriorityDictionary(),
					'applications' => $applicationsSet,);
	}
	
	public function updateAction() {
		$params = $this->getParameters(array('jobId' => 0));
		$jobId = $params['jobId'];
		
		$mapper = $this->getLocator()->get('JobQueue\Model\Mapper'); /* @var $mapper \JobQueue\Model\Mapper */
		$rule = $mapper->getSchedulingRule($jobId);
		
		$recurringJob = new RecurringJob($rule);
		$this->getEvent()->getRouteMatch()->setParam('action', 'form');
		
		$deploymentModel = $this->getLocator()->get('Deployment\FilteredAccessMapper'); /* @var $deploymentModel \Deployment\FilteredAccessMapper */
		$applicationsSet = $deploymentModel->getAllApplicationsInfo();
		$applicationsSet->setHydrateClass('\Deployment\Application\InfoContainer');
		
		$dictionary = $this->getLocator('jq_dictionary'); /* @var $dictionary \JobQueue\Filter\Dictionary */
		return array('rule' => $rule, 'parsedSchedule' => $recurringJob->getParsedSchedule(),
					'statuses' => $dictionary->getRuleStatusDictionary(),
					'priorities' => $dictionary->getPriorityDictionary(),
					'applications' => $applicationsSet,);
	}
}