<?php

namespace JobQueue\Filter;

use ZendJobQueue;

use \ZendServer\Exception;

class Dictionary {	

	// Sort
	const COLUMN_ID = 'id';
	const COLUMN_TYPE = 'type';
	const COLUMN_SCRIPT = 'script';
	const COLUMN_APP = 'application';
	const COLUMN_NAME = 'name';
	const COLUMN_PRIORITY = 'priority';
	const COLUMN_STATUS = 'status';
	const COLUMN_PREDECESSOR = 'predecessor';
	const COLUMN_PERSISTENCE = 'persistence';
	const COLUMN_CREATIONTIME = 'creationTime';
	const COLUMN_SCHEDULETIME = 'scheduleTime';
	const COLUMN_STARTTIME = 'startTime';
	const COLUMN_ENDTIME = 'endTime';
	
	//Filter only
	const FILTER_COLUMN_APP_ID = 'app_id';
	const FILTER_COLUMN_APP_IDS = 'app_ids';
	const FILTER_COLUMN_RULE_ID = 'rule_id';
	const FILTER_COLUMN_RULE_IDS = 'rule_ids';
	const FILTER_COLUMN_SCHEDULED_BEFORE = 'scheduled_before';
	const FILTER_COLUMN_SCHEDULED_AFTER = 'scheduled_after';
	const FILTER_COLUMN_EXECUTED_BEFORE = 'executed_before';
	const FILTER_COLUMN_EXECUTED_AFTER = 'executed_after';
	const FILTER_COLUMN_FREE_TEXT = 'freeText';
	
	
	const STATUS_PENDING 				= 'pending';
	const STATUS_WAITING_PREDECESSOR	= 'waiting_predecessor';	
	const STATUS_RUNNING 				= 'running';
	const STATUS_COMPLETED 				= 'completed';
	const STATUS_OK 					= 'ok';
	const STATUS_FAILED 				= 'failed';
	const STATUS_LOGICALLY_FAILED		= 'logically_failed';
	const STATUS_TIMEOUT 				= 'timeout';
	const STATUS_REMOVED 				= 'removed';
	const STATUS_SCHEDULED 				= 'scheduled';
	const STATUS_SUSPENDED 				= 'suspended';
		
	const PRIORITY_LOW		= 'low';
	const PRIORITY_NORMAL	= 'normal';
	const PRIORITY_HIGH		= 'high';
	const PRIORITY_URGENT	= 'urgent';
	
	const SORT_DESC	= 'DESC';
	const SORT_ASC	= 'ASC';	
	
	/**
	 * @return array
	 */
	public function getSortColumnsDictionary() {
		return array(
				self::COLUMN_ID				=> ZendJobQueue::SORT_BY_ID,
				self::COLUMN_TYPE			=> ZendJobQueue::SORT_BY_TYPE,
				self::COLUMN_SCRIPT			=> ZendJobQueue::SORT_BY_SCRIPT,
				self::COLUMN_APP			=> ZendJobQueue::SORT_BY_APPLICATION,
				self::COLUMN_NAME			=> ZendJobQueue::SORT_BY_NAME,
				self::COLUMN_PRIORITY		=> ZendJobQueue::SORT_BY_PRIORITY,
				self::COLUMN_STATUS			=> ZendJobQueue::SORT_BY_STATUS,
				self::COLUMN_PREDECESSOR	=> ZendJobQueue::SORT_BY_PREDECESSOR,
				self::COLUMN_PERSISTENCE	=> ZendJobQueue::SORT_BY_PERSISTENCE,
				self::COLUMN_SCHEDULETIME	=> ZendJobQueue::SORT_BY_SCHEDULE_TIME,
				self::COLUMN_STARTTIME		=> ZendJobQueue::SORT_BY_START_TIME,
				self::COLUMN_ENDTIME		=> ZendJobQueue::SORT_BY_END_TIME,
				self::COLUMN_CREATIONTIME	=> ZendJobQueue::SORT_BY_CREATION_TIME,
		);
	}


	/**
	 * @return array
	 */
	public function getFilterColumns() {
		return array(
				self::COLUMN_TYPE			=> self::COLUMN_TYPE,
				self::COLUMN_SCRIPT			=> self::COLUMN_SCRIPT,
				self::COLUMN_NAME			=> self::COLUMN_NAME,
				self::COLUMN_PRIORITY		=> self::COLUMN_PRIORITY,
				self::COLUMN_STATUS			=> self::COLUMN_STATUS,
				
				self::FILTER_COLUMN_APP_IDS				=> self::FILTER_COLUMN_APP_IDS,				
				self::FILTER_COLUMN_RULE_IDS			=> self::FILTER_COLUMN_RULE_IDS,				
				self::FILTER_COLUMN_SCHEDULED_BEFORE	=> self::FILTER_COLUMN_SCHEDULED_BEFORE,				
				self::FILTER_COLUMN_SCHEDULED_AFTER		=> self::FILTER_COLUMN_SCHEDULED_AFTER,				
				self::FILTER_COLUMN_EXECUTED_BEFORE		=> self::FILTER_COLUMN_EXECUTED_BEFORE,				
				self::FILTER_COLUMN_EXECUTED_AFTER		=> self::FILTER_COLUMN_EXECUTED_AFTER,
				self::FILTER_COLUMN_FREE_TEXT			=> self::FILTER_COLUMN_FREE_TEXT,
		);
	}
	


	/**
	 * @return array
	 */
	public function getStatusColumnsDictionary() {
		return array(
				self::STATUS_PENDING				=> ZendJobQueue::STATUS_PENDING,
				self::STATUS_WAITING_PREDECESSOR	=> ZendJobQueue::STATUS_WAITING_PREDECESSOR,
				self::STATUS_RUNNING				=> ZendJobQueue::STATUS_RUNNING,
				self::STATUS_COMPLETED				=> ZendJobQueue::STATUS_COMPLETED,
				self::STATUS_OK						=> ZendJobQueue::STATUS_OK,
				self::STATUS_FAILED					=> ZendJobQueue::STATUS_FAILED,
				self::STATUS_LOGICALLY_FAILED		=> ZendJobQueue::STATUS_LOGICALLY_FAILED,
				self::STATUS_TIMEOUT				=> ZendJobQueue::STATUS_TIMEOUT,
				self::STATUS_REMOVED				=> ZendJobQueue::STATUS_REMOVED,
				self::STATUS_SCHEDULED				=> ZendJobQueue::STATUS_SCHEDULED,
				self::STATUS_SUSPENDED				=> ZendJobQueue::STATUS_SUSPENDED,
		);
	}		

	private $priorityToDbPriority = array ( // sadly, the priorities kept in the DB are reversed that actual values!
			ZendJobQueue::PRIORITY_LOW => ZendJobQueue::PRIORITY_URGENT,
			ZendJobQueue::PRIORITY_NORMAL => ZendJobQueue::PRIORITY_HIGH,
			ZendJobQueue::PRIORITY_HIGH => ZendJobQueue::PRIORITY_NORMAL,
			ZendJobQueue::PRIORITY_URGENT => ZendJobQueue::PRIORITY_LOW,
	);

	public function getStatuses() {
		$arrayStatus = array();
		foreach (array_keys($this->getStatusColumnsDictionary()) as $key) {
			$arrayStatus[$key] = $key;
		}
		
		return $arrayStatus;	
	}
	
	public function getStatusConstant($status) {
		if (!$this->isKnownStatus($status)) {
			throw new Exception(_t("status '%s' is not a known status: '%s'", array($status, implode(',', array_keys($this->getStatusDictionaryReversed())))));
		}
	
		$statuses = $this->getStatusDictionaryReversed();
		return $statuses[$status];
	}
	
	public function isKnownStatus($status) {
		return array_key_exists($status, $this->getStatusDictionaryReversed());
	}
	
	public function statusesToBit(array $statuses) {
		$statusBit = 0;
		foreach ($statuses as $status) {
			$statusBit |= $this->getStatusConstant($status);
		}
	
		return $statusBit;
	}
	
	public function statusesToDbValues(array $statuses) {
		$res = array();
		foreach ($statuses as $status) {
			$constant = $this->getStatusConstant($status);
			if (!is_array($constant)) {
				$constant = array($constant);
			}
			
			foreach ($constant as $singleConstant) {
				$res[] = $singleConstant;
			}			
		}
	
		return $res;
	}

		
	/**
	 * @return array
	 */
	public function getSortDirectionColumnsDictionary() {
		return array(
			self::SORT_DESC => ZendJobQueue::SORT_DESC,
			self::SORT_ASC  => ZendJobQueue::SORT_ASC,
		);
	}
	
	/**
	 * @return array
	 */
	public function getStatusDictionary() {
		return array(
			0 => _t('Active'), // Freezed: 'Waiting'(JQ_STATUS_PENDING) becasue of bug #ZSRV-8801, temprorally to show Active in case of Waiting
			1 => _t('Waiting'), // JQ_STATUS_WAITING_PREDECESSOR
			2 => _t('Running'),
			3 => _t('Completed'),
			4 => _t('Completed'),
			5 => _t('Failed'),
			6 => _t('Failed'),
			7 => _t('Timeout'),
			8 => _t('Removed'),
			9 => _t('Scheduled'),
			10 => _t('Suspended'),
		);
	}

	/**
	 * @return array
	 */
	public function getRuleStatusDictionary() {
		return array(
				0 => _t('Active'),
				1 => _t('Waiting'),
				2 => _t('Running'),
				3 => _t('Completed'),
				4 => _t('Completed'),
				5 => _t('Failed'),
				6 => _t('Failed'),
				7 => _t('Timeout'),
				8 => _t('Removed'),
				9 => _t('Scheduled'),
				10 => _t('Suspended'),
		);
	}
	
	
	/**
	 * @return array
	 */
	public function getStatusDictionaryReversed() {
		return array(
				'Waiting' => ZendJobQueue::STATUS_PENDING,
				'Waiting' => ZendJobQueue::STATUS_WAITING_PREDECESSOR,
				'Running' => ZendJobQueue::STATUS_RUNNING,
				'Completed' => array(ZendJobQueue::STATUS_COMPLETED, ZendJobQueue::STATUS_OK),
				'Failed' => array(ZendJobQueue::STATUS_FAILED, ZendJobQueue::STATUS_LOGICALLY_FAILED),
				'Timeout' => ZendJobQueue::STATUS_TIMEOUT,
				'Removed' => ZendJobQueue::STATUS_REMOVED,
				'Scheduled' => ZendJobQueue::STATUS_SCHEDULED,
				'Suspended' => ZendJobQueue::STATUS_SUSPENDED,
		);
	}
	
	/**
	 * @return array
	 */
	public function getStatusDictionaryForFiltering() {
		$arrayForFiltering = array();
		foreach (array_keys($this->getStatusDictionaryReversed()) as $key) {
			$arrayForFiltering[$key] = $key; // identical key,value
		}

		return $arrayForFiltering;
	}
			
	/**
	 * @return array
	 */
	public function getPriorityColumnsDictionary() { // filtering uses 'ZendJobQueue::JOB_PRIORITY_LOW' and friends
		return array(
				self::PRIORITY_LOW		=> ZendJobQueue::PRIORITY_LOW,
				self::PRIORITY_NORMAL	=> ZendJobQueue::PRIORITY_NORMAL,
				self::PRIORITY_HIGH		=> ZendJobQueue::PRIORITY_HIGH,
				self::PRIORITY_URGENT	=> ZendJobQueue::PRIORITY_URGENT,
		);
	}
	
	/**
	 * @return array
	 */
	public function getPriorities() {
		$arrayPriorities = array();
		foreach (array_keys($this->getPriorityColumnsDictionary()) as $key) {
			$arrayPriorities[$key] = $key; // identical key,value
		}
		
		return $arrayPriorities;
	}

	public function getPriorityConstant($priority) {
		if (!$this->isKnownPriority($priority)) {
			throw new Exception(_t("priority '%s' is not a known priority: '%s'", array($priority, implode(',', array_keys($this->getPriorityColumnsDictionary())))));
		}
	
		$priorities = $this->getPriorityColumnsDictionary();
		return $priorities[$priority];
	}
	
	public function isKnownPriority($priority) {
		return array_key_exists($priority, $this->getPriorityColumnsDictionary());
	}
	
	public function prioritiesToDbValues(array $priorities) {
		$res = array();
		foreach ($priorities as $priority) {
			$res[] = $this->priorityToDbPriority[$this->getPriorityConstant($priority)];
		}
	
		return $res;
	}
	
	public function dbPriorityToActualPriority($priority) {
		switch ($priority) {
			case ZendJobQueue::PRIORITY_LOW:
				return ZendJobQueue::PRIORITY_URGENT;
			case ZendJobQueue::PRIORITY_NORMAL:
				return ZendJobQueue::PRIORITY_HIGH;
			case ZendJobQueue::PRIORITY_HIGH:
				return ZendJobQueue::PRIORITY_NORMAL;
			case ZendJobQueue::PRIORITY_URGENT:
				return ZendJobQueue::PRIORITY_LOW;
				
		}
	}
	
	/**
	 * @return array
	 */
	public function getPriorityDictionary() { // result returned from the db uses 'ZendJobQueue::PRIORITY_LOW' and friends
		return array(
				$this->priorityToDbPriority[ZendJobQueue::PRIORITY_LOW]		=> _t(self::PRIORITY_LOW),
				$this->priorityToDbPriority[ZendJobQueue::PRIORITY_NORMAL]	=> _t(self::PRIORITY_NORMAL),
				$this->priorityToDbPriority[ZendJobQueue::PRIORITY_HIGH]	=> _t(self::PRIORITY_HIGH),
				$this->priorityToDbPriority[ZendJobQueue::PRIORITY_URGENT]	=> _t(self::PRIORITY_URGENT),
		);
	}
	
	/**
	 * @return array
	 */
	public function getDbPriorityDictionary() { // result returned from the db uses 'ZendJobQueue::PRIORITY_LOW' and friends
		return array(
				$this->priorityToDbPriority[ZendJobQueue::PRIORITY_URGENT]		=> _t(self::PRIORITY_LOW),
				$this->priorityToDbPriority[ZendJobQueue::PRIORITY_HIGH]	=> _t(self::PRIORITY_NORMAL),
				$this->priorityToDbPriority[ZendJobQueue::PRIORITY_NORMAL]	=> _t(self::PRIORITY_HIGH),
				$this->priorityToDbPriority[ZendJobQueue::PRIORITY_LOW]	=> _t(self::PRIORITY_URGENT),
		);
	}
	
	public function getSortConstant($sortName) {
		$sortColumns = $this->getSortColumnsDictionary();
		if (!isset($sortColumns[$sortName])) {
			throw new Exception(_t("sort column '%s' not found", array($sortName)));
		}
	
		return $sortColumns[$sortName];
	}
	
	public function getSortDirectionConstant($sortDirection) {
		$sortDirectionColumns = $this->getSortDirectionColumnsDictionary();
		if (!isset($sortDirectionColumns[$sortDirection])) {
			throw new Exception(_t("sort Direction '%s' not found", array($sortDirection)));
		}
	
		return $sortDirectionColumns[$sortDirection];
	}

	public function getJQTimeRange() {
		return array (
				'all' => _t ( 'All' ),
				'day' => _t ( '24 Hours' ),
				'week' => _t ( 'Week' ),
				'month' => _t ( 'Month' ),
		);
	}
	
	public function getTimeRanges() {
		$timeRangesArray = array('all' => array());
		$timeRangesArray['month'] = array(date('m/d/Y H:i', strtotime('-1 month')), date('m/d/Y H:i'), strtotime('-1 month'), time());
		$timeRangesArray['week'] = array(date('m/d/Y H:i', time() - 7*24*60*60) , date('m/d/Y H:i'), time() - 7*24*60*60, time());
		$timeRangesArray['day'] = array(date('m/d/Y H:i', time() - 24*60*60), date('m/d/Y H:i'), time() - 24*60*60, time());
		
		return $timeRangesArray;
	}
	
}