<?php
namespace MonitorUi\Model;

use ZendServer\PHPUnit\TestCase;

use PHPUnit_Framework_TestCase,
	MonitorUi\Filter\Container,
	ZendServer\Exception;
use ZendServer\Set;
use Issue\Db\Mapper;
use Deployment\IdentityFilterSimple;

require_once 'tests/bootstrap.php';

class ModelTest extends TestCase
{
	public function testGetIssuesCount() {
		$model = new Model();
		
		$tableGateway = $this->getMock('Zend\Db\TableGateway\TableGateway', array(), array(), '', false);
		$tableGateway->expects($this->any())
		->method('selectWith')
		->will($this->returnValue(array(array('total' => 7))));
		
		$tableGateway->expects($this->any())
		->method('getTable')
		->will($this->returnValue('issues'));
		
		$identityFilter = $this->getMock('Deployment\IdentityFilterSimple');
		$identityFilter->expects($this->once())->method('filterAppIds')->will($this->returnValue(array(-1)));
		$identityFilter->expects($this->once())->method('setAddGlobalAppId');
		
		$mapper = new Mapper($tableGateway);
		$mapper->setIdentityFilter($identityFilter);
		$model->setIssueMapper($mapper);
		
		self::assertEquals(7, $model->getIssuesCount(array()));
	}
	
	public function testGetIssuesCountException() {
		$model = new Model();
		
		$tableGateway = $this->getMock('Zend\Db\TableGateway\TableGateway', array(), array(), '', false);
		$tableGateway->expects($this->any())
		->method('selectWith')
		->will($this->throwException(new Exception('This is some exception', 5)));
		
		$tableGateway->expects($this->any())
		->method('getTable')
		->will($this->returnValue('issues'));
		
		$identityFilter = $this->getMock('Deployment\IdentityFilterSimple');
		$identityFilter->expects($this->once())->method('filterAppIds')->will($this->returnValue(array(-1)));
		$identityFilter->expects($this->once())->method('setAddGlobalAppId');
		
		$mapper = new Mapper($tableGateway);
		$mapper->setIdentityFilter($identityFilter);
		$model->setIssueMapper($mapper);
		
		try {
			$model->getIssuesCount(array());
		} catch (Exception $e) {
			self::assertContains('This is some exception', $e->getMessage());
		}
	}
	
	public function testGetIssues() {
		$model = new Model();

		$expectedLimit = array(
				ZM_LIMITS_FIRST_ITEM	=> 2,
				ZM_LIMITS_NUM_OF_ITEMS	=> 7
		);
		
		$expectedSortby = array(
				ZM_SORTBY_DATA		=> ZM_DATA_LAST_TIMESTAMP,
				ZM_SORTBY_ORDER_ASC	=> true
		);
		
		
		$tableGateway = $this->getMock('Zend\Db\TableGateway\TableGateway', array(), array(), '', false);
		$tableGateway->expects($this->any())
		->method('selectWith')
		->will($this->returnValue(array()));
		
		$tableGateway->expects($this->any())
		->method('getTable')
		->will($this->returnValue('issues'));
		
		$identityFilter = $this->getMock('Deployment\IdentityFilterSimple');
		$identityFilter->expects($this->once())->method('filterAppIds')->will($this->returnValue(array(-1)));
		$identityFilter->expects($this->once())->method('setAddGlobalAppId');
		
		$mapper = new Mapper($tableGateway);
		$mapper->setIdentityFilter($identityFilter);
		$model->setIssueMapper($mapper);
		
		
		// ok if no exception thrown
		$model->getIssues(array(), 7, 2, 'date', 'ASC');
	}
}