<?php

namespace StudioIntegration\Controller;

use ZendServer\Mvc\Controller\ActionController;
use Zend\Stdlib\Parameters;

class IndexController extends ActionController
{
	public function indexAction() {
		$mapper = $this->getLocator()->get('StudioIntegration\Mapper');
		$config = $mapper->getConfiguration();
		
		$directivesMapper = $this->getLocator()->get('Configuration\MapperDirectives');
		$debuggerAllowed = $directivesMapper->getDirectiveValue('zend_debugger.allow_hosts');
		$debuggerDenied = $directivesMapper->getDirectiveValue('zend_debugger.deny_hosts');
		
		$extensionsMapper = $this->getLocator()->get('Configuration\MapperExtensions');
		$debuggerLoaded = $extensionsMapper->isExtensionLoaded('Zend Debugger');
		
		$studioConfigForm = $this->getLocator()->get('StudioIntegration\Form\Configuration');
		$studioConfigForm->setData($studioConfigForm->getHydrator()->extract($config));
		$studioConfigForm->setObject($config);
			
		$studioHostsLists = $this->getLocator()->get('StudioIntegration\Form\HostsList');
		$studioHostsLists->setData(
				array('studioAllowedHostsList' => $debuggerAllowed,
						'studioDeniedHostsList' => $debuggerDenied));
		
		// set the browser ip as default studio host and also propose to add it to allowed host
		$browserHost = $this->getBrowserRemoterAddress();
		if ($browserHost) {
			$studioConfigForm->get('studioHost')->setOptions(array('default' => $browserHost));
			$studioHostsLists->get('studioAllowedHostsList')->setOptions(array('default' => $browserHost));
		}
		
		return array('studioConfig' => $config, 'studioHostsLists' => $studioHostsLists,
				'debuggerLoaded' => $debuggerLoaded, 'studioConfigForm' => $studioConfigForm);
	}
	
	public function exportIssueByEventsGroupAction() {
	    $request = $this->getRequest(); /* @var $request \Zend\Http\PhpEnvironment\Request */
	    $request->setQuery(new Parameters($request->getQuery()->toArray()));
	    $this->forward()->dispatch('StudioWebApi-1_3', array('action' => 'monitorExportIssueByEventsGroup'));
	    
	    return $this->getResponse();
	}
	
	/**
	 * @return string
	 */
	private function getBrowserRemoterAddress() {
		$browserRemoteAddress = $_SERVER['REMOTE_ADDR'];
		// Special case of IPv6 local loopback
		if ('::1' == $browserRemoteAddress) {
			$browserRemoteAddress = '127.0.0.1';
		}
		// Special case of IPv6 which holds an IPv4 address (e.g. ::ffff:10.1.1.1) bug #30319
		if (0 === strpos($browserRemoteAddress, '::ffff:')) {
			$browserRemoteAddress = substr($browserRemoteAddress, 7);
		}
		return $browserRemoteAddress;
	}
}
