<?php

namespace StudioIntegration\Controller;

use Audit\Db\ProgressMapper;

use Audit\Db\Mapper;

use Audit\AuditTypeInterface;

use StudioIntegration\exportIssue;

use ZendServer\Mvc\Controller\WebAPIActionController;

use Zend\Mvc\Controller\ActionController,
	Application\Module,
	WebAPI,
	Zend\Http\PhpEnvironment\Request as PhpRequest,
	ZendServer\FS\FS,
	ZendServer\Log\Log,
	ZendServer\Exception as ZSException,
	Zend\Validator,
	StudioIntegration\Model as StudioIntegrationModel,
	StudioIntegration\Configuration as StudioIntegrationConfiguration,
	StudioIntegration\MonitorIssueGroupData as StudioMonitorIssueGroupData;

use Zend\View\Model\ViewModel;

class WebAPIController extends WebAPIActionController
{
	public function saveAlternateServerAction() {
		
		$this->isMethodPost();
		
		$params = $this->getParameters(array('debugServer', 'alternateServer'));
	
		if ('alternate' == $params['debugServer']) {
			$alternateServer = $params['alternateServer'];
			
			$hostValidator = new \Zend\Validator\Hostname(\Zend\Validator\Hostname::ALLOW_ALL); // allow flexibility in setting the logical name
			if (!$hostValidator->isValid($alternateServer)) {
				throw new WebAPI\Exception(_t("'%s' is not a valid alternate server. Enter a valid IP or server name", array($alternateServer) ), WebAPI\Exception::INVALID_PARAMETER);
			}
			// set alternate debug server
			$this->setStudioIntegrationAlternateServer($alternateServer);
		} else {
			// set an empty debug server for local
			$this->setStudioIntegrationAlternateServer('');
		}
	
		return array();
	}
	
	public function studioStartDebugAction() {	
		$viewModel = new ViewModel();
		$viewModel->setTemplate('studio-integration/web-api/studio');
		try {
			$this->isMethodPost();
			$params = $this->getParameters(array('noRemote' => 'TRUE', 'overrideHost'=>null, 'debug_host' => null, 'debug_port' => null, 'use_ssl' => null));
			$this->validateMandatoryParameters($params, array('eventsGroupId'));
			$this->validateInteger($params['eventsGroupId'], 'eventsGroupId');
			$this->validateBoolean($params['noRemote'], 'noRemote');
			if (!is_null($params['overrideHost'])) $this->validateHost($params['overrideHost'], 'overrideHost');
		} catch (\Exception $e) {
			$this->handleException($e, 'Input validation failed');
		}
	
		set_time_limit(StudioIntegrationModel::REQUEST_TIME_LIMIT);// this request may take longer than a regular request, an extended timeout is needed
	
		try {
			$monitorUiModel = $this->getLocator()->get('MonitorUi\Model\Model'); /* @var $monitorUiModel \MonitorUi\Model\Model */
			$event = $monitorUiModel->getEventGroupData($params['eventsGroupId']);
			$issueId = $event->getIssueId();
				
			// @todo - validate issue can be debugged
				
			$studio = StudioIntegrationConfiguration::getInstance();
	
			if (is_null($params['overrideHost'])) {
				$studio->setConfiguration($studio->getCurrentHost(), StudioIntegrationModel::DEBUGGER_DEFAULT_PORT, '0');
			} else {
				$studio->setConfiguration($params['overrideHost'], StudioIntegrationModel::DEBUGGER_DEFAULT_PORT, '0');
				$studio->setAutoDetect(true);
			}
				
			$model = $this->getLocator()->get('StudioIntegrationModel'); /* @var $model \StudioIntegration\Model */				
			$client	= $model->getDebugClient(new StudioMonitorIssueGroupData($issueId, $params['eventsGroupId'], $monitorUiModel));				
			$client->addDebuggerParam('no_remote', $params['noRemote'] == 'TRUE');
			if ($params['debug_host']) {
				$client->addDebuggerParam('debug_host', $params['debug_host']);
				$client->addDebuggerParam('debug_port', $params['debug_port']);
				$client->addDebuggerParam('use_ssl', $params['use_ssl']);
			}
			$audit = $this->auditMessage(Mapper::AUDIT_STUDIO_DEBUG, ProgressMapper::AUDIT_PROGRESS_STARTED, array($params));
			$model->connect($client);
				
			$this->setHttpResponseCode('200', 'OK');
			$this->auditMessageProgress($audit->getAuditId(), ProgressMapper::AUDIT_PROGRESS_ENDED_SUCCESFULLY);
			$viewModel->setVariables(array('content' => 'Debug session completed successfully', 'success' => '1'));
		} catch (\Exception $e) {
			if (isset($audit)) {
				$this->auditMessageProgress($audit->getAuditId(), ProgressMapper::AUDIT_PROGRESS_ENDED_FAILED);
			} else {
				$this->auditMessage(Mapper::AUDIT_STUDIO_DEBUG, ProgressMapper::AUDIT_PROGRESS_ENDED_FAILED, array($params));
			}
			$this->auditMessage(Mapper::AUDIT_STUDIO_DEBUG, ProgressMapper::AUDIT_PROGRESS_ENDED_FAILED, array($params));
			Log::err(_t('Debug session failed : %s. full trace: %s', array($e->getMessage(), $e->getTraceAsString())));
			$viewModel->setVariables(array('content' => 'Debug session failed: ' . $e->getMessage(), 'success' => '0'));
		}
		
		return $viewModel;
	}
	

	public function studioStartProfileAction() {

		$viewModel = new ViewModel();
		$viewModel->setTemplate('studio-integration/web-api/studio');
		try {
			$this->isMethodPost();
			$params = $this->getParameters(array('overrideHost'=>null, 'debug_host' => null, 'debug_port' => StudioIntegrationModel::DEBUGGER_DEFAULT_PORT, 'use_ssl' => '0'));
			$this->validateMandatoryParameters($params, array('eventsGroupId'));
			$this->validateInteger($params['eventsGroupId'], 'eventsGroupId');
			if (!is_null($params['overrideHost'])) $this->validateHost($params['overrideHost'], 'overrideHost');
		} catch (\Exception $e) {
			$this->handleException($e, 'Input validation failed');
		}
		
		set_time_limit(StudioIntegrationModel::REQUEST_TIME_LIMIT);// this request may take longer than a regular request, an extended timeout is needed		
		
		try {
			$monitorUiModel = $this->getLocator()->get('MonitorUi\Model\Model'); /* @var $monitorUiModel \MonitorUi\Model\Model */
			$event = $monitorUiModel->getEventGroupData($params['eventsGroupId']);
			$issueId = $event->getIssueId();
		
			// @todo - validate issue can be debugged
		
			$studio = StudioIntegrationConfiguration::getInstance();
		
			if (is_null($params['overrideHost'])) {
				$studio->setConfiguration($studio->getCurrentHost(), StudioIntegrationModel::DEBUGGER_DEFAULT_PORT, '0');
			} else {
				$studio->setConfiguration($params['overrideHost'], StudioIntegrationModel::DEBUGGER_DEFAULT_PORT, '0');
				$studio->setAutoDetect(true);
			}
		
			if ($params['debug_host']) {
				$studio->setConfiguration($params['debug_host'], $params['debug_port'], $params['use_ssl']);
			}
			
			$model = $this->getLocator()->get('StudioIntegrationModel'); /* @var $model \StudioIntegration\Model */		
			$client	= $model->getProfileClient(new StudioMonitorIssueGroupData($issueId, $params['eventsGroupId'], $monitorUiModel));
			$audit = $this->auditMessage(Mapper::AUDIT_STUDIO_PROFILE, ProgressMapper::AUDIT_PROGRESS_STARTED, array($params));
			$model->connect($client);
		
			$this->setHttpResponseCode('200', 'OK');
			$this->auditMessageProgress($audit->getAuditId(), ProgressMapper::AUDIT_PROGRESS_ENDED_SUCCESFULLY);
			$viewModel->setVariables(array('content' => 'Profile session completed successfully', 'success' => '1'));
		} catch (\Exception $e) {
			if (isset($audit)) {
				$this->auditMessageProgress($audit->getAuditId(), ProgressMapper::AUDIT_PROGRESS_ENDED_FAILED);
			} else {
				$this->auditMessage(Mapper::AUDIT_STUDIO_PROFILE, ProgressMapper::AUDIT_PROGRESS_ENDED_FAILED, array($params));
			}
			Log::err(_t('Profile session failed : %s. full trace: %s', array($e->getMessage(), $e->getTraceAsString())));
			$viewModel->setVariables(array('content' => 'Profile session failed: ' . $e->getMessage(), 'success' => '0'));
		}
		return $viewModel;
	}	
	
	public function studioShowSourceAction() {
		$viewModel = new ViewModel();
		$viewModel->setTemplate('studio-integration/web-api/studio');
		try {
			$this->isMethodPost();
			$params = $this->getParameters(array('overrideHost'=>null, 'debug_host' => null, 'debug_port' => StudioIntegrationModel::DEBUGGER_DEFAULT_PORT, 'use_ssl' => '0'));
			$this->validateMandatoryParameters($params, array('eventsGroupId'));
			$this->validateInteger($params['eventsGroupId'], 'eventsGroupId');
			if (!is_null($params['overrideHost'])) $this->validateHost($params['overrideHost'], 'overrideHost');
		} catch (\Exception $e) {
			$this->handleException($e, 'Input validation failed');
		}
		
		set_time_limit(StudioIntegrationModel::REQUEST_TIME_LIMIT);// this request may take longer than a regular request, an extended timeout is needed		
		
		try {
			$monitorUiModel = $this->getLocator()->get('MonitorUi\Model\Model'); /* @var $monitorUiModel \MonitorUi\Model\Model */
			$event = $monitorUiModel->getEventGroupData($params['eventsGroupId']);
			$issueId = $event->getIssueId();
		
			// @todo - validate issue can be debugged
		
			$studio = StudioIntegrationConfiguration::getInstance();
		
			if (is_null($params['overrideHost'])) {
				$studio->setConfiguration($studio->getCurrentHost(), StudioIntegrationModel::DEBUGGER_DEFAULT_PORT, '0');
			} else {
				$studio->setConfiguration($params['overrideHost'], StudioIntegrationModel::DEBUGGER_DEFAULT_PORT, '0');
				$studio->setAutoDetect(true);
			}
		
			if ($params['debug_host']) {
				$studio->setConfiguration($params['debug_host'], $params['debug_port'], $params['use_ssl']);
			}
			
			$model = $this->getLocator()->get('StudioIntegrationModel'); /* @var $model \StudioIntegration\Model */		
			$client	= $model->getShowSourceClient(new StudioMonitorIssueGroupData($issueId, $params['eventsGroupId'], $monitorUiModel));
			$audit = $this->auditMessage(Mapper::AUDIT_STUDIO_SOURCE, ProgressMapper::AUDIT_PROGRESS_STARTED, array($params));
			$model->connect($client);
		
			$this->setHttpResponseCode('200', 'OK');
			$this->auditMessageProgress($audit->getAuditId(), ProgressMapper::AUDIT_PROGRESS_ENDED_SUCCESFULLY);
			$viewModel->setVariables(array('content' => 'Show source session completed successfully', 'success' => '1'));
		} catch (\Exception $e) {
			if (isset($audit)) {
				$this->auditMessageProgress($audit->getAuditId(), ProgressMapper::AUDIT_PROGRESS_ENDED_FAILED);
			} else {
				$this->auditMessage(Mapper::AUDIT_STUDIO_SOURCE, ProgressMapper::AUDIT_PROGRESS_ENDED_FAILED, array($params));
			}
			Log::err(_t('Show source session failed : %s. full trace: %s', array($e->getMessage(), $e->getTraceAsString())));
			$viewModel->setVariables(array('content' => 'Show source session failed: ' . $e->getMessage(), 'success' => '0'));
		}
		return $viewModel;
	}
	
	public function monitorExportIssueByEventsGroupAction() {
		try {
			$this->isMethodGet();
			$params = $this->getParameters();
			$this->validateMandatoryParameters($params, array('eventsGroupId'));
			$this->validateInteger($params['eventsGroupId'], 'eventsGroupId');
			if (!is_null($params['overrideHost'])) $this->validateHost($params['overrideHost'], 'overrideHost');
		} catch (\Exception $e) {
			$this->handleException($e, 'Input validation failed');
		}
	
		try {
			$monitorUiModel = $this->getLocator()->get('MonitorUi\Model\Model'); /* @var $monitorUiModel \MonitorUi\Model\Model */
			$event = $monitorUiModel->getEventGroupData($params['eventsGroupId']);
			$issueId = $event->getIssueId();
			
			$exportIssue = $this->getLocator()->get('StudioIntegration\exportIssue'); /* @var $exportIssue \StudioIntegration\exportIssue */
			$traceFilepath = $event->getCodeTracingPath();
			
			$zipArchivePath = $exportIssue->createFile($event->getIssueId(), $params['eventsGroupId'], $traceFilepath);
			$zipArchive = FS::getFileObject($zipArchivePath);
			
			// prepare the environment for a file download		
			$this->setHttpResponseCode('200', 'OK');
			$response = $this->getResponse(); /* @var $response \Zend\Http\PhpEnvironment\Response */			
			$response->getHeaders()->addHeaders(array(
					'Content-Length'	=> $zipArchive->getSize(),
					'Content-Type'		=>  'application/vnd.zend.eventexport',
					'Content-Disposition'	=>  'attachment; filename="' . basename($zipArchivePath) . '"',
			));
			$response->sendHeaders();
			$response->setContent($zipArchive->readAll()); // @todo - used to be appendBody()
			$response->send();
		} catch (ZSException $e) {
			Log::logException('Error exporting issue details', $e);
			throw new \WebAPI\Exception($e->getMessage(), \WebAPI\Exception::INTERNAL_SERVER_ERROR, $e);
		}
	
	}
	
	/**
	 *
	 * @return \StudioIntegration\Model
	 */
	protected function getModel() {
		return $this->getLocator()->get('StudioIntegrationModel');
	}
	
	public function studioStartDebugModeAction() {
		
		$started = true;
		$audit = $this->auditMessage(AuditTypeInterface::AUDIT_STUDIO_DEBUG_MODE_START, ProgressMapper::AUDIT_PROGRESS_REQUESTED); /* @var $audit \Audit\Container */
		
		try {
			$this->isMethodPost();
			$params = $this->getParameters ( array (
					'filters' => array(),
					'options' => array()
					
			) );
			
			$this->validateMandatoryParameters($params, array('filters'));
			
			$this->validateArray($params['options'], 'options');
			$this->validateArray($params['filters'], 'filters');
						
		} catch (\Exception $e) {
			$started = false;
			$this->auditMessageProgress($audit->getAuditId(), ProgressMapper::AUDIT_PROGRESS_ENDED_FAILED);
			
			$this->handleException($e, 'Input validation failed');
		}		
			

		$this->auditMessageProgress($audit->getAuditId(), ProgressMapper::AUDIT_PROGRESS_STARTED);
		
		try {
			$this->getModel()->debuggerStartDebugMode($params['options'], $params['filters']);
			$this->auditMessageProgress($audit->getAuditId(), ProgressMapper::AUDIT_PROGRESS_ENDED_SUCCESFULLY);

		} catch (\Exception $ex) {
			$started = false;
			$this->auditMessageProgress($audit->getAuditId(), ProgressMapper::AUDIT_PROGRESS_ENDED_FAILED);
			throw $ex;
		}

		
		return array ('started' => $started);
	}	
	
	public function studioStopDebugModeAction() {
	
		$started = false;
		
		$audit = $this->auditMessage(AuditTypeInterface::AUDIT_STUDIO_DEBUG_MODE_STOP, ProgressMapper::AUDIT_PROGRESS_REQUESTED); /* @var $audit \Audit\Container */
		
		try {
			$this->isMethodPost();
			$params = $this->getParameters ( array () );
		} catch (\Exception $e) {
			$this->auditMessageProgress($audit->getAuditId(), ProgressMapper::AUDIT_PROGRESS_ENDED_FAILED);
			$started = true;
			$this->handleException($e, 'Input validation failed');
		}
			
		$this->auditMessageProgress($audit->getAuditId(), ProgressMapper::AUDIT_PROGRESS_STARTED);
	
		try {
			$this->getModel()->debuggerStopDebugMode();
			$this->auditMessageProgress($audit->getAuditId(), ProgressMapper::AUDIT_PROGRESS_ENDED_SUCCESFULLY);
		} catch (\Exception $ex) {
			$started = true;
			$this->auditMessageProgress($audit->getAuditId(), ProgressMapper::AUDIT_PROGRESS_ENDED_FAILED);
			throw $ex;
		}		
		
		return array ('started' => $started);
	}
	
	public function studioIsDebugModeEnabledAction() {	
		$started = false;
	
		try {
			$this->isMethodGet();
			$params = $this->getParameters ( array () );
		} catch (\Exception $e) {
			$this->handleException($e, 'Input validation failed');
		}
			
		try {
			$started = $this->getModel()->debuggerIsDebugModeEnabled();
		} catch (\Exception $ex) {
			throw $ex;
		}
	
		return array ('started' => $started);
	}
	

	/**
	 * Alternate server host
	 * @param string $host
	 */
	private function setStudioIntegrationAlternateServer($host) {
		$directives = array('alternateDebugServer' => $host);
		try { // TODO - use special audit type rather than AUDIT_DIRECTIVES_MODIFIED
			$auditMessage = $this->auditMessage(Mapper::AUDIT_DIRECTIVES_MODIFIED,	ProgressMapper::AUDIT_PROGRESS_REQUESTED, $directives); /* @var $auditMessage \Audit\Container */
			$this->getTasksMapper()->setGuiDirectives($directives, $auditMessage->getAuditId());
		} catch (\Exception $e) {
			$this->auditMessageProgress($auditMessage->getAuditId(), ProgressMapper::AUDIT_PROGRESS_ENDED_FAILED, array(array('errorMessage' => $e->getMessage())));
			Log::err("Set UI directives failed: " . $e->getMessage());
			throw new \WebAPI\Exception(_t('Setting UI directives failed: %s', array($e->getMessage())), \WebAPI\Exception::INTERNAL_SERVER_ERROR);
		}
	}
}
