<?php

namespace ZendServer;

use ZendServer\License\ChangeListener;

use Zend\EventManager\EventInterface;

use Zend\ModuleManager\Feature\BootstrapListenerInterface;

use ZendServer\Authentication\Adapter\DbTable;

use ZendServer\Authentication\Adapter\Ldap;

use Zend\ModuleManager\Feature\ServiceProviderInterface;

use Zend\ModuleManager\Feature\AutoloaderProviderInterface as AutoloaderProvider;

use ZendServer\Configuration\Container as ConfigurationContainer;

use Application\Module as appModule;
use ZendServer\Permissions\AclQuery;
use ZendServer\Log\Log;
use Zend\Json\Json;
use Configuration\DdMapper;
use ZendServer\Configuration\Manager;
require_once (__DIR__ . DIRECTORY_SEPARATOR . 'functions.php');// require the global convenience functions
class Module implements AutoloaderProvider, ServiceProviderInterface
{
	
	public function getAutoloaderConfig()
	{
		return array(
				'Zend\Loader\StandardAutoloader' => array(
						'namespaces' => array(
								__NAMESPACE__ => __DIR__ . '/src/' . __NAMESPACE__,
						),
				),
		);
	}
	
	public function getConfig() {
		return include __DIR__ . '/config/module.config.php';
	}
	
	/*
	 * (non-PHPdoc)
	* @see \Zend\ModuleManager\Feature\ServiceProviderInterface::getServiceConfig()
	*/
	public function getServiceConfig() {
		$module = $this;
		return array(
			'aliases' => array(
				'AuthAdapterLdap' => 'ZendServer\Authentication\Adapter\Ldap',
				'AuthAdapterDbTable' => 'ZendServer\Authentication\Adapter\DbTable',
				'ZendServerAcl' => 'ZendServer\Permissions\AclQuery',
			),
			'invokables' => array(
			),
			'factories' => array(
				'ZendServer\Permissions\AclQuery' => function($sm) {
					$acl = new AclQuery();
					if ($sm->has('ZendServerIdentityAcl')) {
						$acl->setAcl($sm->get('ZendServerIdentityAcl'));
					}
					if ($sm->has('ZendServerLicenseAcl')) {
						$acl->setEditionAcl($sm->get('ZendServerLicenseAcl'));
					}
					return $acl;
				},
				'ViewManager' => 'ZendServer\Mvc\Service\ViewManagerFactory',
					
				'ZendServer\Configuration\Container'  => function($sm) {
					$adapter = new ConfigurationContainer();
					$adapter->setDirectivesMapper($sm->get('Configuration\MapperDirectives'));
					$adapter->setExtensionsMapper($sm->get('Configuration\MapperExtensions'));
					$adapter->setLibrariesMapper($sm->get('DeploymentLibrary\Mapper'));
					$zeMapFile = get_cfg_var('zend.install_dir') . DIRECTORY_SEPARATOR . 'share/zend_extensions_map.json';
					$content = file_get_contents($zeMapFile, Json::TYPE_OBJECT);
					$adapter->setDdMapper(new DdMapper($content));
					$adapter->setManager(new Manager());
					return $adapter;
				},
				'ZendServer\Authentication\Adapter\Ldap' => function($sm) {
					$adapter = new Ldap(array('zend_server_authentication' => appModule::config('zend_server_authentication')->toArray()));
					$adapter->setMapperGroups($sm->get('Acl\Db\MapperGroups'));
					$adapter->setGroupsAttribute(appModule::config('authentication','groupsAttribute'));
					return $adapter;
				},
				'ZendServer\Authentication\Adapter\DbTable' => function($sm) {
					$dbAdapter = $sm->get('guiDbAdapter');
					$adapter = new DbTable($dbAdapter, 'GUI_USERS', 'name', 'password');
					return $adapter;
				},
				'ZendServer\Filter\Mapper' => function($sm) {
				    $mapper = $sm->get('filterMapper');
				    $mapper->setTableGateway($sm->get('guiFilters_tg'));
				    return $mapper;
				},
			)
		);
	}
}
