<?php

namespace ZendServer\View\Helper\Form;

use Zend\View\Helper\AbstractHelper,
	Zend\Form\Form as ZendForm,
	Zend\Form\Element,
	Zend\Form\Fieldset,
	ZendServer\Exception;

class Settings extends AbstractHelper {
	/**
	 * Get a filesize label.
	 *
	 * @param ZendForm $form
	 * @return string
	 */
	public function __invoke($form = null, $renderer = null) {
		if (is_null($form)) {
			return $this;
		}

		if (is_null($renderer)) {
			$renderer = $this->getView()->zendFormSettings();
		}
		
		$result = '';
		
		// Render the opening tag
		$result .= $this->getView()->form()->openTag($form);
		
		$result .= $renderer->openTag();
		
		foreach ($form->getElements() as $element) { /* @var $element Element */
			$result .= $renderer->element($element);
		}
		
		$result .= $renderer->closeTag();
		
		$result .= $this->getView()->form()->closeTag($form);
		
		$formId = $form->getName();
		$title = $form->getLabel();
		$description = '';
		if ($form->hasAttribute('description')) {
			$description = $form->getAttribute('description');
		}
		
		return <<<FORM
<div class="settings-wrapper" id="{$formId}" onsubmit="return false;" >
	<h2>{$title}</h2>
	<div class="settings-desc">
		{$description}
	</div>
	{$result}
</div>
FORM;
	}
	
	/**
	 * @param ZendForm $form
	 * @return array
	 */
	public function getElements($form) {
		$elements = $form->getElements();
		foreach ($form->getFieldsets() as $fieldSet) { /* @var $fieldSet Fieldset */
			$elements = array_merge($elements, $fieldSet->getElements());
		}
		return $elements;
	}
}