<?php

abstract class ZSComponentUpgrader {
	
		
	abstract protected function getSqliteDbName($version = null);
	
	abstract protected function getSqliteSqlFilenames();
	abstract protected function getMysqlSqlFilenames();
	
	protected function createCleanSqliteDatabase() {
		message("Creating new SQLITE database at " . $this->getSqliteDbPath());
		
		$pdo = $this->getSqliteDbh();
		
		foreach ($this->getSqliteSqlFilenames() as $sqlFile) {
			$sql = file_get_contents($this->getSqlsDir() . "/" . $sqlFile);
			$pdo->exec($sql);
		}	

		chmod($this->getSqliteDbPath(), 0664);
	} 
	
	protected function isClusterUpgrade() {
		$dbSettings = $this->getDbSettings();
		if ($dbSettings && $dbSettings->isMysql()) {
			message("Upgrade from mysql db detected");
			return true;
		}
		
		$dbSettings601 = $this->getDbSettings("6.0.1");
		if ($dbSettings601 && $dbSettings601->isMysql()) {
			message("Upgrade from 6.0.1 cluster detected");
			return true;
		}
		
		$dbSettings6 = $this->getDbSettings("6.0.0");
		if ($dbSettings6 && $dbSettings6->isMysql()) {
			message("Upgrade from 6.0.0 cluster detected");
			return true;
		}
		$dbSettings56 = $this->getDbSettings("5.6.0");
		if ($dbSettings56 && $dbSettings56->isMysql()) {
			message("Upgrade from 5.6.0 cluster detected");
			return true;
		}
		
		return false;		
	}
	
	protected function createCleanMysqlDatabase() {
		message("Creating MySQL tables");
	
		$pdo = null;
		if ($this->getDbSettings()->isMysql()) {
			$pdo = $this->getMySqlDbh();
		} else if ($this->getDbSettings("6.0.1")) {
			$pdo = $this->getMySqlDbh("6.0.1");
			message("Using mysql settings from 6.0.1");
		} else if ($this->getDbSettings("6.0.0")) {
			$pdo = $this->getMySqlDbh("6.0.0");
			message("Using mysql settings from 6.0");
		} else if ($this->getDbSettings("5.6.0")) {
			$settings = $this->getDbSettings("5.6.0");
			$settings->setDbName("ZendServer");
			message("Using mysql settings from 5.6");
			$pdo = $this->getMySqlDbhFromSettings($settings);
		}
		
		if (!$pdo) {
			ERROR("Cannot create mysql connection");
			return;
		}
		
		$pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
	
			
		foreach ($this->getMysqlSqlFilenames() as $sqlFile) {
			$sql = file_get_contents($this->getSqlsDir() . "/" . $sqlFile);
			
			try {
			
				$pdo->exec($sql);
				
			} catch (Exception $ex) {
				if (!strstr($ex->getMessage(), "already exists") && !stristr($ex->getMessage(), "duplicate")) {
					throw $ex;
				}
			}
		}		
	}
	
	/**
	 * 
	 * @return PDO
	 */
	protected function getSqliteDbh($version = null, $createIfNotExists = true) {
		if (!$createIfNotExists && !file_exists($this->getSqliteDbPath($version))) {
			return null;
		}
		
		$pdo = new PDO("sqlite:" . $this->getSqliteDbPath($version));
		$pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_WARNING);
		
		return $pdo;
	}
	
	protected function getUpgradeFromVersion() {
		if (isset($_GET['fromVersion'])) {
			return $_GET['fromVersion'];
		} else if (is_dir($_GET['zsDir'] . "/etc-6.1.0")) {
            return "6.1.0";
        } else 	if (is_dir($_GET['zsDir'] . "/etc-6.0.1")) {
            return "6.0.1";
		} else 	if (is_dir($_GET['zsDir'] . "/etc-6.0.0")) {
			return "6.0.0";
		} else if (is_dir($_GET['zsDir'] . "/etc-5.6.0")) {
			return "5.6.0";
		}  
		
		return false;		
	}
	
	protected function getDbSettings($version = null) {
		if (!$version || ($version == '6.1.0')) {
			return ZSDbSettings::fromIni($_GET['zsDir'] . "/etc/zend_database.ini");
		} else if ($version[0] == "6") {
			return ZSDbSettings::fromIni($_GET['zsDir'] . "/etc-$version/zend_database.ini");
		} else {
			return ZSDbSettings::fromIni($_GET['zsDir'] . "/etc-$version/monitor_node.ini");
		}
	}
	
	/**
	 *
	 * @return PDO
	 */
	protected function getMySqlDbh($version = null) {
		
		$dbSettings = $this->getDbSettings($version);
		
		if (!$dbSettings || !$dbSettings->isMysql()) {
			return null;
		}
		
		return $this->getMySqlDbhFromSettings($dbSettings);
	}   
	
	protected function getFromVersion() {
		if (isset($_GET['fromVersion'])) {
			return $_GET['fromVersion'];
		} else {
			return "";
		}
	}
	
	protected function getToVersion() {
		if (isset($_GET['toVersion'])) {
			return $_GET['toVersion'];
		} else {
			return "";
		}
	}
	
	protected function getCurrentSchemaVersionFromMySQLDB($dbh) {
		
		$res = $dbh->query("SELECT property_value FROM schema_properties where property = 'ZS_VERSION'")->fetchAll();
		foreach ($res as $row) {
			return $row['property_value'];
		}	

		return false;
	}
	
	protected function getMysqlSchemaVersion($dbh) {
	
		$res = $dbh->query("SELECT property_value FROM schema_properties where property = 'ZS_VERSION'")->fetchAll();
		foreach ($res as $row) {
			return $row['property_value'];
		}
	
		return false;
	}
	
	/**
	 * @param $dbh PDO
	 */
	protected function isTableExists($dbh, $tableName) {
		
		$origErrMode = $dbh->getAttribute(PDO::ATTR_ERRMODE);
		$ret = false;
		
		try {
			$isSqlite = false;
			$dbh->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
			if (stristr($dbh->getAttribute(PDO::ATTR_DRIVER_NAME), "sqlite")) {
				$isSqlite = true;
				$sql_query = "select count(*) from sqlite_master where name = '" . $tableName . "' and type = 'table'";
				$res = $dbh->query( $sql_query );
								 
			} else {
				$res = $dbh->query("show tables");
				
			}
			
			foreach ($res->fetchAll() as $row) {
				if ( $isSqlite ) {
					return ($row[0] == 1);
					
				} else {
					// MySQL
					if ($row[0] == $tableName . "a") {
						$ret = true;
						break;
					}
				}
			}			
			
		} catch (Exception $ex) {
			error($ex->getMessage());
		}
		
		$dbh->setAttribute(PDO::ATTR_ERRMODE, $origErrMode);

		message ("Table $tableName exists - " . ($ret?"yes":"no"));
		return $ret;
	}
	
	protected function getUpgradedMySqlDbh() {
	
		$dbSettings = $this->getDbSettings();
		if ($dbSettings && $dbSettings->isMysql()){
			message("Using db " . $dbSettings->getDbName());
			return $this->getMySqlDbhFromSettings($dbSettings);
		}
		
		$dbSettings601 = $this->getDbSettings("6.0.1");
		if ($dbSettings601 && $dbSettings601->isMysql()){
			message("Using db " . $dbSettings601->getDbName());
			return $this->getMySqlDbhFromSettings($dbSettings601);
		}
		
		$dbSettings6 = $this->getDbSettings("6.0.0");
		if ($dbSettings6 && $dbSettings6->isMysql()){
			message("Using db " . $dbSettings6->getDbName());
			return $this->getMySqlDbhFromSettings($dbSettings6);
		}
		
		$dbSettings56 = $this->getDbSettings("5.6.0");
		if ($dbSettings56 && $dbSettings56->isMysql()){
			$dbSettings56->setDbName("ZendServer");
			message("Using db " . $dbSettings56->getDbName());
			return $this->getMySqlDbhFromSettings($dbSettings56);
		}
		
		return null;
	}	
	
	
	/**
	 * @param $dbSettings ZSDbSettings
	 * @return PDO
	 */
	protected function getMySqlDbhFromSettings($dbSettings) {
	
		$dbHost = $dbSettings->getDbHost();
		$dbPort = $dbSettings->getDbPort();
		$dbName = $dbSettings->getDbName();
		$dbUsername = $dbSettings->getDbUser();
		$dbPassword = $dbSettings->getDbPassword();
	
		//message("Connecting to Mysql Db " . $dbName . " at $dbHost");
		try {
			$pdo = new PDO("mysql:host=$dbHost;port=$dbPort;dbname=$dbName",$dbUsername, $dbPassword);
			$pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_WARNING);
		} catch (\Exception $ex) {
			error("Cannot connect to mysql:host=$dbHost;port=$dbPort;dbname=$dbName");
			return null;
		}
	
		return $pdo;
	}
	
	protected function getSqlsDir() {
		return get_cfg_var("zend.install_dir") . "/share";
	}
	
	protected function getDbDir($version = null) {
		if (!$version) {
			return get_cfg_var("zend.data_dir") . "/db";
		} else {
			return get_cfg_var("zend.data_dir") . "/db-$version";
		}
	}
	
	protected function getEtcDir($version = null) {
		if (!$version) {
			return get_cfg_var("zend.conf_dir");
		} else {
			return get_cfg_var("zend.conf_dir") . "-$version";
		}
	}
	
	
	protected function copyWithPerms($orig, $target) {
		$res = copy($orig, $target);
		if (!$res) {
			return false;
		}
		$perms = fileperms($orig);
		return chmod($target, $perms);
	}  
	
	
	protected function getSqliteDbPath($version = null) {
		return $this->getDbDir($version) . "/" . $this->getSqliteDbName($version);
	}
	
	
	protected function copyDatabaseFromVersion($version) {
		$orig = $this->getSqliteDbPath($version);
		$target = $this->getSqliteDbPath();
		if (!file_exists($target) && file_exists($orig)) {
			message("Copying database from $orig to $target");
			return $this->copyWithPerms($orig, $target);
		}
		
		return false;
	}
	
	abstract protected function upgradeSqlite();
	
	
	protected function copySqliteFromPreviousVersions($version = null) {
		
		if (!$version) {
			
			$res = $this->copyDatabaseFromVersion("6.1.0");
			if ($res) {
				return "6.1.0";
			}
			
			$res = $this->copyDatabaseFromVersion("6.0.1");
			if ($res) {
				return "6.0.1";
			}
			
			$res = $this->copyDatabaseFromVersion("6.0.0");
			if ($res) {
				return "6.0.0";
			}
			
			$res = $this->copyDatabaseFromVersion("5.6.0");
			if ($res) {
				return "5.6.0";
			}
				
			
		} else {
			
			$res = $this->copyDatabaseFromVersion($version);
			if ($res) {
				return $version;
			}
		}
		
		return false;
	}
	
	abstract protected function upgradeMysql();
	
	abstract public function upgrade();
	
	protected function getMysqlDbSettings($version = null) {
		
		if ($version == "5.6.0") {
			return ZSDbSettings::fromIni($_GET['zsDir'] . "/etc/monitor_node.ini");
		}		
		
		if (!$version || $version[0] == "6") {
			return ZSDbSettings::fromIni($_GET['zsDir'] . "/etc/zend_database.ini");
		}
		
		return null;
	}
	
	
	/**
	 * @param PDO $dbh
	 * @param string $queries
	 */
	function execQueriesDiscardDuplicates($dbh, $queries) {
	
		$errMode = $dbh->getAttribute(PDO::ATTR_ERRMODE);
	
		$wasInTransaction = $dbh->inTransaction();
	
		$dbh->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
		try {
			if (!$wasInTransaction) {
				$dbh->beginTransaction();
			}
				
			execQueries($dbh, $queries);
				
			if (!$wasInTransaction) {
				$dbh->commit();
			}
				
		} catch (PDOException $exc) {
			if (!$wasInTransaction) {
				$dbh->rollback();
			}
				
			if (stripos($exc->getMessage(), "duplicate") === false && stripos($exc->getMessage(), "already") === false) {
				if (!$wasInTransaction) {
					error($exc->getMessage());
				} else {
					throw $exc;
				}
			}
		}
	
		$dbh->setAttribute(PDO::ATTR_ERRMODE, $errMode);
	}
	
}

?>