<?php

	ini_set("error_reporting", E_ALL);
	ini_set("output_buffering", 0);
	ini_set("max_execution_time", 60 * 60); // one hour timeout
	ini_set("html_errors", 0);
	ini_set("memory_limit", -1);
	
	
	// Define the monitor tables that will be renamed
	// during upgrade incase the number of events in the tables
	// is larger than 10K
	$TABLES[] = "backtraces";
	$TABLES[] = "event_attribute_types";
	$TABLES[] = "event_attributes";
	$TABLES[] = "event_types";
	$TABLES[] = "events";
	$TABLES[] = "func_args";
	$TABLES[] = "requests";
	$TABLES[] = "scripts";
	$TABLES[] = "vhosts";
	$TABLES[] = "matched_rules";
	$TABLES[] = "event_tags";
	$TABLES[] = "issues";
	//$TABLES[] = "request_components";  // This table only exists in ZS5.6 and UP
	$TABLES[] = "cluster_issues";
	$TABLES[] = "trace_files";
	$TABLES[] = "schema_properties";
	//$TABLES[] = "request_uid_to_event_id"; // This table only exists in ZS5.6 and UP
	
	require_once dirname(__FILE__) . '/zs_jq_db_handler.php';
	require_once dirname(__FILE__) . '/zs_zsd_db_handler.php';
	require_once dirname(__FILE__) . '/zs_gui_db_handler.php';
	require_once dirname(__FILE__) . '/zs_deployment_db_handler.php';
	require_once dirname(__FILE__) . '/zs_db_settings.php';
	require_once dirname(__FILE__) . '/zs_statistics_db_handler.php';
	require_once dirname(__FILE__) . '/zs_monitor_db_handler.php';
	require_once dirname(__FILE__) . '/zs_tracing_db_handler.php';
		
	global $sqliteDbVersion;
	$sqliteDbVersion = "1.0.10";
	global $mysqlDbVersion;
	$mysqlDbVersion = "1.0.10";
	
	global $verbosity;
	$verbosity = 0;
	
	global $logFile;
	$logFile = NULL;
	
	define('ZS6_DATABASE_NAME', "ZendServer");
	
	global $commonUpgradeQueries;
	$commonUpgradeQueries = "
		
		ALTER TABLE events MODIFY node_id INTEGER; 
		
		ALTER TABLE issues MODIFY node_id INTEGER;		
		ALTER TABLE issues MODIFY agg_key varchar(32);		
		ALTER TABLE issues ADD cluster_issue_id INTEGER default -1;
		ALTER TABLE events ADD cluster_issue_id INTEGER default -1;
		ALTER TABLE events MODIFY agg_key varchar(32);		
		ALTER TABLE cluster_issues MODIFY agg_key varchar(32);
						
		CREATE INDEX EVENTS_CLUSTER_ISSUE_ID_IDX ON events(cluster_issue_id);
		CREATE INDEX ISSUES_CLUSTER_ISSUE_ID_IDX ON issues (cluster_issue_id);
		CREATE INDEX EVENTS_ISSUE_ID_IDX ON events (issue_id);
		CREATE INDEX EVENTS_REQUEST_ID ON events (request_id);
		CREATE INDEX ISSUES_NODE_ID_STATUS_IDX ON issues (node_id, status);
		CREATE INDEX ISSUES_CLUSTER_ISSUE_ID_SEVERITY_IDX ON issues (cluster_issue_id, severity);
		CREATE INDEX EVENTS_CLUSTER_ISSUE_ID_NODE_LAST_IDX ON events (cluster_issue_id, node_id, last_timestamp);		
	
	";
	
	
	/**
	  * @return  ZSDbSettings
	 */
	function getZS56DbSettings() {
		if (!file_exists($_GET['zsDir'] . "/etc-5.6.0/monitor_node.ini")) {
			return false;
		}
		return ZSDbSettings::fromIni($_GET['zsDir'] . "/etc-5.6.0/monitor_node.ini");		
	}
	
	/**
	 * 
	 * @return ZSDbSettings
	 */
	function getDbSettings() {
		return ZSDbSettings::fromIni($_GET['zsDir'] . "/etc/zend_database.ini");
	}
	
	/*
	 * @return PDO
	 */
	function getZS6MysqlDbh() {
		
		$dbSettings = getZS56DbSettings();
		if ($dbSettings && $dbSettings->isMysql()) {
			$dbSettings->setDbName(ZS6_DATABASE_NAME);			
		} else {
			$dbSettings = getDbSettings();
		}
				
		$dbh = null;
		
		if ($dbSettings->isMysql()) {
			$dbh = new PDO("mysql:host=" . $dbSettings->getDbHost() . ";port=" . $dbSettings->getDbPort() . ";dbname=" . $dbSettings->getDbName(), $dbSettings->getDbUser(), $dbSettings->getDbPassword());
			$dbh->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_WARNING);
		} 
		
		
		return $dbh;
	}	
	
	/*
	 * @return PDO
	*/
	function getZS56MysqlDbh() {
		$dbSettings = getZS56DbSettings();
		if (!$dbSettings) {
			return null;
		}
		$dbh = null;
	
		if ($dbSettings->isMysql()) {
			$dbh = new PDO("mysql:host=" . $dbSettings->getDbHost() . ";port=" . $dbSettings->getDbPort() . ";dbname=" . $dbSettings->getDbName(), $dbSettings->getDbUser(), $dbSettings->getDbPassword());
			$dbh->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_WARNING);
		}		
		
	
		return $dbh;
	}
	
		
	/**
	 * @param PDO $dbh
	 * @param string $queries
	 */
	function execQueriesDiscardDuplicates($dbh, $queries) {
		
		$errMode = $dbh->getAttribute(PDO::ATTR_ERRMODE);
		
		$wasInTransaction = $dbh->inTransaction();
		
		$dbh->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
		try {
			if (!$wasInTransaction) {
				$dbh->beginTransaction();
			}
			
			execQueries($dbh, $queries);
			
			if (!$wasInTransaction) {
				$dbh->commit();
			}
			
		} catch (PDOException $exc) {
			if (!$wasInTransaction) {
				$dbh->rollback();
			}
			
			if (stripos($exc->getMessage(), "duplicate") === false && stripos($exc->getMessage(), "already") === false) {
				if (!$wasInTransaction) {
					error($exc->getMessage());
				} else {
					throw $exc;
				}
			}	
		}
		
		$dbh->setAttribute(PDO::ATTR_ERRMODE, $errMode);
	}
	
	
	function startsWith($str, $start) {
		return (strpos($str, $start) === 0);	
	}
	
	function createJoinClusterTask() {
		
		$db56Settings = getZS56DbSettings();
		if (!$db56Settings) {
			message("Not cluster upgrade");
			return;
		}
		$db56 = getZS56MysqlDbh();
		

		if (stripos(PHP_OS, "WIN") === 0) {
			message ("Windows cluster upgrade");
			$zsdDb = new PDO("sqlite:" . $_GET['zsDir'] . "/data/db/zsd.db");
		} else {
			message ("Unix cluster upgrade");
			$zsdDb = new PDO("sqlite:" . $_GET['zsDir'] . "/var/db/zsd.db");
		}		
		
		$scdIni = parse_ini_file($_GET['zsDir'] . "/etc-5.6.0/scd.ini");
				
		$joinInfo = array();
		$joinInfo['database_host'] = $db56Settings->getDbHost();
		$joinInfo['database_port'] = $db56Settings->getDbPort();
		$joinInfo['database_name'] = "ZendServer";
		$joinInfo['user'] = $db56Settings->getDbUser();
		$joinInfo['password'] = $db56Settings->getDbPassword();
		$joinInfo['node_ip'] = $scdIni['zend_sc.network.hostname'];
		$joinInfo['node_name'] = gethostname();
		$joinInfo['creationTime'] = time();
		
		
		
		try {
			$zsdDb->beginTransaction();
			$stmt = $zsdDb->prepare("INSERT INTO ZSD_TASKS (ZSD_TASKS_SEQUENCE, NODE_ID, TASK_ID, EXTRA_DATA, AUDIT_ID, OWNER_NODE_ID) VALUES(NULL, :NODE_ID, :TASK_ID, :EXTRA_DATA, :AUDIT_ID, :OWNER_NODE_ID)");
			$stmt->bindValue(":NODE_ID", 0);
			$stmt->bindValue(":TASK_ID", 3); //for connect
			$stmt->bindValue(":EXTRA_DATA", json_encode($joinInfo));
			$stmt->bindValue(":AUDIT_ID", -1);
			$stmt->bindValue(":OWNER_NODE_ID", -1);
			
			$stmt->execute();
			$zsdDb->commit();
			
			message("Join cluster task id - " . $zsdDb->lastInsertId());
		} catch (Exception $ex) {
			$zsdDb->rollBack();
				error($ex->getMessage());
			}
		}
	
	function createZS6DbIni() {
		message("Creating zend_database.ini");
		$ini56 = $_GET['zsDir'] . "/etc-5.6.0/monitor_node.ini";
		$ini60 = $_GET['zsDir'] . "/etc/zend_database.ini";
		
		$upgradeRequired = true;
		if (!file_exists($ini56)) {
			message($ini56 . " does not exist");
			$upgradeRequired = false;
		} else {
			message("Upgrade from ZS 5.6 is required");
		}
				
		if (!$upgradeRequired) {
			return;
		}		
		
		$iniContents = file($ini56);
		$newIniContents = array();
		$newIniContents[] = "[Zend.zend_database]" . PHP_EOL . PHP_EOL;
		for ($i = 0 ; $i < count($iniContents) ; $i++) {
			$line = trim($iniContents[$i]);
			if (startsWith($line, 'zend_monitor.database')) {
				if (startsWith($iniContents[$i -1], ";")) {
					$newIniContents[] = $iniContents[$i -1];
				}				
				
				$line = preg_replace('#^zend_monitor.database.name.*#', "zend.database.name=ZendServer", $line);
				
				$line = preg_replace('#^zend_monitor(.database.*)#', "zend$1", $line);
				$newIniContents[] = $line . PHP_EOL . PHP_EOL;
			}
		}
		
		
		file_put_contents($ini60, implode($newIniContents));
		
		$newIniContents['zend.database.password'] = "******" . PHP_EOL;
		message("New contents:" . PHP_EOL .  implode($newIniContents));
		
		$perms = fileperms($ini56);
		chmod($ini60, $perms);		
	}
	
		
	$uiIni = parse_ini_file($_GET['zsDir'] . "/gui/config/zs_ui.ini", true);
	define("ZEND_SERVER_APP_VERSION", "PE");	
		
	function execQueries($dbh, $queries) {
		$queries = explode(";", $queries);
		foreach ($queries as $query) {
			$query = trim($query);
			if (!$query) {
				continue;
			}
			
			//message($query);
			$res = $dbh->exec($query);
			if($res === FALSE)
				message(sprintf("Error occurred when executing the following query:\n%s\n",$query));
			//message(var_export($res, true));
		}	
	}
	
	function setZendNewPassword ($dbh, $username, $password)
	{
		$old_passwords = 1;
		
		foreach ($dbh->query("SELECT @@old_passwords;") as $row) {
			$old_passwords = (int)$row[0];
			if ($old_passwords == 0) {
				message("No need to fix new password");
				return 0;
			}
			break;
		}
		
		message("fixing new password");
		$queries = <<<EOF
			set @@old_passwords = 0;
			SET PASSWORD = PASSWORD('$password');
			set @@old_passwords = $old_passwords;
			FLUSH PRIVILEGES;
EOF
;
		execQueries($dbh, $queries);	
	}

	function getIssuesCount($dbh) {
		$minRowId = 0;
		$maxRowId = 0;
		
		foreach ($dbh->query("select id from cluster_issues ORDER BY id DESC LIMIT 1") as $row) {
			$maxRowId = $row['id'];
			break;
		} 
		foreach ($dbh->query("select id from cluster_issues ORDER BY id ASC LIMIT 1") as $row) {
			$minRowId = $row['id'];
			break;
		} 
		
		$issuesCount = $maxRowId - $minRowId;
		
		message("Db holds $issuesCount issues");
		
		return $issuesCount;
	}
	
	function createDropDbMark() {
		$mark = $_GET['zsDir'] . "/tmp/monitor_db_upgrade_504_510";
		message("Creating drop db mark at $mark");
		touch($mark);
		chmod($mark, 0777);
	}
	
	function mysqlCreateDb($host, $port, $username, $password, $dbName, $dropIfExists, $dropIfLargerThan) {
		global $mysqlDbVersion;
		global $TABLES;
		
		if(ZEND_SERVER_APP_VERSION == 'CE') {
			message("CE version found, Monitor upgrade is skipped");
			return;
		}
		
		message("Creating MySQL Monitor database at " . $host . "...");
		$dbh = NULL;
		try {
			$dbh = new PDO("mysql:host=$host;port=$port;dbname=$dbName", $username, $password);
			$dbh->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_WARNING);
			message("Connection successful");
			$dbh->beginTransaction();
			// create base database
				
			$queries = file_get_contents($_GET["zsDir"] . DIRECTORY_SEPARATOR .  "share" . DIRECTORY_SEPARATOR . "mysql_create_monitor_db.sql");
			execQueriesDiscardDuplicates($dbh, $queries);			
						
			$dbCurrentVersion = getCurrentDbVersion($dbh, false);
			
			if (version_compare($dbCurrentVersion, "1.0.7", "<")) {
				createMysqlDeploymentDatabase($dbh);
			}
			
			if (ZEND_SERVER_APP_VERSION == 'EE') {
				if ( ($dropIfExists || $dropIfLargerThan) && ($dbCurrentVersion != $mysqlDbVersion) ) {
					$shouldDrop = $dropIfExists;
					if ($dropIfLargerThan) {
						$shouldDrop = (getIssuesCount($dbh) > $dropIfLargerThan);
					}
					
					if ($shouldDrop) {
						message("Backuping tables");
						
						foreach ($TABLES as $table) {
							// Make sure that the "old" name is suffixed with a unique string
							// to make this flow repeatable
							$newName = "ZOLD_" . $table . "_" . time();
							message("Backing up table : " . $table . " => " . $newName);
							execQueries($dbh, "ALTER TABLE $table RENAME TO $newName");
						}
						
						message("Recreating schema...");
						$queries = file_get_contents($_GET["zsDir"] . DIRECTORY_SEPARATOR .  "share" . DIRECTORY_SEPARATOR . "mysql_create_monitor_db.sql");
						execQueries($dbh, $queries);
						
						$dbCurrentVersion = $mysqlDbVersion;
						createDropDbMark();
						
					}
				}
				
				// check upgrade if needed
				if (version_compare($dbCurrentVersion, $mysqlDbVersion, "<")){
					message("Upgrade to $mysqlDbVersion is needed...");
					mysqlUpgradeDb($dbh, $dbCurrentVersion, $mysqlDbVersion);
				}
				
			}
				
			if (version_compare($dbCurrentVersion, "1.0.6", "<")) {
				// if db version is before 1.0.6 - we need to change password according to old mechanism
				setZendNewPassword($dbh, $username, $password);
			}
						
			$dbh->commit();
		} catch (PDOException $exc) {
			if ($dbh) {
				$dbh->rollback();
			}
			ERROR($exc->getMessage());
			return false;
		}
		
		return true;
	}
	
	function createMysqlDeploymentDatabase($dbh) {
		message("Creating deployment tables");
		$sqlFile = $_GET['zsDir'] . "/share/deployment_mysql_create_database.sql";
		$deploymentQueries = file_get_contents($sqlFile);
		if (!$deploymentQueries) {
			ERROR("Cannot find deployment schema SQL file at $sqlFile");
			return false;
		}
		execQueries($dbh, $deploymentQueries);
		
		return true;
	}
	
	///////////////////////////////////////////////////////////
	// 107 => 108
	function upgrade_107_108_mysql($dbh) {
		message("Upgrading from version 1.0.7 -> 1.0.8(MYSQL)");
		execQueries($dbh, "CREATE TABLE IF NOT EXISTS `request_components` (`id` INTEGER NOT NULL UNIQUE PRIMARY KEY AUTO_INCREMENT, `cluster_issue_id` INTEGER NOT NULL, `comp_name` varchar(128) NOT NULL, `comp_value` varchar(128))ENGINE=MyISAM;");
		execQueries($dbh, "CREATE INDEX REQUEST_COMPONENTS_IDX on `request_components`(comp_value);");
		execQueries($dbh, "CREATE INDEX REQUEST_COMPONENTS_IDX2 on `request_components`(cluster_issue_id);");
		execQueries($dbh, "REPLACE INTO schema_properties VALUES('SCHEMA_VERSION', '1.0.8');");
	}

	function upgrade_107_108_sqlite($dbh) {
		message("Upgrading from version 1.0.7 -> 1.0.8 (SQLITE)");
		execQueries($dbh, "CREATE TABLE IF NOT EXISTS request_components ( id INTEGER NOT NULL UNIQUE PRIMARY KEY AUTOINCREMENT, cluster_issue_id INTEGER NOT NULL, comp_name varchar(128) NOT NULL, comp_value varchar(128) )");
		execQueries($dbh, "CREATE INDEX IF NOT EXISTS REQUEST_COMPONENTS_IDX on request_components(comp_value);");
		execQueries($dbh, "CREATE INDEX IF NOT EXISTS REQUEST_COMPONENTS_IDX2 on request_components(cluster_issue_id);");
		execQueries($dbh, "INSERT OR REPLACE INTO version VALUES('SCHEMA_VERSION', '1.0.8');");
	}
	
	/////////////////////////////////////////////////////////
	// 108 => 109
	function upgrade_108_109_sqlite($dbh, $dbh_trace) {
		message("Upgrading from version 1.0.8 -> 1.0.9 (SQLITE)");
		execQueries($dbh, "CREATE TABLE IF NOT EXISTS `request_uid_to_event_id` (request_id_str VARCHAR(128),request_id INTEGER,event_id   INTEGER,last_updated INTEGER);
		CREATE INDEX REQUEST_UID_TO_EVENT_ID_IDX1 on `request_uid_to_event_id`(request_id_str);
		CREATE INDEX REQUEST_UID_TO_EVENT_ID_IDX2 on `request_uid_to_event_id`(request_id);
		CREATE INDEX REQUEST_UID_TO_EVENT_ID_IDX3 on `request_uid_to_event_id`(last_updated);
		");
		execQueries($dbh_trace, "ALTER TABLE `trace_files` add column `app_id`  INTEGER NOT NULL default -1;");
		execQueries($dbh_trace, "CREATE INDEX IF NOT EXISTS TRACE_FILES_IDX1 on trace_files(app_id);");
		execQueries($dbh, "INSERT OR REPLACE INTO version VALUES('SCHEMA_VERSION', '1.0.9');");		
	}

	function upgrade_108_109_mysql($dbh) {
		message("Upgrading from version 1.0.8 -> 1.0.9 (MYSQL)");
		execQueries($dbh, "CREATE TABLE IF NOT EXISTS `request_uid_to_event_id` (request_id_str VARCHAR(128),request_id INTEGER,event_id   INTEGER,last_updated INTEGER);
		CREATE INDEX REQUEST_UID_TO_EVENT_ID_IDX1 on `request_uid_to_event_id`(request_id_str);
		CREATE INDEX REQUEST_UID_TO_EVENT_ID_IDX2 on `request_uid_to_event_id`(request_id);
		CREATE INDEX REQUEST_UID_TO_EVENT_ID_IDX3 on `request_uid_to_event_id`(last_updated);
		");
		execQueries($dbh, "ALTER TABLE `trace_files` add column `app_id`  INTEGER NOT NULL default -1;");
		execQueries($dbh, "CREATE INDEX TRACE_FILES_IDX1 on `trace_files`(`app_id`);");
		execQueries($dbh, "REPLACE INTO schema_properties VALUES('SCHEMA_VERSION', '1.0.9');");
	}
	
	function upgrade_109_1010_mysql($dbh) {
		message("Upgrading from version 1.0.9 -> 1.0.10 (MYSQL)");
	
		execQueries($dbh, "
				UPDATE issues SET STATUS = " . ZM_STATUS_DELETED . ";
				CREATE TABLE IF NOT EXISTS event_actions(event_id INTEGER NOT NULL PRIMARY KEY, url varchar(256) DEFAULT NULL, email varchar(128) DEFAULT NULL);
				CREATE INDEX TRACE_FILES_IDX1_PATH on trace_files(filepath);
				REPLACE INTO schema_properties VALUES('SCHEMA_VERSION', '1.0.10');");
	}
	
	function upgrade_109_1010_sqlite($dbh, $trace_dbh) {
		message("Upgrading from version 1.0.9 -> 1.0.10 (SQLITE)");
	
		execQueries($dbh, "
				UPDATE issues SET STATUS = " . ZM_STATUS_DELETED . ";
				CREATE TABLE IF NOT EXISTS event_actions(event_id INTEGER NOT NULL PRIMARY KEY, url varchar(256) DEFAULT NULL, email varchar(128) DEFAULT NULL);
				INSERT OR REPLACE INTO version VALUES('SCHEMA_VERSION', '1.0.10')");
		
		execQueries($trace_dbh, "CREATE INDEX TRACE_FILES_IDX1_PATH on trace_files(filepath);");
	}
	

	function mysqlUpgradeDb($dbh, $dbCurrentVersion, $dbVersion){
		global $commonUpgradeQueries;
		
		if ($dbCurrentVersion == "1.0.4" || $dbCurrentVersion == "1.0.5") {
			$queries = $commonUpgradeQueries . "
				UPDATE IGNORE issues SET cluster_issue_id = (SELECT id from cluster_issues WHERE issues.agg_key = cluster_issues.agg_key);
				UPDATE IGNORE events SET cluster_issue_id = (SELECT id from cluster_issues WHERE events.agg_key = cluster_issues.agg_key);
				UPDATE IGNORE events SET issue_id = (SELECT id from issues WHERE events.agg_key = issues.agg_key AND events.node_id = issues.node_id);
				REPLACE INTO schema_properties VALUES('SCHEMA_VERSION', '1.0.6');
				";
			execQueries($dbh, $queries);
		} 
		
		$cur_ver = getCurrentDbVersion($dbh, false);
		if ($cur_ver == "1.0.6") {
		  upgrade_106_107($dbh);
		}

		$cur_version = getCurrentDbVersion($dbh, false);
		if ($cur_version == "1.0.7") {
			upgrade_107_108_mysql($dbh);
		}
		
		$cur_version = getCurrentDbVersion($dbh, false);
		if($cur_version == "1.0.8") {
			upgrade_108_109_mysql($dbh);
		}
		
		$cur_version = getCurrentDbVersion($dbh, false);
		if($cur_version == "1.0.9") {
			upgrade_109_1010_mysql($dbh);
		}
		execQueries($dbh, "REPLACE INTO schema_properties VALUES('SCHEMA_VERSION', '{$dbVersion}');");
	}
	
	function sqliteUpgradeDb($dbh, $dbCurrentVersion, $dbVersion){
		global $commonUpgradeQueries;
		if ($dbCurrentVersion == "1.0.4" || $dbCurrentVersion == "1.0.5") {
			message("Upgrading from version $dbCurrentVersion");
			
			$queries = 
			$commonUpgradeQueries 
			.  "UPDATE OR IGNORE issues SET cluster_issue_id = (SELECT id from cluster_issues WHERE issues.agg_key = cluster_issues.agg_key);
				UPDATE OR IGNORE events SET cluster_issue_id = (SELECT id from cluster_issues WHERE events.agg_key = cluster_issues.agg_key);
				UPDATE OR IGNORE events SET issue_id = (SELECT id from issues WHERE events.cluster_issue_id = issues.cluster_issue_id AND events.node_id = issues.node_id);
				INSERT OR REPLACE INTO version VALUES('SCHEMA_VERSION', '1.0.6');
				";
			execQueries($dbh, $queries);
		}
		
		$cur_ver = getCurrentDbVersion($dbh, true);
		if ($cur_ver == "1.0.6") {
		  upgrade_106_107_sqlite($dbh);
		}
		
		$cur_ver = getCurrentDbVersion($dbh, true);
		if ($cur_ver == "1.0.7") {
			upgrade_107_108_sqlite($dbh);
		}
		
		$cur_ver = getCurrentDbVersion($dbh, true);
		if ($cur_ver == "1.0.8") {
			// version 108 -> 109 is changes in the tracing DB 
			// which requires a second connection to the tracing DB
			$traceDb = getSqliteTracingDbName();
			$trace_dbh = new PDO("sqlite:$traceDb");
			$trace_dbh->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_WARNING);
			
			upgrade_108_109_sqlite($dbh, $trace_dbh);
		}
		
		if ($cur_ver == "1.0.9") {
			$traceDb = getSqliteTracingDbName();
			$trace_dbh = new PDO("sqlite:$traceDb");
			$trace_dbh->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_WARNING);
			upgrade_109_1010_sqlite($dbh, $trace_dbh);
		}
		
		execQueries($dbh, "INSERT OR REPLACE INTO version VALUES('SCHEMA_VERSION', '{$dbVersion}');");
	}
	
	function sqliteBackupDb($filename) {
		$newName = $filename . ".5_0_4";
		if (!rename($filename, $newName)) {
			message("Cannot rename database $filename");
			return false;
		}		

		$filename .= "_codetracing";
		$newName = $filename . ".5_0_4";
		if (!rename($filename, $newName)) {
			message("Cannot rename database $filename");
			return false;
		}
		
		return true;
	}
	
	function sqliteCreateMonitorDb($filename, $dropIfExists, $dropIfLargerThan) {
				
		global $sqliteDbVersion;
		
		$dbExists = file_exists($filename);
		if (!$dbExists) {
			message("Creating new db file... " . $filename);
		} else {
			message("Database exists... " . $filename);
		}
	
		message("Creating Monitor SQLITE database at $filename");
		
		$dbh = NULL;
		try {
			$dbh = new PDO("sqlite:$filename");
			$dbh->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_WARNING);
			$dbh->beginTransaction();
			
			// upgrade if needed
			message("Checking if upgrade is needed...");
			$dbCurrentVersion = "";
			foreach ($dbh->query('SELECT version FROM version') as $row) {
				$dbCurrentVersion = $row['version'];
			}
			message("Found schema version - " . $dbCurrentVersion);
			
			if ( ($dropIfExists || $dropIfLargerThan) && ($dbCurrentVersion != $sqliteDbVersion) ) {
				$shouldDrop = $dropIfExists;
				if ($dropIfLargerThan) {
					$shouldDrop = (getIssuesCount($dbh) > $dropIfLargerThan);						
				}
				
				if ($shouldDrop) {
					$dbh->commit();
					$dbh = NULL;
					
					message("Backuping database $filename");
					if (sqliteBackupDb($filename)) {
						
						createDropDbMark();	
						
						return true;
					} 
					
					$dbh = new PDO("sqlite:$filename");
					$dbh->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_WARNING);
					$dbh->beginTransaction();
				}
			}
			
			if (version_compare($dbCurrentVersion, $sqliteDbVersion, "<")){
				message("Upgrade to $sqliteDbVersion is needed...");
				sqliteUpgradeDb($dbh, $dbCurrentVersion, $sqliteDbVersion);
			} else {
				message("No upgrade needed");
			}
			
			$dbh->commit();
		} catch (PDOException $exc) {
			if ($dbh) {
				$dbh->rollback();
			}
			ERROR("DB creation of $filename failed with the following message '" . $exc->getMessage() . "' and with the following exception: " . PHP_EOL . $exc->getTraceAsString());
			return false;
		}
		
		return true;
			
	}
	
	function ERROR($str) {
		$str = "ERROR: " . $str;
		echo $str . PHP_EOL;
		logMsg($str);
	}
	
	function getSqliteTracingDbName() {
		if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') {
			$tracing_file_name = $_GET['zsDir'] . "/data/db/monitor_codetracing" ;
		} else {
			$tracing_file_name = $_GET['zsDir'] . "/var/db/monitor_codetracing" ;
		}
		return $tracing_file_name;
	}
	
	function logMsg($str) {
		global $logFile;
		if (!$logFile) {
			if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') {
				$logPath = $_GET['zsDir'] . "/logs/zs_create_databases.log" ;
			} else {
				$logPath = $_GET['zsDir'] . "/var/log/zs_create_databases.log" ;
			}
			$logFile = @fopen($logPath, "a+");
			ini_set("log_errors", 1);
			ini_set("error_log", $logPath);
		}
		
		$str = "[" . date('c') . "] " . $str;
		@fwrite($logFile, $str . PHP_EOL);
	}
	
	function message($str) {
		global $verbosity;
		if ($verbosity > 0) {
			echo $str . PHP_EOL;		
		}		
		
		logMsg($str);
	}
	
	function getCurrentDbVersion($dbh, $isSqlite) {
		if($isSqlite) {
			foreach ($dbh->query('SELECT version FROM version') as $row) {
				$current_version = $row['version'];
			}
		} else {
            foreach ($dbh->query("SELECT property_value FROM schema_properties WHERE property = 'SCHEMA_VERSION'") as $row) {
				$current_version = $row['property_value'];
			}
		}
		message("Found schema version " . $current_version);
		return $current_version;
	}
	
	
	// Upgrade schema from version 1.0.6 -> 1.0.7
	function upgrade_106_107($dbh) {
    try {	
  	  message(" ==> Calling upgrade 1.0.6 -> 1.0.7 (MYSQL)");
  		message("Upgrading from 1.0.6 -> 1.0.7 (MYSQL)");
						
			$commonUpgradeQueries_106_107 = "
				ALTER TABLE events ADD app_id INTEGER default -1;
				REPLACE INTO schema_properties VALUES('SCHEMA_VERSION', '1.0.7');
			";
			execQueries($dbh, $commonUpgradeQueries_106_107);
			
		} catch (PDOException $exc) {
			ERROR($exc->getMessage());
			return false;
		}
		return true;
	}
	
	function upgrade_106_107_sqlite($dbh) {
		try {
			message("Upgrading from 1.0.6 -> 1.0.7 (SQLITE)");
			$commonUpgradeQueries_106_107 = "
				ALTER TABLE events ADD app_id INTEGER default -1;
				INSERT OR REPLACE INTO version VALUES('SCHEMA_VERSION', '1.0.7');
			";
			execQueries($dbh, $commonUpgradeQueries_106_107);
			
		} catch (PDOException $exc) {
			ERROR($exc->getMessage());
			return false;
		}
		return true;
	}
	
	
	function updateTempLicense() {
		message("Creating temp license");
		if (!file_exists($_GET['zsDir'] . "/etc/monitor_node.ini")) {
			message("Cannot find monitor_node.ini");
			return;
		}
		$ini = parse_ini_file($_GET['zsDir'] . "/etc/monitor_node.ini");
		$globalDirectives = $ini['zend_monitor.global_directives_ini_file'];
		if (!file_exists($_GET['zsDir'] . "/etc/monitor_node.ini")) {
			message("Cannot find monitor_node.ini");
			return;
		}
		$arr = file($globalDirectives);
		
		foreach ($arr as $key => $line) {
			if (strstr($line, "zend.serial_number")) {
				$arr[$key] = "zend.serial_number=EQ126J40041G21AC230E9B51CEB94B00" . PHP_EOL;
			} else if (strstr($line, "zend.user_name")) {
				$arr[$key] = "zend.user_name=zend-evaluation" . PHP_EOL;
			}
		}
		
		file_put_contents($globalDirectives, implode($arr));
		
		
	}
	
	// Create sqlite database at $filePath using the sql statement
	// read from $sqlFile
	function create_sqlite_database($filePath, $sqlFile) {
		try {
			if(!file_exists($filePath)) {
				message("creating sqlite database " . $filePath);
				$sqliteDb = new PDO("sqlite:$filePath");
				$sqlStatements = file_get_contents($sqlFile);
				execQueries($sqliteDb , $sqlStatements);
				chmod($filePath, 0664);
			}
		} catch ( PDOException $e) {
			ERROR($e->getMessage());
		}
	}
	
	/***************/
	/**** MAIN *****/
	/***************/
	
	$tz = @date_default_timezone_get();
	ini_set("date.timezone", $tz); 

	if (!isset($_GET['zsDir'])) {
		error("Please define zsDir parameter");
		die(1);	
	}
	
	$_GET['zsDir'] = realpath($_GET['zsDir']);
	
	
	if (isset($_GET['fromVersion'])) {
		message("***** Upgrading from Zend Server version " . $_GET['fromVersion'] . " to version " . $_GET['toVersion'] . '*****');
	} else if (isset($_GET['toVersion'])){
		message("***** Installing Zend Server version " . $_GET['toVersion'] . '*****');
	} else {
		message("***** Installing new Zend Server version ***** ");
	}
		
	// create the zend_database file according to monitor_node.ini
	//createZS6DbIni();	

	//updateTempLicense();

	message("Creating/Upgrading Monitor database...");
	$monitorHandler = new MonitorDBHandler();
	$monitorHandler->upgrade();
	
	message("Creating/Upgrading ZSD database...");
	$zsdHandler = new ZSDDbCreator();
	$zsdHandler->upgrade();
	
	message("Creating/Upgrading Deployment database...");
	$depHandler = new DeploymentDBHandler();
	$depHandler->upgrade();	
	
	message("Creating/Upgrading Statistics database...");
	$statisticsHandler = new StatisticsDBHandler();
	$statisticsHandler->upgrade();
		
	message("Creating/Upgrading Code Tracing database...");
	$ctHandler = new CodeTracingDBHandler();
	$ctHandler->upgrade();	
	
	message("Creating/Upgrading JQ database...");
	$jqHandler = new JQDBHandler();
	$jqHandler->upgrade($sqliteDbVersion);
	
	message("Creating/Upgrading GUI database...");
	$guiHandler = new GUIDBHandler();
	$guiHandler->createSchema();
	$guiHandler->removeBootSnapshot();
	$guiHandler->mergeApiKeys();
	$guiHandler->mergeStudioIntegration();
	$guiHandler->removeGuiFile();
	
	$phpIni = parse_ini_file($_GET['zsDir'] . "/etc/php.ini");
	if ($phpIni["session.save_handler"] == "cluster") {
		message("Enabling SCD for cluster");
		$scdIni = file_get_contents($_GET['zsDir'] . "/etc/scd.ini");
		$scdIni = str_replace("zend_sc.daemon.enable=0", "zend_sc.daemon.enable=1", $scdIni);
		file_put_contents($_GET['zsDir'] . "/etc/scd.ini", $scdIni);
	}
	
	if (!getDbSettings()->isMysql() && getZS56MysqlDbh()) {
		message("Preparing join cluster settings");
		$zsdHandler->updateClusterNodeId(get_cfg_var("zend.node_id"), getZS56DbSettings()->getDbHost());
		$zsdHandler->updateNodeId(0);
		message("Creating join cluster task for zsd");
		createJoinClusterTask();
	}
	
	
	