<?php

require_once dirname(__FILE__) . "/zs_component_upgrader.php";

class DeploymentDBHandler extends ZSComponentUpgrader {
	
	protected function getSqliteDbName($version = "6.0.0") {
		return "deployment.db";		 
	}
	
	protected function getMysqlSqlFilenames() {
		return array("deployment_mysql_create_database.sql");
	}
	
	protected function getSqliteSqlFilenames() {
		return array("deployment_sqlite_create_database.sql");
	}
	
	public function upgradeSqlite() {
		 
		if (file_exists($this->getSqliteDbPath())) {
			message("Database at " . $this->getSqliteDbPath() . " already exists");
			return;	
		}
		
		$fromVersion = $this->copySqliteFromPreviousVersions();
		if (!$fromVersion) {
			$this->createCleanSqliteDatabase();
			return;
		}
		
		if ($fromVersion == "6.0.0") {
			return;
		}
		
		$dbh = $this->getSqliteDbh();
		if (!$this->isTableExists($dbh, "deployment_libs_status")) {
			message ("Creating libraries tables");
		
			$dbh->exec("
					CREATE TABLE IF NOT EXISTS deployment_libs (
						lib_id INTEGER PRIMARY KEY AUTOINCREMENT,
						is_defined INTEGER DEFAULT 0
						);
		
					CREATE TABLE IF NOT EXISTS deployment_libs_versions (
									lib_version_id INTEGER PRIMARY KEY AUTOINCREMENT,
									lib_id INTEGER,
									task_descriptor_id INTEGER UNIQUE,
									creation_time INTEGER
									);
			
					CREATE TABLE IF NOT EXISTS deployment_libs_status (
									lib_status_id INTEGER PRIMARY KEY AUTOINCREMENT,
									lib_version_id INTEGER,
									status VARCHAR(32),
									node_id INTEGER,
									install_path VARCHAR(4096),
									last_message VARCHAR(1024),
									last_updated INTEGER
									);
		
					CREATE INDEX IF NOT EXISTS IDX_DEPLOYMENT_LIBS_VERSIONS_LIB_ID on deployment_libs_versions(lib_id);
					CREATE INDEX IF NOT EXISTS IDX_DEPLOYMENT_LIBS_STATUS_VERSION_ID on deployment_libs_status(lib_version_id);
		
					INSERT OR REPLACE INTO deployment_properties VALUES('SCHEMA_VERSION', '1.0.11');
					");
		}
		
		if ($fromVersion == "5.6.0") {
			
			$dbh = $this->getSqliteDbh();

			try {
				$dbh->beginTransaction();
							
				message("Upgrading 5.6 Deployment SQLITE DB");
				$dbh->exec("ALTER TABLE deployment_apps ADD is_defined INTEGER DEFAULT 0;
							ALTER TABLE deployment_packages ADD monitor_rules TEXT;
							ALTER TABLE deployment_packages ADD pagecache_rules TEXT;
							ALTER TABLE deployment_tasks ADD audit_id INTEGER DEFAULT 0;
							UPDATE deployment_properties SET value = '1.0.10' WHERE name = 'SCHEMA_VERSION';
							");
					
				$dbh->commit();
			} catch (PDOException $exc) {
				ERROR($exc->getMessage());
				$dbh->rollback();
			}
		}
				
	}
	
	
	private function copyAppsDataFrom56() {
		message("Copying deployment information from old Mysql schema");
		$dbh56 = $this->getMySqlDbh("5.6.0");

		$dbSettings56 = $this->getDbSettings("5.6.0");
		$dbName56 = $dbSettings56->getDbName();
		$dbName = "ZendServer";
		
		$queries = "";
		$queries.= "INSERT INTO $dbName.deployment_apps select app_id, base_url, user_app_name, 0 from $dbName56.deployment_apps;";
		$queries.= "INSERT INTO $dbName.deployment_packages select package_id, path, eula, logo, package_descriptor, name, version, NULL, NULL from $dbName56.deployment_packages;";
		$queries.= "INSERT INTO $dbName.deployment_tasks select task_id, group_id, node_id, type, task_descriptor_id, 0 from $dbName56.deployment_tasks;";
		$queries.= "INSERT INTO $dbName.deployment_package_data select * from $dbName56.deployment_package_data;";
		$queries.= "INSERT INTO $dbName.deployment_tasks_descriptors select * from $dbName56.deployment_tasks_descriptors;";
		$queries.= "INSERT INTO $dbName.deployment_apps_versions select * from $dbName56.deployment_apps_versions;";
		$queries.= "INSERT INTO $dbName.deployment_app_status select * from $dbName56.deployment_app_status;";
		$queries.= "INSERT INTO $dbName.deployment_sequencer select * from $dbName56.deployment_sequencer;";
		$queries.= "INSERT INTO $dbName.deployment_nodes_status select * from $dbName56.deployment_nodes_status;";
		$queries.= "INSERT INTO $dbName.deployment_vhosts select * from $dbName56.deployment_vhosts;";
			
		$dbh = $this->getMySqlDbh("5.6.0");
		$dbh->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
		try {
			$dbh->beginTransaction();
			
			$dbh->exec($queries);
			
			$dbh->commit();
			
		} catch (Exception $ex) {
			
			if (!strstr($ex->getMessage(), "already exists") && !stristr($ex->getMessage(), "duplicate")) {
				$dbh->rollBack();
				message($ex->getMessage());
			}
		}		
	}
	
	private function isDbPopulated() {
		$dbh = $this->getMySqlDbh();
		foreach ($dbh->query('select count(*) as apps_count from deployment_apps') as $row) {
			if ($row['apps_count'] > 0) {
				message("App data is populated");
				return true;
			} else {
				message("App data is not populated");
				return false;
			}
		}
		
		message("Cannot determine app data");
		return false;
	}
	
	protected function upgradeMysql() {
		
		$this->createCleanMysqlDatabase();
		
		$dbh = $this->getMySqlDbh();
		if ($dbh) {
			if (!$this->isTableExists($dbh, "deployment_libs_status")) {
				message ("Creating libraries tables");
				
				$dbh->exec("
					CREATE TABLE IF NOT EXISTS deployment_libs (
						lib_id INTEGER PRIMARY KEY AUTO_INCREMENT,
						is_defined INTEGER DEFAULT 0
						) ENGINE=MyISAM;
						
					CREATE TABLE IF NOT EXISTS deployment_libs_versions (
									lib_version_id INTEGER PRIMARY KEY AUTO_INCREMENT,
									lib_id INTEGER,
									task_descriptor_id INTEGER UNIQUE,
									creation_time INTEGER
									) ENGINE=MyISAM;
					
					CREATE TABLE IF NOT EXISTS deployment_libs_status (
									lib_status_id INTEGER PRIMARY KEY AUTO_INCREMENT,
									lib_version_id INTEGER,
									status VARCHAR(32),
									node_id INTEGER,
									install_path VARCHAR(4096),
									last_message VARCHAR(1024),
									last_updated INTEGER
									) ENGINE=MyISAM;	
						
					CREATE INDEX IDX_DEPLOYMENT_LIBS_VERSIONS_LIB_ID on deployment_libs_versions(lib_id);
					CREATE INDEX IDX_DEPLOYMENT_LIBS_STATUS_VERSION_ID on deployment_libs_status(lib_version_id);
				
					REPLACE INTO schema_properties VALUES('DEPLOYMENT_SCHEMA_VERSION', '1.0.11');
					");
			}
		}
				
						
		$dbSettings = $this->getDbSettings();
		if ($dbSettings && $dbSettings->isMysql() && $this->isDbPopulated()) {
			return;
		}
		
		$dbh6 = $this->getMySqlDbh("6.0.0");
		if ($dbh6) {
			message("Upgrade from 6.0.0 - doing nothing");
			return;
		}
		
		$dbh56 = $this->getMysqlDbh("5.6.0");
		if ($dbh56) {
			$this->copyAppsDataFrom56();
		} 	
	}	
	
	public function upgrade() {
		
		$this->upgradeSqlite();
		
		if ($this->isClusterUpgrade()) {
			$this->upgradeMysql();
		}		
	}
				
}

?>