<?php

require_once dirname(__FILE__) . "/zs_component_upgrader.php";


class GUIDBHandler extends ZSComponentUpgrader  {
	
	private $dbPath;
	private $dbh = null;
	private $dbName='gui';
	
	
	protected function getSqliteDbName($version = "6.0.0") {
		return "gui.db";
	}
	
	protected function getMysqlSqlFilenames() {
		return array(
				"gui_mysql_create_database.sql"
				, "gui_mysql_acl_fixtures.sql"
		);
	}
	
	protected function getSqliteSqlFilenames() {
		return array(
				"gui_sqlite_create_database.sql"
				, "gui_sqlite_acl_fixtures.sql"
		);
	}
	
	protected function getI5UpdateSqlFilenames() {
		return "gui_ibmi_update_database.sql";
	}
	
	public function __construct() {
		$this->dbPath = get_cfg_var('zend.data_dir') . "/db/{$this->dbName}.db";
		
		$this->copySqliteFromPreviousVersions($this->getUpgradeFromVersion());
	}

	/**
	 * @param PDO $db
	 */
	private function updateByGuiSchemaVersion(PDO $db) {
		
	}
	
	/**
	 * @param PDO $db
	 */
	private function cleanOldTablesData(PDO $db) {
		$db->exec('DELETE FROM GUI_ACL_RESOURCES');
		$db->exec('DELETE FROM GUI_ACL_PRIVILEGES');
		$db->exec('DELETE FROM GUI_AVAILABLE_LOGS');
	}
	
	public function createSchema() {
		$dbSqlite = $this->getSqliteDbh();
        if (file_exists($this->dbPath)) {
			message("Upgrade {$this->dbName} database in {$this->dbPath}");
			/// countRow doesn't work here for some reason
			$hasMetadata = (count($dbSqlite->query('SELECT name FROM sqlite_master WHERE type=\'table\' AND name=\'GUI_METADATA\';')->fetchAll(PDO::FETCH_ASSOC)) > 0);
			
			if ($hasMetadata) {
				$this->cleanOldTablesData($dbSqlite);
				$schemaVersion = $dbSqlite->query('SELECT DATA FROM GUI_METADATA WHERE NAME = \'gui_schema_version\'')->fetchColumn(0);
				if (version_compare($schemaVersion, '1.1.2', '<')) {
					message('Add unique index to gui.db GUI_AVAILABLE_LOGS');
					$dbSqlite->exec('CREATE UNIQUE INDEX unique_logs on GUI_AVAILABLE_LOGS (NAME)');
				}
			}
        }
        
        $this->createCleanSqliteDatabase();
        
        
		
		// create the MYSQL schema if needed
		if ($this->isClusterUpgrade()) {
			message("Upgrade {$this->dbName} Mysql tables");
			$dbMysql = $this->getMySqlDbh();
			if ($dbMysql) {
				$hasMetadata = ($dbMysql->query('SHOW TABLES LIKE \'schema_properties\';')->rowCount() > 0);
					
				if ($hasMetadata) {
					$this->cleanOldTablesData($dbMysql);
					$schemaVersion = $dbMysql->query('SELECT property_value FROM schema_properties WHERE property = \'gui_schema_version\'')->fetchColumn(0);
					if (version_compare($schemaVersion, '1.1.2', '<')) {
						message('Add unique index to mysql GUI_AVAILABLE_LOGS');
						$dbMysql->exec('CREATE UNIQUE INDEX unique_logs on GUI_AVAILABLE_LOGS (NAME)');
					}
				}
			}
			
			$this->createCleanMysqlDatabase();
		}
		
		// OS is IBMI
		if (in_array(PHP_OS, array('AIX', 'OS400'))) {
			message("Update the GUI database for IBMi");			
			$sql = file_get_contents($this->getSqlsDir() . "/" . $this->getI5UpdateSqlFilenames());
			$dbSqlite->exec($sql);
			if ($this->isClusterUpgrade()) {
				$dbMysql->exec($sql);
			}
		}
		
	}
	
	public function removeBootSnapshot() {
		$sqliteDb = $this->getSqliteDbh();
		$query = "DELETE FROM GUI_SNAPSHOTS WHERE NAME = 'SystemBoot';";
		$sqliteDb->query($query);
		if ($this->isClusterUpgrade()) {
			$this->getMySqlDbh($this->getUpgradeFromVersion())->query($query);
		}
	}
	
	public function mergeApiKeys() {
		/// perform only when upgrading from 5.6 and below
		if (version_compare('5.6.0', $this->getUpgradeFromVersion(), '<')) {
			return ;
		}
		
		message("Merge WebApi keys");

		$sqliteDb = $this->getSqliteDbh();
		
		$keysFile = $this->getZendServerUserIniPath();
		
		if (! file_exists($keysFile)) {
			message("Could not find gui configuration file at {$keysFile}");
			return;
		}
		
		$ini_array = parse_ini_file($keysFile, true);
		$keys = array();
		if (isset($ini_array['apiKeys'])) {
			foreach ($ini_array['apiKeys'] as $key => $value) {
				$exploded = explode(':', $key);
				$keys[$exploded[0]][$exploded[1]] = $value;
			}
		}
		
		foreach ($keys as $keyName => $keyValue) {
			$role = $keyValue['role'];
			if ($role == 'fullAccess') {
				$role = 'admin';
			} elseif ($role == 'readOnly') {
				$role = 'developer';
			}
			
			message("Found WebApi key {$keyValue['name']} with role {$role}");
			try {
				$query = "INSERT into GUI_WEBAPI_KEYS (NAME, HASH, USERNAME, CREATION_TIME) values('{$keyValue['name']}', '{$keyValue['hash']}', '{$role}', '{$keyValue['creationTime']}');";
				$sqliteDb->query($query);
			} catch (PDOException $exc) {
				ERROR($exc->getMessage());
				$sqliteDb->rollback();
			}
		}
	}
	
	public function mergeStudioIntegration() {
		
		if (version_compare('5.6.0', $this->getUpgradeFromVersion(), '<')) {
			return ;
		}
		
		message("Merge Zend Studio configurations");
		
		$iniFile = $this->getZendServerUserIniPath();
		
		if (! file_exists($iniFile)) {
			message("Could not find gui configuration file at {$iniFile}");
			return;
		}
		
		$ini_array = parse_ini_file($iniFile, true);
		if (isset($ini_array['studioIntegration'])) {
			$directives = array(
				'studioAutoDetectionEnabled'	=> 'studioDetectBrowser',
				'studioAutoDetection' 			=> 'studioAutoDetection',
				'studioPort' 					=> 'studioPort',
				'studioUseSsl' 					=> 'studioUseSsl',
				'studioHost' 					=> 'studioHost',
			);
			
			$newFile = get_cfg_var('zend.install_dir') . DIRECTORY_SEPARATOR . 'gui' . DIRECTORY_SEPARATOR . 'config' . DIRECTORY_SEPARATOR . 'zs_ui.ini';
			$ini = file($newFile);
			foreach ($ini as $line => $str) {
				foreach ($directives as $newDirective => $oldDirective) {
					if (strstr($str, "zend_gui." . $newDirective)) {
						$ini[$line] =  "zend_gui." . $newDirective . "=" . $ini_array['studioIntegration'][$oldDirective] . PHP_EOL;
						unset($directives[$newDirective]);
						break;
					}
				}
			}
			
			// find the location of studioIntegration section
			$integrationLine = -1;
			foreach ($ini as $line => $str) {
				if (trim($str) == '[studioIntegration]') {
					$integrationLine = $line;
					break;
				}
			}
			
			// insert the new rows under the section
			if ($integrationLine != -1) {
				foreach ($directives as $newDirective => $oldDirective) {
					$newVal =  "zend_gui." . $newDirective . "=" . $ini_array['studioIntegration'][$oldDirective] . PHP_EOL;
					array_splice( $ini, $integrationLine + 1, 0, array($newVal) );
				}
			}
			
			file_put_contents($newFile, implode("", $ini));
		}
	}
	
	public function removeGuiFile() {
		if (version_compare('5.6.0', $this->getUpgradeFromVersion(), '<')) {
			return ;
		}
		$keysFile = $this->getZendServerUserIniPath();
		
		if (! file_exists($keysFile)) {
			message("Could not find gui configuration file at {$keysFile}");
			return;
		}
		
		// remove the file since we dont need it
		unlink($keysFile);
	}
	
	private function getZendServerUserIniPath() {
		return get_cfg_var('zend.install_dir') . DIRECTORY_SEPARATOR . 'gui' . DIRECTORY_SEPARATOR . 'config' . DIRECTORY_SEPARATOR . 'zend-server-user.ini';
	}
	
	protected function upgradeSqlite() {
	}
	
	protected function upgradeMysql() {
	}

	
	public function upgrade() {
	}
}