<?php

require_once dirname(__FILE__) . "/zs_component_upgrader.php";

class JQDBHandler extends ZSComponentUpgrader   {
		
	private $dbPath;
	private $dbh;
	
	protected function getSqliteDbName($version = "6.0.0") {
		return "jobqueue.db";
	}
	
	protected function getMysqlSqlFilenames() {
		return array("jobqueue_mysql_create_database.sql");
	}
	
	protected function getSqliteSqlFilenames() {
		return array("jobqueue_sqlite_create_database.sql");
	}
	
	
	function __construct() {
		
		$this->copySqliteFromPreviousVersions();
		
		$this->dbPath = get_cfg_var('zend.data_dir') . "/db/jobqueue.db";
		$this->dbh = NULL;

		$db6Path = get_cfg_var('zend.data_dir') . "/db-6.0.0/jobqueue.db";
		if (file_exists($db6Path) && !file_exists($this->dbPath)) {
			message("Using jq db at $db6Path");
			copyWithPerms($db6Path, $this->dbPath);
		}
			
		$exists = file_exists($this->dbPath);
		
		$this->dbh = new PDO("sqlite:{$this->dbPath}");
		$this->dbh->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_WARNING);
					
		if (!$exists ) {
			message("Creating JQ database in {$this->dbPath}");
			$queries = file_get_contents($_GET['zsDir'] . "/share/jobqueue_sqlite_create_database.sql");
				
			try {
				$this->dbh->beginTransaction();
				execQueries($this->dbh, $queries);
				$this->dbh->commit();
			} catch (PDOException $exc) {
				ERROR($exc->getMessage());
				$this->dbh->rollback();
			}
				
			chmod($this->dbPath, 0664);
			
		} else {
			message("JQ database exists in {$this->dbPath}");
		}			
		
		$mysqlDbh = getZS6MysqlDbh();
		if ($mysqlDbh) {
			message("Creating JQ mysql tables");
			$queries = file_get_contents($_GET['zsDir'] . "/share/jobqueue_mysql_create_database.sql");
			execQueriesDiscardDuplicates($mysqlDbh, $queries);				
		}			
	}
	
	function getZS56JqDb() {
		$dbh = getZS56MysqlDbh();
		if ($dbh) {
			return $dbh;
		} else {
			$sqlitePath = get_cfg_var('zend.data_dir') . "/db-5.6.0/jobqueue.db";
			if (file_exists($sqlitePath)) {
				$dbh = new PDO("sqlite:$sqlitePath");
				$dbh->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_WARNING);
			}
		}

		return $dbh;
	}
	
	function getZS6JqDb() {
		$dbh = getZS6MysqlDbh();
		if ($dbh) {
			return $dbh;
		} else {
			$sqlitePath = get_cfg_var('zend.data_dir') . "/db/jobqueue.db";
			if (file_exists($sqlitePath)) {
				$dbh = new PDO("sqlite:$sqlitePath");
				$dbh->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_WARNING);
			}
		}

		return $dbh;
	}
	
	function getMyIps() {
		
		$res = array();
		
		if (stripos(PHP_OS, "WIN") === 0) {
			exec("ipconfig /all", $out);
			$patt = "/IP.+Address.+:\s(\d+\.\d+\.\d+\.\d+)/";
		} else {
			exec("ifconfig", $out);
			$patt = "/addr:(\d+\.\d+\.\d+\.\d+)/";
		}
					
		foreach ($out as $line) {
			if (preg_match($patt, $line, $matches)) {
				$res[] = $matches[1];
			}		
		}
		
		return $res;
	}
	
	public function upgrade() {
		
		$targetVersion = '1.0.11';
		
		if ($this->dbh) {
						
			message("Checking if sqlite upgrade is needed...");
			$dbCurrentVersion = "";
			foreach ($this->dbh->query('SELECT version FROM version') as $row) {
				$dbCurrentVersion = $row['version'];
			}
			message("Found current version $dbCurrentVersion");
			
			try {	
				//$this->dbh->beginTransaction();
				
				$nodeId = get_cfg_var('zend.node_id');
				
				if ($dbCurrentVersion == "1.0.0") {
					message("Upgrading from 1.0.0 to 1.0.1");
					
					$jqdIni = parse_ini_file($_GET['zsDir'] . "/etc/jqd.ini");
					if (!isset($jqdIni['zend_jobqueue.queue_name'])){
						$queueName = "";
					} else {
						$queueName = $jqdIni['zend_jobqueue.queue_name'];
					}
					
					$queries = "
					ALTER TABLE job ADD timeout INTEGER;
					ALTER TABLE schedule ADD timeout INTEGER;
					INSERT INTO queue (id, name) VALUES (NULL, '{$queueName}');
					ALTER TABLE queue ADD status INTEGER;
					UPDATE job SET queue_id = (select id from queue limit 1) WHERE queue_id = 0;
					UPDATE schedule SET queue_id = (select id from queue limit 1) WHERE queue_id = 0;
					
					INSERT OR REPLACE INTO version (key, version) VALUES ('SCHEMA_VERSION', '1.0.1');
					";	
					execQueries($this->dbh, $queries);
					$dbCurrentVersion = "1.0.1";
				}
				
				
				if (version_compare($dbCurrentVersion, "1.0.8", "<=")) {
					
					message("Upgrading from 1.0.8 or less");
					$queries = 
					"ALTER TABLE job RENAME TO jobqueue_job;
					ALTER TABLE schedule RENAME TO jobqueue_schedule;
					ALTER TABLE application RENAME TO jobqueue_application;
					
					ALTER TABLE queue ADD served_jobs INTEGER NOT NULL DEFAULT 0;
					ALTER TABLE queue ADD added_jobs INTEGER NOT NULL DEFAULT 0;
					ALTER TABLE queue RENAME TO jobqueue_queue;
					
					ALTER TABLE jobqueue_job ADD node_id INTEGER;
					UPDATE jobqueue_job SET node_id = $nodeId WHERE end_time NOT NULL;
					ALTER TABLE jobqueue_queue ADD max_http_jobs INTEGER;
					
					DROP INDEX next_job;
					CREATE INDEX next_job on jobqueue_job (status, priority); 
					
					DROP INDEX next_scheduled_job;
					CREATE INDEX next_scheduled_job on jobqueue_job (status, schedule_time, priority);
					
					INSERT OR REPLACE INTO version (key, version) VALUES ('SCHEMA_VERSION', '{$targetVersion}');
					";
					
					execQueries($this->dbh, $queries);
					
				}
				
				if (version_compare($dbCurrentVersion, "1.0.9", "<=")) {
				
					message("Upgrading from 1.0.9 to $targetVersion");
					$queries =
					"UPDATE jobqueue_schedule SET application_id = -1;
					UPDATE jobqueue_job set application_id = -1 where status = " . ZendJobQueue::STATUS_SCHEDULED . ";
					INSERT OR REPLACE INTO version (key, version) VALUES ('SCHEMA_VERSION', '{$targetVersion}');
					";
				
					execQueries($this->dbh, $queries);
				}
				
				if (version_compare($dbCurrentVersion, "1.0.10", "<=")) {
				
					message("Upgrading from 1.0.10 to $targetVersion");
					$queries =
					"ALTER TABLE jobqueue_job ADD data_size INTEGER DEFAULT 0;
					UPDATE jobqueue_job set data_size = LENGTH(vars) + LENGTH(http_headers) + LENGTH(output) + LENGTH(error) where data_size = 0;
					INSERT OR REPLACE INTO version (key, version) VALUES ('SCHEMA_VERSION', '{$targetVersion}');
					";
				
					execQueries($this->dbh, $queries);
				}
				
				$dbh56 = $this->getZS56JqDb();
				if ($dbh56) {
					if ($this->isClusterUpgrade()) {
						
						$dbh6 = $this->getZS6JqDb();
						
						message("Moving jobs and recurring rules from ZS 5.6 (" . $dbh56->getAttribute(PDO::ATTR_DRIVER_NAME) . ")");
										
						$this->copySchdeulesAndJobs($dbh56, $dbh6);
					}
					
					$dbh56 = NULL;

					return;
				}
			} catch (PDOException $exc) {
				ERROR($exc->getMessage());
				//$this->dbh->rollback();
			}
		}
		
		try {
			message('Checking if MYSQL upgrade is in needed...');
			$dbIni = parse_ini_file($_GET['zsDir'] . "/etc/zend_database.ini");
			
			$oldDbIni = NULL;
			if (file_exists($_GET['zsDir'] . "/etc-5.6.0/monitor_node.ini")) {
				$oldDbIni = parse_ini_file($_GET['zsDir'] . "/etc-5.6.0/monitor_node.ini");
			}
					
			
			$dbType = strtolower($dbIni['zend.database.type']);
			if (strstr($dbType, "mysql")) {
				message("PE with MYSQL installation detected");
				$dbHost     = $dbIni['zend.database.host_name'];
				$dbPort     = $dbIni['zend.database.port'];
				$dbUsername = $dbIni['zend.database.user'];
				$dbPassword = $dbIni['zend.database.password'];
				$dbName     = $dbIni['zend.database.name'];
				$mysqlDbh = new PDO("mysql:host=$dbHost;port=$dbPort;dbname=$dbName",$dbUsername, $dbPassword);
				$mysqlDbh->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_WARNING);
				
				if ($oldDbIni) {
					$dbHostOld     = $dbIni['zend_monitor.database.host_name'];
					$dbPortOld     = $dbIni['zend_monitor.database.port'];
					$dbUsernameOld = $dbIni['zend_monitor.database.user'];
					$dbPasswordOld = $dbIni['zend_monitor.database.password'];
					$dbNameOld     = $dbIni['zend_monitor.database.name'];

				}
				
				
				$res = $mysqlDbh->query("SELECT property_value FROM schema_properties WHERE property = 'JOBQUEUE_SCHEMA_VERSION'");
				$jqSchemaVersion = false;
				foreach ($res as $row) {
					$jqSchemaVersion = $row['property_value'];
					break;						
				}
				if (!$jqSchemaVersion) {
					message("Job Queue tables not found in MySQL db, will create...");
					$mysqlCreationQueries = file_get_contents($_GET['zsDir'] . "/share/jobqueue_mysql_create_database.sql");
					
					$mysqlCreationQueries .= "
						REPLACE INTO schema_properties VALUES('JOBQUEUE_SCHEMA_VERSION', '{$targetVersion}');";
						
					try {			
						$mysqlDbh->beginTransaction();
						execQueries($mysqlDbh, $mysqlCreationQueries);	
						$mysqlDbh->commit();
					} catch (PDOException $exc) {
						ERROR($exc->getMessage());
						$mysqlDbh->rollback();
					}
				} else {
					message("Job Queue tables already created. will upgrade to $targetVersion");
					try {
						$mysqlDbh->beginTransaction();
						
						if ($jqSchemaVersion == "1.0.10") {
							
							$queries =
							"ALTER TABLE jobqueue_job ADD data_size INTEGER DEFAULT 0;
							UPDATE jobqueue_job set data_size = LENGTH(vars) + LENGTH(http_headers) + LENGTH(output) + LENGTH(error) where data_size = 0;							
							";
								
						} else if (version_compare($jqSchemaVersion, "1.0.9", "<=")) {
							message("Upgrading for <= 1.0.9");
							$queries = 
							"UPDATE jobqueue_schedule SET application_id = -1;
							UPDATE jobqueue_job set application_id = -1 where status = " . ZendJobQueue::STATUS_SCHEDULED . ";";
							
						}
						
						$queries .= "REPLACE INTO schema_properties VALUES('JOBQUEUE_SCHEMA_VERSION', '{$targetVersion}')";
						execQueries($mysqlDbh, $queries);
						
						$mysqlDbh->commit();
					} catch (PDOException $exc) {
						ERROR($exc->getMessage());
						$mysqlDbh->rollback();
					}
				}

				$dbh56 = $this->getMySqlDbh("5.6.0");
				if ($dbh56){
					$this->copySchdeulesAndJobs($dbh56, $this->getMySqlDbh());
				}
				
			} else {
				message("Non MYSQL configuration detected - found " . $dbType. ".");
			}			
			
		} catch (Exception $exc) {
			ERROR($exc->getMessage());				
		}
	}		
	
	
	/**
	 * 
	 * @param PDO $oldDb
	 * @param PDO $newDb
	 */
	private function copySchdeulesAndJobs($oldDb, $newDb) {
		
		try {
			$oldDb->beginTransaction();
			
			$oldDb->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_SILENT);
			$oldDb->exec("ALTER TABLE jobqueue_schedule ADD migrated INTEGER DEFAULT 0");
			$oldDb->exec("CREATE INDEX SCHEDULE_MIGRATED_IDX on jobqueue_schedule(migrated)");
			$oldDb->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
			
			$newDb->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
			
			$res = $oldDb->query("SELECT * from jobqueue_schedule WHERE migrated = 0");
			
			$newDb->beginTransaction();
			foreach ($res as $row) {
				message("Migrating schedule " . $row['id'] . " " . $row['script']);
				unset($row['migrated']);
				$values = $row;
				$values['application_id'] = -1;
				
				$stmt = $newDb->prepare("INSERT INTO jobqueue_schedule (id, status, queue_id, application_id, type, priority, persistent, timeout, schedule, name, script, vars, http_headers) VALUES (:id, :status, :queue_id, :application_id, :type, :priority, :persistent, :timeout, :schedule, :name, :script, :vars, :http_headers)");
				foreach($values as $key=>$value) {
					if (is_string($key)) {
						$stmt->bindValue(":" . $key, $value);
					}
				}
				$stmt->execute();											
			}
					
			$oldDb->exec("UPDATE jobqueue_schedule SET migrated = 1 WHERE migrated = 0");
			
			$oldDb->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_SILENT);
			$oldDb->exec("ALTER TABLE jobqueue_job ADD migrated INTEGER DEFAULT 0");
			$oldDb->exec("CREATE INDEX JOB_MIGRATED_IDX on jobqueue_job(migrated)");
			$oldDb->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_WARNING);
			
			$statusesToMigrate = array (
					ZendJobQueue::STATUS_SCHEDULED, 
					ZendJobQueue::STATUS_RUNNING,
					ZendJobQueue::STATUS_PENDING,
					ZendJobQueue::STATUS_WAITING_PREDECESSOR,
					ZendJobQueue::STATUS_SUSPENDED,
					ZendJobQueue::STATUS_REMOVED,
					);				
			$res = $oldDb->query("SELECT * from jobqueue_job WHERE migrated = 0 AND status IN (" . implode(",", $statusesToMigrate) . ")");
			
			foreach ($res as $row) {
				message("Migrating job " . $row['id'] . " " . $row['script']);
				unset($row['migrated']);
				$values = $row;
				$values['application_id'] = -1;
					
				$stmt = $newDb->prepare("INSERT INTO jobqueue_job (id, queue_id, application_id, type, priority, status, predecessor, persistent, timeout, schedule_time, schedule_id, creation_time, start_time, end_time, name, script, vars, http_headers, output, error, node_id) VALUES (:id, :queue_id, :application_id, :type, :priority, :status, :predecessor, :persistent, :timeout, :schedule_time, :schedule_id, :creation_time, :start_time, :end_time, :name, :script, :vars, :http_headers, :output, :error, :node_id)");
				foreach($values as $key=>$value) {
					if (is_string($key)) {
						$stmt->bindValue(":" . $key, $value);
					}
				}
				$stmt->execute();
			}
			
			$oldDb->exec("UPDATE jobqueue_job SET migrated = 1 WHERE migrated = 0");
			
			$newDb->commit();
			$oldDb->commit();
		} catch (Exception $ex) {
			
			error("copySchdeulesAndJobs: " . $ex->getMessage());
			
			if ($oldDb->inTransaction()) {
				$oldDb->rollBack();
			}
			if ($newDb->inTransaction()) {
				$newDb->rollBack();
			}
			
		}
	}
	
	function migrate()
	{
		message("Not migrating jobs from sqlite");
		return;
		
		if(ZEND_SERVER_APP_VERSION == 'PE') {
			
			$isActiveJqd = false;
			
			if (isset($_GET['dbHost'])) {
				message("PE with special MYSQL installation detected");
				$isActiveJqd = ($_GET['isActive'] == "1");
				$dbType		= "mysql";
				$dbHost     = $_GET['dbHost'];
				$dbPort     = $_GET['dbPort'];
				$dbUsername = $_GET['dbUsername'];
				$dbPassword = $_GET['dbPassword'];
				$dbName     = $_GET['dbName'];					
			} else {
				$dbIni = parse_ini_file($_GET['zsDir'] . "/etc/zend_database.ini");
				$dbType = strtolower($dbIni['zend.database.type']);	
				if (strstr($dbType, "mysql")) {
					message("Node with MYSQL installation detected");
					$dbHost     = $dbIni['zend.database.host_name'];
					$dbPort     = $dbIni['zend.database.port'];
					$dbUsername = $dbIni['zend.database.user'];
					$dbPassword = $dbIni['zend.database.password'];
					$dbName     = $dbIni['zend.database.name'];
				}
			}
			
			if (strstr($dbType, "mysql")) {
				
				$mysqlDbh = new PDO("mysql:host=$dbHost;port=$dbPort;dbname=$dbName",$dbUsername, $dbPassword);
				$mysqlDbh->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_WARNING);
				
				if (stripos(PHP_OS, "WIN") === 0) {
					$jqIni = parse_ini_file($_GET['zsDir'] . "/etc/cfg/jobqueue.ini.bak");
				} else {
					$jqIni = parse_ini_file($_GET['zsDir'] . "/etc/conf.d/jobqueue.ini.bak");
				}
				if (!$jqIni) {
					ERROR("Cannot find jobqueue.ini.bak that was saved by ZS upgrade. Will not migrate JQ database.");
					return;
				}
				
				$binding = $jqIni['zend_jobqueue.default_binding'];
				message("Current jq binding is $binding");
				if ($isActiveJqd || preg_match("/(\d+\.\d+\.\d+\.\d+)/", $binding, $matches)) {
					$theIp = $matches[1];
					message("Active jqd in $theIp");
										
					$myIps = $this->getMyIps();
					message("Detected local IP addresses - " . implode(" ", $myIps));
					if ($isActiveJqd) {
						message("Special JQD - Migrating JobQueue SQLITE database to MySql at $dbHost");
						$this->migrateToMysql($mysqlDbh);	
					} else if (!in_array($theIp, $myIps)) {
						message("Current ip is not of the active jqd");
					} else {
						message("Migrating JobQueue SQLITE database to MySql at $dbHost");
						$this->migrateToMysql($mysqlDbh);		
					}
				} else {
					message("Unable to find ip in binding");
				}
			} else {
				message("Non MYSQL installation detected. Will not migrate local db.");
			}
		} else {
			message("Will not migrate. Not in PE mode");
		}
	}
	
	private function migrateRow($targetDbh, $table, $uniqueFields, $row) {
	
		if ($uniqueFields) {
			//echo "reading existing id for table $table" . PHP_EOL;
			$query = "select id from $table where ";
			for ($i = 0 ; $i < count($uniqueFields) ; $i++) {
				if ($i > 0) {
					$query .= " and ";
				}
				$query .= $uniqueFields[$i] . " = ?";
			}
			//echo $query . PHP_EOL;
			
			$stmt = $targetDbh->prepare($query);
			
			for ($i = 0 ; $i < count($uniqueFields) ; $i++) {
				//echo "Binding " . $row[$uniqueFields[$i]] . PHP_EOL;
				$stmt->bindValue($i + 1, $row[$uniqueFields[$i]]);
			}
			$stmt->execute();
			$idRow = $stmt->fetch(PDO::FETCH_ASSOC);
			//var_dump($idRow);
			if ($idRow) { 
				return $idRow['id'];
			}
		}	
		
		
		$arrKeys = array_keys($row);
		$fields = implode(",", $arrKeys);
		$values = array_values($row);
		$params = implode(",", array_fill(0, count($arrKeys), '?'));
		$query = "INSERT INTO $table ($fields) VALUES ($params)";
		$stmt = $targetDbh->prepare($query);
		
		for ($i = 0 ; $i < count($arrKeys)  ; $i++) {
			if ($arrKeys[$i] == "id") {
				$stmt->bindValue($i + 1, NULL);
			} else if ($arrKeys[$i] == "schedule_time" && !$values[$i]) {
				$stmt->bindValue($i + 1, NULL);
			} else {
				$stmt->bindValue($i + 1, $values[$i]);
				//echo "Binding " . $values[$i] . PHP_EOL;
			}			
		}		
		$stmt->execute();
	
		return $targetDbh->lastInsertId();

	}
	
	private function markMigratedRows($idsMigrated, $table) {
		try {
			$this->dbh->beginTransaction();
			$firstId = $idsMigrated[0];
			$lastId = $idsMigrated[count($idsMigrated) - 1];
			message("marking rows from SQLITE table $table (ids $firstId to $lastId");
			$this->dbh->exec("UPDATE $table SET migrated_row = 1 WHERE id between $firstId AND $lastId");			
			$this->dbh->commit();
		} catch (PDOException $exc) {
			ERROR($exc->getMessage());
			$this->dbh->rollback();
		}			
	}
	
	
	private function migrateToMysql($mysqlDbh) {
		$queueIds = array();
		$scheduleIds = array();
		
		$tables = array ( 
				"jobqueue_queue" => array("name")
				, "jobqueue_application" => array("name")
				, "jobqueue_schedule" => array("queue_id", "application_id", "script", "schedule")
				, "jobqueue_job" => NULL
				);			
		
		$migratedQueues = array();
		$migratedApps = array();
		$migratedSchedules = array();
		$migratedJobs = array();
		
		foreach (array_keys($tables) as $table) { 
			message("Migrating table $table");
			
			$stmt1 = $this->dbh->prepare("ALTER TABLE $table ADD migrated_row BIT DEFAULT 0");
			if ($stmt1) {
				$stmt1->execute();
			}
			
			$stmt = $this->dbh->prepare("select * from $table WHERE migrated_row = 0 ORDER BY id ASC");
			$stmt->execute();
			$count = 0;
			
			$commitChunks = 1;
			$commitChunksSize = 10000;
			
			$idsMigrated = array();
			
			try {
				$mysqlDbh->beginTransaction();
				while ($row = $stmt->fetch(PDO::FETCH_ASSOC)) {	
					
					switch ($table) {
						case "jobqueue_schedule":
							$row['queue_id'] = $migratedQueues[$row['queue_id']];
							$row['application_id'] = $migratedApps[$row['application_id']];
							break;
						case "jobqueue_job":
							$row['queue_id'] = $migratedQueues[$row['queue_id']];
							$row['application_id'] = $migratedApps[$row['application_id']];
							if ($row['schedule_id'] > 0){
								$row['schedule_id'] = $migratedSchedules[$row['schedule_id']];
							}
							if ($row['predecessor'] > 0){
								$row['predecessor'] = $migratedJobs[$row['predecessor']];
							}
							break;
							
						default:
							break;					
					};			
					
				
					unset($row['migrated_row']);
					
					$newRowId = $this->migrateRow($mysqlDbh, $table, $tables[$table], $row);		
					
					switch ($table) {
						case "jobqueue_queue":
							$migratedQueues[$row['id']] = $newRowId;
							break;
						case "jobqueue_application":
							$migratedApps[$row['id']] = $newRowId;
							break;
						case "jobqueue_schedule":
							$migratedSchedules[$row['id']] = $newRowId;
							break;
						case "jobqueue_job":
							$migratedJobs[$row['id']] = $newRowId;
							break;
							
						default:
							break;					
					};	
					
					$idsMigrated[] = $row['id'];

					$count++;
					
					if ($count > ($commitChunks * $commitChunksSize) ) {
						message("Committing $commitChunksSize rows");
						$commitChunks++;
						$mysqlDbh->commit();
						
						$this->markMigratedRows($idsMigrated, $table);			
						
						$idsMigrated = array();
						$mysqlDbh->beginTransaction();
					}
				}
				
				if ($idsMigrated) {
					$this->markMigratedRows($idsMigrated, $table);
				}
				
				$mysqlDbh->commit();
				
				message($count . " rows migrated");
			} catch (PDOException $exc) {
				ERROR($exc->getMessage());
				$mysqlDbh->rollback();
			}
		}
		
		/*
		message("Migrated queues:");
		print_r ($migratedQueues);
		
		message("Migrated apps:");
		print_r ($migratedApps);
		
		message("Migrated schedules:");
		print_r ($migratedSchedules);	
		*/
	}
	
	protected function upgradeSqlite() {
	
	}
	
	protected function upgradeMysql() {
	
	
	}	
	
		
}
	