<?php 

	ini_set("error_reporting", E_ALL);
	ini_set("output_buffering", 0);
	ini_set("max_execution_time", 60 * 60); // one hour timeout
	ini_set("html_errors", 0);
	ini_set("memory_limit", -1);

	if(function_exists('zend_disable_statistics')){
		zend_disable_statistics();
	}

	function logMessage($msg) {
		$fLog = fopen(get_cfg_var("zend.log_dir") . "/zs_maintenance.log", "a+");
		fwrite($fLog,"[" . strftime("%c", time()) . "] " . $msg . PHP_EOL);
		fclose($fLog);
	}
	
	function error($msg) {
		$msg = "ERROR: " . $msg;
		logMessage($msg);
	}
	
	function notice($msg) {
		$msg = "NOTICE: " . $msg;
		logMessage($msg);	
	}

	/**
	 * @param PDO $zsdDb 
	 */
	function handleAudit($auditDb) {
		foreach ($auditDb->query('SELECT PROPERTY_VALUE FROM ZSD_AUDIT_PROPERTIES WHERE PROPERTY_NAME = \'AUDIT_HISTORY\' LIMIT 1') as $row) {
			$months = (int) $row['PROPERTY_VALUE'];			
		}
		
		notice("Audit history set to $months months");
		try {
			
			$timeLimit = time() - ($months * 60 * 60 * 24 * 30);
			
			$auditDb->beginTransaction();
			
			$deleted = 0;
			$deleted += $auditDb->exec("DELETE FROM ZSD_AUDIT WHERE CREATION_TIME < $timeLimit");
			$auditDb->exec("DELETE FROM ZSD_AUDIT_PROGRESS WHERE CREATION_TIME < $timeLimit");
			
			$auditDb->commit();
			
			notice($deleted . " audit entries deleted");
			
		} catch (Exception $ex) {
			$auditDb->rollback();
			error($ex->getMessage());
			return;
		}
	
	}
	
	/**
	 * @param PDO $zsdDb
	 */
	function handleZSD($zsd) {
		
		try {
				
			$timeLimit = time() - (60 * 60 * 24 * 7); //delete entries older than a week
			
			$zsd->beginTransaction();
				
			$deleted = 0;
			$deleted = $zsd->exec("DELETE FROM ZSD_NODES WHERE IS_DELETED = 1 AND LAST_UPDATED < $timeLimit");

			$zsd->exec("DELETE FROM ZSD_NODES_PROFILE WHERE NOT EXISTS (SELECT NODE_ID FROM ZSD_NODES WHERE ZSD_NODES.NODE_ID = ZSD_NODES_PROFILE.NODE_ID)");
			
			$zsd->commit();
				
			if ($deleted) {
				notice($deleted . " obsolete nodes entries deleted");
			}
				
		} catch (Exception $ex) {
			$zsd->rollback();
			error($ex->getMessage());
			return;
		}
	
	}
	
	/**
	 * 
	 * @param PDO $jqDb
	 */
	function handleJobqueue($jqDb) {
		$jqdIni = parse_ini_file(get_cfg_var("zend.conf_dir") . '/jqd.ini');
		$history = (int) $jqdIni['zend_jobqueue.history'];
		$historyFailed = (int)  $jqdIni['zend_jobqueue.history_failed'];
				
		$historyLimit = time() - ($history * 60 * 60 * 24); 
		$historyFailedLimit = time() - ($historyFailed * 60 * 60 * 24);
		
		notice("Job Queue history is $history days");
		notice("Job Queue failed history is $historyFailed");
		if (!$historyFailed) {
			$historyFailed = $history;
		}

		$jqDb->beginTransaction();

		try {
			$deleted = 0;
			$deleted += $jqDb->exec("DELETE FROM jobqueue_job where status = " . ZendJobQueue::STATUS_REMOVED);
			if ($history) {
				if ($jqDb->getAttribute(PDO::ATTR_DRIVER_NAME) == 'sqlite') {
					$stmt = $jqDb->prepare("DELETE FROM jobqueue_job where status IN (" . ZendJobQueue::STATUS_COMPLETED . "," . ZendJobQueue::STATUS_OK . ") AND persistent = 0 AND strftime('%s',end_time) - :historyLimit < 0 ");
				} else {
					$stmt = $jqDb->prepare("DELETE FROM jobqueue_job where status IN (" . ZendJobQueue::STATUS_COMPLETED . "," . ZendJobQueue::STATUS_OK . ") AND persistent = 0 AND UNIX_TIMESTAMP(end_time) - :historyLimit < 0 ");
				}
				$stmt->bindValue(":historyLimit", $historyLimit);
				$stmt->execute();
				$deleted += $stmt->rowCount();
			}
				
			if ($historyFailed) {
				if ($jqDb->getAttribute(PDO::ATTR_DRIVER_NAME) == 'sqlite') {
					$stmt = $jqDb->prepare("DELETE FROM jobqueue_job where status IN (" . ZendJobQueue::STATUS_FAILED . "," . ZendJobQueue::STATUS_LOGICALLY_FAILED . "," . ZendJobQueue::STATUS_TIMEOUT . ") AND persistent = 0 AND strftime('%s',end_time) - :historyLimit < 0 ");
				} else {
					$stmt = $jqDb->prepare("DELETE FROM jobqueue_job where status IN (" . ZendJobQueue::STATUS_FAILED . "," . ZendJobQueue::STATUS_LOGICALLY_FAILED . "," . ZendJobQueue::STATUS_TIMEOUT . ") AND persistent = 0 AND UNIX_TIMESTAMP(end_time) - :historyLimit < 0 ");
				}
				$stmt->bindValue(":historyLimit", $historyFailedLimit);
				$stmt->execute();
				$deleted += $stmt->rowCount();
			}
			
			$jqDb->commit();
			
			notice($deleted . " jobqueue jobs deleted");
			
		} catch (Exception $ex) {
			$jqDb->rollBack();
			error($ex->getMessage());
			return;
		}
		
		
		// delete completed and failed jobs that exceed the db size limit
		
		$completedDbSize = (int) $jqdIni['zend_jobqueue.db_size_completed'];
		$failedDbSize = (int)  $jqdIni['zend_jobqueue.db_size_failed'];
		
		$completedDbSize *= 1024; // in KB
		$failedDbSize *= 1024; // in KB
		
		$avg = false;
		$res = $jqDb->query("select avg(data_size) as avg from (select data_size from jobqueue_job order by id ASC limit 10000) as j");
		foreach ($res as $row) {
			$avg = intval($row['avg']);
		}	
		if (!$avg) {
			notice('No jobs to calculate average size');
		} else {		
			notice("Jobs average size is {$avg}kb");
			
			$res = $jqDb->query("select sum(data_size) as sum from jobqueue_job where status IN (" . ZendJobQueue::STATUS_COMPLETED . "," . ZendJobQueue::STATUS_OK . ")" );
			$sum = 0;
			foreach ($res as $row) {
				$sum = intval($row['sum']); // in KBytes
			}
			
			$sumMb = intval($sum / 1024);
			notice("Completed jobs take ~$sumMb MB in space");
			
			if ($sum > $completedDbSize) {
				
				$numToDelete = (int) (($sum - $completedDbSize) / $avg);
				notice("Will need to delete $numToDelete jobs");
				$jqDb->query("DELETE from jobqueue_job where id in (select id from jobqueue_job where status IN (" . ZendJobQueue::STATUS_COMPLETED . "," . ZendJobQueue::STATUS_OK . ") order by id ASC limit $numToDelete) ");
			}
			
			$res = $jqDb->query("select sum(data_size) as sum from jobqueue_job where status IN (" . ZendJobQueue::STATUS_FAILED . "," . ZendJobQueue::STATUS_LOGICALLY_FAILED . ")" );
			$sum = 0;
			foreach ($res as $row) {
				$sum = intval($row['sum']); // in KBytes
			}
			$sumMb = $sum / 1024; // now in MBYTES
			notice("Failed jobs take ~$sumMb MB in space");
			if ($sum > $failedDbSize) {
			
				$numToDelete = (int) ($sum - $failedDbSize) / $avg;
				notice("Will need to delete $numToDelete jobs");
				$jqDb->query("DELETE from jobqueue_job where id in (select id from jobqueue_job where status IN (" . ZendJobQueue::STATUS_FAILED . "," . ZendJobQueue::STATUS_LOGICALLY_FAILED . ") order by id ASC limit $numToDelete) ");
			}
		}
				
	}
	
	/**
	 *
	 * @param PDO $statsDb
	 */
	function handleStatistics($statsDb) {
		
		$zendConfDir = get_cfg_var("zend.conf_dir");
		if (strpos(PHP_OS, "WIN") === 0) {
			$ini = parse_ini_file($zendConfDir . "/cfg/statistics.ini");
		}  else {
			$ini = parse_ini_file($zendConfDir . "/conf.d/statistics.ini");
		}
		$dataDir = $ini["zend_statistics.data_dir"];
		$days = (int) $ini["zend_statistics.history_daily"];
		$weeks = (int) $ini["zend_statistics.history_weekly"];
		$months = (int) $ini["zend_statistics.history_monthly"];		
		
		notice("Statistics cleanup: $days days, $weeks weeks, $months months, data dir at $dataDir");
		
		if (!$days) {
			error("Cannot find zend_statistics.history_daily directive");
			$days = 1;
			$weeks = 1;
			$months = 2;		
		}		
		
		if ($dataDir) {
			$files = glob($dataDir . "/*");
			$now = time();
			$deleted = 0;
			foreach ($files as $file) {
				$secs = 60 * 60 * 24 * $days;
				$diff = $now - filemtime($file);
				if ($diff > $secs) {
					$deleted++;
					unlink($file);
				}
			}
			notice("Statistics cleanup: $deleted statistics temporary files deleted");
		}
			
		$statsDb->beginTransaction();
	
		try {
			$deleted = 0;
			
			$deleted += $statsDb->exec("DELETE FROM stats_daily where from_time < " . (time() - (60 * 60 * 24 * $days)));
			$deleted += $statsDb->exec("DELETE FROM stats_weekly where from_time < " . (time() - (60 * 60 * 24 * 7 * $weeks)));
			$deleted += $statsDb->exec("DELETE FROM stats_monthly where from_time < " . (time() - (60 * 60 * 24 * 30 * $months)));
			
			
			notice($deleted . " statistics records deleted");
			
			$statsDb->commit();
			
		} catch (Exception $ex) {
			$statsDb->rollBack();
			error($ex->getMessage());	
			return;		
		}		
		
	}
	
	/**
	 *
	 * @param PDO $monDb
	 * @param PDO $zsdDb
	 */
	function handleMonitor($monDb, $zsdDb) {
		
		notice("Started Monitor cleanup");
	
		if (!function_exists("zend_monitor_delete_old_issues")) {
			error("zend_monitor_delete_old_issues() function is not available");
			return;
		}
		$expiration = (int) ini_get("zend_monitor_ui.expiration_days");
		$closedItemsExpiration = (int) ini_get("zend_monitor_ui.expiration_days_closed");
		notice("Cleaning up with expiration $expiration");
		$arr = zend_monitor_delete_old_issues($expiration, $closedItemsExpiration);
                $result = count($arr);
              	notice("$result old issues were deleted");
		
		$nodes = array();
		foreach ($zsdDb->query('SELECT NODE_ID FROM ZSD_NODES') as $row) {
			$nodes[] = (int) $row['NODE_ID'];
		}
		if ($nodes) {
			notice("Servers found - Node Id: " . implode(",", $nodes));
		} else {
			notice("No servers found in cluster");
		}
		zend_monitor_delete_obsolete_issues($nodes);
		zend_monitor_delete_old_traces();
		
	}
	
	/**
	 *
	 * @param PDO $traceDb
	 */
	function handleCodeTracing($traceDb) {
	
		notice("Started Code Trace cleanup");
		
		$zendConfDir = get_cfg_var("zend.conf_dir");
		if (strpos(PHP_OS, "WIN") === 0) {
			$ini = parse_ini_file($zendConfDir . "/cfg/codetracing.ini");
		}  else {
			$ini = parse_ini_file($zendConfDir . "/conf.d/codetracing.ini");
		}
		
		$maxSpace = (int) $ini["zend_codetracing.max_disk_space"];
		$directory = get_cfg_var('zend.data_dir') . '/' . $ini['zend_codetracing.dump_file'];
		$directory = dirname($directory);
		$files = glob($directory . "/*.*");
		
		$totalSize = 0;
		foreach ($files as $file) {
			$totalSize += filesize($file);
		}
		$totalSize = (int) ($totalSize / 1024 / 1024); // to MB
		notice("Total disk space used - {$totalSize}MB");
		if ($totalSize > $maxSpace) {
			
			// first delete all amf files
			$files = glob($directory . "/*.amf");
			notice("Deleting " . count($files) . " .amf files" );
			$allOk = true;
			foreach ($files as $file) {
				$allOk = $allOk && unlink($file);
			}
			if (!$allOk) {
				error("Could not delete some amf files");
			}
			
			// now start cleaning the rest of the code trace files
			$files = glob($directory . "/*.*");
			$totalSize = 0;
			foreach ($files as $file) {
				$totalSize += filesize($file);
			}
			$totalSizeMB = (int) ($totalSize / 1024 / 1024); // to MB
			notice("Total disk space after amf cleanup - {$totalSizeMB}MB");
			if ($totalSizeMB > $maxSpace) {
				$allOk = true;
				$filesSorted = array();
				
				foreach ($files as $file) {
					$mtime = filemtime($file);
					$filesSorted[$mtime] = $file;
				}
				ksort($filesSorted);	// delete oldest files first								
					
				$filesDeleted = array();
				foreach ($filesSorted as $file) {
								
					$filesize = filesize($file);
					
					if (unlink($file)) {
						$filesDeleted[] = $file;							
						$totalSize -= $filesize;
						$totalSizeMB = $totalSize / 1024 / 1024;
						if ($totalSizeMB < $maxSpace) {
							break;
						}
							
					} else {
						$allOk = false;
					}
				}
				
				$traceIds = array();
				foreach ($filesDeleted as $fileToDelete) {
					$idStart = stripos($fileToDelete, "dump.");
					if ($idStart > 0) {
						$traceIds[] = substr($fileToDelete, $idStart + 5);
					}
				}
				
				$traceDb->beginTransaction();
				foreach ($traceIds as $traceId) {
					$traceDb->exec("DELETE FROM trace_files where trace_id = '" . $traceId . "'");
				}
				$traceDb->commit();
				
				if ($filesDeleted) {
                	if(zend_monitor_delete_tracing_data($filesDeleted)){
                    	notice(count($filesDeleted) . " files of traces were deleted");
                    }
				}
			}
		}
	
	}
	
	$tz = @date_default_timezone_get();
	ini_set("date.timezone", $tz); 
	
	notice("Started maintenance");
	
	$clusterMode = false;
	$zsdDb = NULL;	
	$zendDatabaseIni = parse_ini_file(get_cfg_var("zend.conf_dir") . '/zend_database.ini');
	if ($zendDatabaseIni['zend.database.type'] == "SQLITE") {
		$zsdDb = new PDO("sqlite:" . get_cfg_var("zend.data_dir") . "/db/zsd.db" );
		$monitorDb = new PDO("sqlite:" . get_cfg_var("zend.data_dir") . "/db/monitor.db" );
		$codeTracingDb = new PDO("sqlite:" . get_cfg_var("zend.data_dir") . "/db/codetracing.db" );
		$statisticsDb = new PDO("sqlite:" . get_cfg_var("zend.data_dir") . "/db/statistics.db" );
		$jobqueueDb = new PDO("sqlite:" . get_cfg_var("zend.data_dir") . "/db/jobqueue.db" );
	} else {
		$clusterMode = true;
		$zsdDb = new PDO("mysql:host=" . $zendDatabaseIni['zend.database.host_name'] . ";port=" . $zendDatabaseIni['zend.database.port'] . ";dbname=" . $zendDatabaseIni['zend.database.name'], $zendDatabaseIni['zend.database.user'], $zendDatabaseIni['zend.database.password']);
		$monitorDb = $zsdDb;
		$codeTracingDb = $zsdDb;
		$statisticsDb = $zsdDb;
		$jobqueueDb = $zsdDb;
	}
	
	$zsdDb->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
	$monitorDb->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
	$codeTracingDb->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
	$statisticsDb->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
	$jobqueueDb->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);

	if ($clusterMode) {
		foreach ($zsdDb->query('SELECT GET_LOCK(\'ZS_MAINTENANCE\', 1)') as $row) {
			if ($row[0] === "0") {
				notice("ZS_MAINTENANCE already running");
				exit(1);
			}
		}
	}

	handleAudit($zsdDb);
	handleZSD($zsdDb);
	handleJobqueue($jobqueueDb);
	handleStatistics($statisticsDb);
	handleMonitor($monitorDb, $zsdDb);
	handleCodeTracing($codeTracingDb);
?>
