<?php

/// Include our db settings
require_once dirname(__FILE__) . '/zs_db_settings.php';

class ZendDbOptimizer
{

    protected $m_dbConnection;
    protected $m_dbSettings;

    /**
     * Construct ZendDbOptimizer class. This function also establishes a db connection based
     * on the INI file zend_database.ini
     */
    public function __construct() {
        try {

            $this->m_dbSettings = ZSDbSettings::fromIni($_GET['zsDir'] . "/etc/zend_database.ini");
            if ( $this->m_dbSettings->isMysql() ) {
                $this->m_dbConnection = new PDO("mysql:host=" . $this->m_dbSettings->getDbHost() . ";port=" . $this->m_dbSettings->getDbPort() . ";dbname=" . $this->m_dbSettings->getDbName(),
                                                $this->m_dbSettings->getDbUser(),
                                                $this->m_dbSettings->getDbPassword());
                $this->m_dbConnection->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_WARNING);
            } else {
                /// SQLite

            }

        } catch ( PDOException $exec ) {

        }
    }

    /**
     * Fetch list of tables from the database and
     * perform an OPTIMIZE query on them
     * Note: this function only works for MYSQL and does nothing for SQLITE database
     */
    public function optimize() {
        if ( $this->m_dbSettings->isMysql() ) {
            $rows = $this->m_dbConnection->query("show tables")->fetchAll();
            foreach ($rows as $row ) {
                echo "Optimzing table " . $row[0] . "\n";
                $optimizeSql = "OPTIMIZE TABLE " . $row[0];
                $statement = $this->m_dbConnection->prepare($optimizeSql);
                $statement->execute();
                $statement->closeCursor();
            }
        }
    }
}

/**
 * Main
 */

ini_set("error_reporting", E_ALL);
ini_set("output_buffering", 0);
ini_set("max_execution_time", 60 * 60); // one hour timeout
ini_set("html_errors", 0);
ini_set("memory_limit", -1);

$optimizer = new ZendDbOptimizer();
$optimizer->optimize();
