<?php

require_once dirname(__FILE__) . "/zs_component_upgrader.php";

class StatisticsDBHandler extends ZSComponentUpgrader {
	
	protected function getSqliteDbName($version = "6.0.0") {
		return "statistics.db";
	}
	
	protected function getMysqlSqlFilenames() {
		return array("statistics_mysql_create_database.sql", "stats_mysql_fixtures.sql");
	}
	
	protected function getSqliteSqlFilenames() {
		return array("statistics_sqlite_create_database.sql", "stats_sqlite_fixtures.sql");
	}
	
	protected function upgradeSqlite() {
		if (file_exists($this->getSqliteDbPath())) {
			message("Database at " . $this->getSqliteDbPath() . " already exists");
			return;
		}
		
		$fromVersion = $this->copySqliteFromPreviousVersions();
		if (!$fromVersion) {
			$this->createCleanSqliteDatabase();
			return;
		}
		
		message("Upgrade from $fromVersion");
		
		$dbh = $this->getSqliteDbh();
		
		$dbh->exec("UPDATE stats_daily set entry_type_id = " . ZEND_STATS_TYPE_MON_NUM_OF_WARNING_EVENTS . " WHERE entry_type_id = " . ZEND_STATS_TYPE_MON_NUM_OF_EVENTS);
		$dbh->exec("UPDATE stats_weekly set entry_type_id = " . ZEND_STATS_TYPE_MON_NUM_OF_WARNING_EVENTS . " WHERE entry_type_id = " . ZEND_STATS_TYPE_MON_NUM_OF_EVENTS);
		$dbh->exec("UPDATE stats_monthly set entry_type_id = " . ZEND_STATS_TYPE_MON_NUM_OF_WARNING_EVENTS . " WHERE entry_type_id = " . ZEND_STATS_TYPE_MON_NUM_OF_EVENTS);
		
		$dbh->exec("DROP TABLE IF EXISTS schema_properties"); // need to make 'name' column unique
		
		foreach ($this->getSqliteSqlFilenames() as $sqlFile) {
			message("Executing $sqlFile ");
			$sql = file_get_contents($this->getSqlsDir() . "/" . $sqlFile);
			$this->execQueriesDiscardDuplicates($dbh, $sql);
		}
	}
	
	
	
	protected function getMysqlSchemaVersion($dbh) {
		$res = $dbh->query("SELECT property_value FROM schema_properties WHERE property = 'STATS_SCHEMA_VERSION'")->fetchAll();
		foreach ($res as $row) {
			return $row['property_value'];
		}
		
		return false;
	}	
		
	protected function upgradeMysql() {
		
		$upgradeFromZSVersion = $this->getUpgradeFromVersion();
		if ($upgradeFromZSVersion[0] == '5') {
			$this->createCleanMysqlDatabase();
			return;
		}
		
		$dbh = $this->getMySqlDbh();
		if (!$dbh) {
			return;
		}
		
		$fromVersion = $this->getMysqlSchemaVersion($dbh);
						
		$this->createCleanMysqlDatabase();
		
		if (!$fromVersion) {
			
			$dbh->exec("UPDATE stats_daily set entry_type_id = " . ZEND_STATS_TYPE_MON_NUM_OF_WARNING_EVENTS . " WHERE entry_type_id = " . ZEND_STATS_TYPE_MON_NUM_OF_EVENTS);
			$dbh->exec("UPDATE stats_weekly set entry_type_id = " . ZEND_STATS_TYPE_MON_NUM_OF_WARNING_EVENTS . " WHERE entry_type_id = " . ZEND_STATS_TYPE_MON_NUM_OF_EVENTS);
			$dbh->exec("UPDATE stats_monthly set entry_type_id = " . ZEND_STATS_TYPE_MON_NUM_OF_WARNING_EVENTS . " WHERE entry_type_id = " . ZEND_STATS_TYPE_MON_NUM_OF_EVENTS);
			
			$dbh->exec("REPLACE INTO schema_properties (property, property_value) VALUES('STATS_SCHEMA_VERSION', '1.0.11')");
						
		} else {
		
			$dbh6 = $this->getMySqlDbh("6.0.0");
			if ($dbh6) {
				message("Upgrade from 6.0.0 - doing nothing");
				return;
			}
		}
	}
	
	public function upgrade() {
		$this->upgradeSqlite();
		
		if ($this->isClusterUpgrade()) {
			$this->upgradeMysql();
		}
	}
	
}

?>