<?php

require_once dirname(__FILE__) . "/zs_component_upgrader.php";


const CURRENT_SCHEMA_VERSION = "6.0.1";

class ZSDDbCreator extends ZSComponentUpgrader {

	protected function getSqliteDbName($version = "6.0.0") {
		return "zsd.db";
	}
	
	protected function getMysqlSqlFilenames() {
		return array(
				"zsd_mysql_create_schema.sql"
				, "mysql_create_pagecache_rules_db.sql"
				, "mysql_create_monitor_rules_db.sql"
				, "create_monitor_rules_fixtures.sql"
				, "mysql_create_message_center_db.sql"	
				);
	}
	
	protected function getSqliteSqlFilenames() {
		return array(
				"zsd_sqlite_create_schema.sql"
				, "create_pagecache_rules_db.sql"
				, "create_monitor_rules_db.sql"
				, "create_monitor_rules_fixtures.sql"
				, "sqlite_create_message_center_db.sql"				
				);
	}
	
	protected function importLicenseFromBlueprint() {
	
		if ($this->getUpgradeFromVersion() != "5.6.0") {
			message("Not an upgrade from 5.6.0 - no need to import license");
			return;
		}
		
		message("Importing license from 6.0.0 blueprint if exists");
	
		$dbh = $this->getUpgradedMySqlDbh();
		if (!$dbh) {
			$dbh = $this->getSqliteDbh("6.0.0", false);
			if ($dbh) {
				message("Importing from SQLITE DB");
			} else {
				message("6.0.0 blueprint not found");
				return false;
			}
		} else {
			message("Importing from MYSQL DB");
		}
		
		if ($dbh) {
			$serial = "";
			if ($this->isTableExists($dbh, "ZSD_DIRECTIVES")) {
				$res = $dbh->query("select * from ZSD_DIRECTIVES WHERE NAME IN ('zend.serial_number', 'zend.user_name')")->fetchAll();
				if ($res) {
					foreach ($res as $row) {
						
						if ($row['NAME'] == 'zend.serial_number') {
							$serial = $row['DISK_VALUE'];
						} else if ($row['NAME'] == 'zend.user_name') {
							$user = $row['DISK_VALUE'];
						}
					}					
				}
			}
	
			if ($serial) {
				message("License found ($user)");
				$this->updateLicense($user, $serial);
				return $user;
			} else {
				message("Could not find license in blueprint");
			}
		}
		
		return false;
	}
	
	protected function upgradeSqlite() {
		
		$upgradeFromVersion = $this->copySqliteFromPreviousVersions();
		$clearBlueprint = file_exists($this->getSqliteDbPath());
		$createSchema = true;
		
		if (!$upgradeFromVersion || $this->getUpgradeFromVersion() == "5.6.0") {
			if (file_exists($this->getSqliteDbPath())) {
				message("Database at " . $this->getSqliteDbPath() . " already exists");
				$clearBlueprint = true;
				$createSchema = false;				
				
			} else {
				$this->createCleanSqliteDatabase();		
				$createSchema = false;
				$clearBlueprint = false;
				
				$this->importPageCacheRules($this->getSqliteDbh());				
			}
		} 
		
		if ($createSchema && !file_exists($this->getSqliteDbPath())) {
			$this->createCleanSqliteDatabase();			
		} 
		
		if ($clearBlueprint && file_exists($this->getSqliteDbPath())) {
			message("Database at " . $this->getSqliteDbPath() . " already exists - clearing blueprint");				
			$this->clearBlueprint($this->getSqliteDbh());					
		}
		
		
		// upgrade to 6.1
		$dbh = $this->getSqliteDbh();
		
		// First make sure we got the schema version table
		$this->addSQLiteZSDSchemaVersionTableIfNeeded($dbh);
		
		$currentSchemaVersion = $this->getCurrentSchemaVersionFromDB($dbh);
		message( "Current ZSD schema version is: " . $currentSchemaVersion );
		
		if ( $currentSchemaVersion == 6001 ) {
			$this->upgradeTo61( $dbh  );
			// update the version number in the database to 6.1 (6100) (MAJOR * 1000) + (MINOR * 100) + (MINOR-MINOR)
			$this->setCurrentDbSchemaVersion( $dbh, 6100 );
		}
	}
	
	/**
	 * set the current database version
	 * @param PDO $dbh
	 * @param integer $versionNumber
	 */
	protected function setCurrentDbSchemaVersion( $dbh, $versionNumber ) {
		$stmnt = $dbh->prepare("REPLACE INTO ZSD_SCHEMA_PROPERTIES (NAME, VERSION_NUMBER) VALUES ( 'schema_version', :VERSION_NUMBER )");
		$stmnt->bindValue(":VERSION_NUMBER", $versionNumber);
		$stmnt->execute();
	}
	
	/**
	 * Upgrade ZSD database from 6.0.1/0 to 6.1
	 * @param PDO $dbh
	 */
	protected function upgradeTo61( $dbh ) {
		message( "Upgrading ZSD database to 6.1" );
		
		// In ZS 6.1 the blueprint should no longer contain
		// Extension Manager 'dir' directives
		message( "Deleting zend_extension_manager.dir.* directives from ZSD_DIRECTIVES table..." );
		$dbh->exec("DELETE FROM ZSD_DIRECTIVES WHERE NAME LIKE 'zend_extension_manager.dir.%'");
		
		// Fix apc entry in the database
		$apcLoaded    = $this->isAPCLoaded( $dbh );
		$apcInstalled = $this->isAPCInstalled($dbh);
		
		message( "Current APC status: installed: " . ($apcInstalled ? "Yes" : "No") . ", loaded: " . ($apcLoaded ? "Yes" : "No") );
		message( "Upgrading APC entries..." );
		
		if ( $apcInstalled ) {
			// Don't touch the entry in the database, only add new one
			// with the string 'APC Compatibility Layer'
			$this->addAPCExntesionEntry( $dbh, 'APC Compatibility Layer', false, 'apc_compatibility_layer.ini' );
				
		} else if ( $apcLoaded && !$apcInstalled ) {
			// APC is loaded but not installed
			// -> the apc entry is actually representing the fake APC
			// update this entry to 'not loaded' and add new 'APC Compatibility Layer' entry
			$this->addAPCExntesionEntry($dbh, 'APC Compatibility Layer', true, 'apc_compatibility_layer.ini');
			$this->addAPCExntesionEntry($dbh, 'apc', false, 'apc.ini');
				
		} else {
			// not loaded and not installed.
			// Just add the 'APC Compatibility Layer entry
			$this->addAPCExntesionEntry($dbh, 'APC Compatibility Layer', false, 'apc_compatibility_layer.ini');
		}
	}
	/**
	 * @param PDO $dbh
	 */
	protected function addSQLiteZSDSchemaVersionTableIfNeeded( $dbh ) {
		// if the table exists, do nothing
		if ( $this->isTableExists($dbh, "ZSD_SCHEMA_PROPERTIES") ) {
			message( "ZSD_SCHEMA_PROPERTIES table already exists - will not add it" );
			return;
		}
		
		message( "ZSD_SCHEMA_PROPERTIES table does not exist - will add it" );
		// Create the table
		$stmnt = $dbh->prepare("CREATE TABLE IF NOT EXISTS ZSD_SCHEMA_PROPERTIES (NAME VARCHAR(256) NOT NULL PRIMARY KEY UNIQUE, VERSION_NUMBER INTEGER)");
		$stmnt->execute();
		
		message( "Updating ZSD schema version to 6001" );
		
		$stmnt2 = $dbh->prepare("INSERT OR IGNORE INTO ZSD_SCHEMA_PROPERTIES (NAME, VERSION_NUMBER) VALUES ( 'schema_version', 6001 )");
		$stmnt2->execute();
	}
	
	/**
	 * return the current database schema from the database
	 * @param unknown $dbh
	 */
	protected function getCurrentSchemaVersionFromDB( $dbh ) {
		$res = $dbh->query("SELECT VERSION_NUMBER from ZSD_SCHEMA_PROPERTIES where NAME = 'schema_version'");
		foreach ($res as $row) {
			return $row['VERSION_NUMBER'];
		}
		/// default value 6.0.1 
		return 6001;
	}
	
	/**
	 * @brief insert or replace entry into the ZSD_EXTENSIONS table
	 * @param PDO $dbh
	 * @param bool $loaded
	 * @param string $iniFile
	 */
	protected function addAPCExntesionEntry( $dbh, $name, $loaded, $iniFile ) {
		$stmt = $dbh->prepare("REPLACE INTO ZSD_EXTENSIONS (NAME, IS_INSTALLED, IS_LOADED, INI_FILE, IS_ZEND_COMPONENT) VALUES (:NAME, :IS_INSTALLED, :IS_LOADED, :INI_FILE, :IS_ZEND_COMPONENT)");
		$stmt->bindValue(":NAME",              $name);
		$stmt->bindValue(":IS_INSTALLED",      0);
		$stmt->bindValue(":IS_LOADED",         ($loaded ? 1 : 0));
		$stmt->bindValue(":INI_FILE",          (string) $iniFile);
		$stmt->bindValue(":IS_ZEND_COMPONENT", 0);
		$stmt->execute();
	}
	
	/**
	 * @brief return true if apc is installed in the database
	 * @param PDO $dbh
	 * @return true on success, false otherwise
	 */
	protected function isAPCInstalled( $dbh ) {
		$res = $dbh->query("SELECT IS_INSTALLED,IS_LOADED from ZSD_EXTENSIONS where NAME = 'apc'");
		foreach ($res as $row) {
			if ( $row['IS_INSTALLED'] ) {
				// if IS_INSTALLED is 1 it means that ZSD was able to locate 
				// the apc.so on the file system
				return true;
			}
		}
		return false;
	}
	
	/**
	 * @brief return true if apc is loaded
	 * @param PDO $dbh
	 * @return true on success, false otherwise
	 */
	protected function isAPCLoaded( $dbh ) {
		$res = $dbh->query("SELECT IS_LOADED from ZSD_EXTENSIONS where NAME = 'apc'");
		foreach ($res as $row) {
			if ( $row['IS_LOADED'] ) {
				return true;
			}
		}
		return false;
	}
	
	protected function clearMysqlBlueprintIfNeeded() {
		
		$clear = false;
		
		$dbh = $this->getMySqlDbh();
		if (!$dbh) {
			message("Cannot clear blueprint");
			return;
		} 
		
		$profilesTableExists = $this->isTableExists($dbh, "ZSD_NODES_PROFILE");
		$versions = array();
		if ($profilesTableExists) {
			$res = $dbh->query("SELECT PHPVERSION FROM ZSD_NODES_PROFILE group by PHPVERSION")->fetchAll();
			foreach ($res as $row) {
				$versions[] = $row['PHPVERSION'];
			}
		}
		message("PHP versions found in profiles - " . implode(",", $versions));
		if (!in_array(phpversion(), $versions)){
			message("Installation version " . phpversion() . "not in profiles");
			$clear = true;
		}
		if ($this->getUpgradeFromVersion() == "6.0.0") {
			$zsVersion = $this->getCurrentSchemaVersionFromMySQLDB($dbh);
			message("ZS version in schema properties - " . $zsVersion);
			if ($zsVersion == "6.0.0") {
				$clear = true;
			}
		}
				
		if ($clear) {
			message("Clearing blueprint..");
			$this->clearBlueprint($dbh);
		}		
	}
	
	
	/**
	 * 
	 * @param PDO $dbh
	 * @param string $version
	 */
	protected function updateMySQLSchemaVersion($dbh, $version) {
		try {
			$dbh->exec("REPLACE INTO schema_properties VALUES ('ZSD_VERSION', '" . $version . "')");
		} catch (Exception $ex) {
			error("Cannot update schema version - " . $ex->getMessage());
			return false;
		}
		
		return true;
	}
	
	/**
	 * return the current database schema version of ZSD
	 * @param PDO $dbh
	 */
	protected function getMySQLSchemaVersion($dbh) {
		try {
			$res = $dbh->query("SELECT property_value from schema_properties where property = 'ZSD_VERSION'");
			foreach ($res as $row) {
				return $row['property_value'];
			}
			return false;
			
		} catch (PDOException $e) {
			error( $e->getMessage() );
			return  false;
		}
	}
	
	protected function upgradeMysql() {
	
		$this->createCleanMysqlDatabase();
	
		if ($this->getUpgradeFromVersion() == "5.6.0") {
			$dbh56 = $this->getMySqlDbh("5.6.0");
			if ($dbh56)	{
				$dbh56->query("use ZendServer;");
				$this->importPageCacheRules($dbh56);
			}
		}
		
		$dbh = $this->getMySqlDbh();
		if (!$dbh) {
			return;
		}
		
		$schemaVersion = $this->getMySQLSchemaVersion($dbh);
		message("Current schema version is " . $schemaVersion);
				
		$this->clearMysqlBlueprintIfNeeded();
		
		if (!$schemaVersion || version_compare('6.0.1', $schemaVersion, ">")) {
			message("Updating schema version to 6.0.1");
			$this->updateMySQLSchemaVersion($dbh, '6.0.1');
			$schemaVersion = '6.0.1';
		}
		
		message( "Current ZSD schema version is: " . $schemaVersion );
		$dbh = $this->getMysqlDbh();
		
		if ( $schemaVersion == '6.0.1' ) {
			$this->upgradeTo61( $dbh  );
			// update the version number in the database to 6.1 (6100) (MAJOR * 1000) + (MINOR * 100) + (MINOR-MINOR)
			$this->updateMySQLSchemaVersion( $dbh, '6.1' );
		}
	}
	
	
	/**
	 * 
	 * @param PDO $dbh
	 */
	protected function clearBlueprint($dbh) {
		try {
			
			$dbh->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
			
			$dbh->beginTransaction();
			
			$dbh->exec("DELETE FROM ZSD_DIRECTIVES;");
			$dbh->exec("DELETE FROM ZSD_EXTENSIONS;");
			
			$dbh->commit();
			
		} catch (Exception $ex) {
			$dbh->rollBack();
			message($ex->getMessage());		
		}
	}
	
	public function upgrade() {
		$this->upgradeSqlite();
	
		$licenseName = "";
		$upgradeFromVersion = $this->getUpgradeFromVersion();
		if ( $upgradeFromVersion == "6.0.0") {
			$licenseName = $this->importLicenseFromBlueprint();
		} else if ($upgradeFromVersion == "5.6.0") {
			if (!$this->importLicenseFromBlueprint()) {
				$this->updateLicense("", "");
			}
		}

		if ($this->isClusterUpgrade()) {
			$this->upgradeMysql();
		}
	}	
	
	private function updateLicense($user, $serial) {
		
		message("Update license ($user)");
		
		$monIni = parse_ini_file($_GET['zsDir'] . "/etc/monitor_node.ini");
		$globalsIniPath = $monIni['zend_monitor.global_directives_ini_file'];
		$ini = file($globalsIniPath);
		foreach ($ini as $line => $str) {
			if (strstr($str, "zend.user_name")) {
				$ini[$line] = "zend.user_name=" . $user . PHP_EOL;
			}
			if (strstr($str, "zend.serial_number")) {
				$ini[$line] = "zend.serial_number=" . $serial . PHP_EOL;
			}
		}

		message("Updating license in $globalsIniPath");
		file_put_contents($globalsIniPath, implode("", $ini));
	}
	
	public function updateNodeId($id) {
	
		message("Updating node id to $id");
	
		$monIni = parse_ini_file($_GET['zsDir'] . "/etc/monitor_node.ini");
		$globalsIniPath = $monIni['zend_monitor.global_directives_ini_file'];
		$ini = file($globalsIniPath);
		foreach ($ini as $line => $str) {
			if (strstr($str, "zend.node_id")) {
				$ini[$line] = "zend.node_id=$id" . PHP_EOL;
			}			
		}
	
		file_put_contents($globalsIniPath, implode("", $ini));
	}
	
	public function updateClusterNodeId($id, $mysqlIp) {
	
		$id = "$id@$mysqlIp@ZendServer@" . time();
		message("Updating cluster node id - $id ");
	
		$zsdIni = $_GET['zsDir'] . "/etc/zsd.ini";
		$ini = file($zsdIni);
		foreach ($ini as $line => $str) {
			if (strstr($str, "zend_server_daemon.cluster_node_id")) {
				$ini[$line] = "zend_server_daemon.cluster_node_id=$id" . PHP_EOL;
				break;
			}			
		}
	
		file_put_contents($zsdIni, implode("", $ini));
	}
	
	private function importPageCacheRules($dbh) {
		
		message("Importing Page Cache rules");
		
		$res = $dbh->query("SELECT count(*) as count from ZSD_PAGECACHE_RULES");
		foreach ($res as $row) {
			if ($row['count'] > 0) {
				message("DB already contains page cache rules");
				return;
			}
		}		
		
		$rules = simplexml_load_file($this->sqlFile = $_GET['zsDir'] . "/etc/pagecache_rules.xml");
		
		try {
			$dbh->beginTransaction();
			foreach ($rules->cache->url as $rule) { /* @var $rule SimpleXMLElement */
				$path = "http://" . $rule['host'] . "/" . $rule['path'];
				message("Found page cache rule " . $rule['id'] . ": $path");
				
				$stmt = $dbh->prepare("INSERT INTO ZSD_PAGECACHE_RULES (RULE_ID, APP_ID, URL, NAME, META_XML) VALUES (:RULE_ID, :APP_ID, :URL, :NAME, :META_XML)");
				$stmt->bindValue(":RULE_ID", $rule['id']);
				$stmt->bindValue(":APP_ID", -1);
				$stmt->bindValue(":URL", $path);
				$stmt->bindValue(":NAME", (string) $rule->name);
				$stmt->bindValue(":META_XML", strstr($rule->asXML(), "<url"));
				$stmt->execute();			
			}	
			$dbh->commit();
		} catch (Exception $ex) {
			$dbh->rollBack();
			error($ex->getMessage());
		}		
	}	
	
}


