#!/bin/bash
ZSST_PLUGIN_NAME="Zend Server Information"
exec >> $ZEND_ERROR_LOG 2>&1


# root-specific commands
if [ ! $NONSU ]; then
	$ZCE_PREFIX/bin/zendctl.sh status > $ZEND_DATA_TMPDIR/zs_status.txt 2>&1
fi




# Users start
zenduser=$(id -a zend)
apacheuser=$(id -a ${WEB_USER})
gduser=$(grep "zend.httpd_" $ZCE_PREFIX/etc/conf.d/ZendGlobalDirectives.ini)

cat > $ZEND_DATA_TMPDIR/zs_users.txt <<EOUSERS

Zend User:
$zenduser


Apache User:
$apacheuser


In Global Directives:
$gduser


EOUSERS
# Users end


# Manifest start
mkdir $ZEND_DATA_TMPDIR/zs_manifest

$ZCE_PREFIX/bin/ZManifest -vi $ZCE_PREFIX/lib/ZendExtensionManager.so >  $ZEND_DATA_TMPDIR/zs_manifest/ZendExtensionManager.so.txt

for ext in $ZCE_PREFIX/lib/*/php-*.*.x/*.so; do
	$ZCE_PREFIX/bin/ZManifest -vi  $ext > $ZEND_DATA_TMPDIR/zs_manifest/$(basename $ext).txt
done

while read daemon; do
	$ZCE_PREFIX/bin/ZManifest -vi  $ZCE_PREFIX/bin/$daemon > $ZEND_DATA_TMPDIR/zs_manifest/bin_$daemon.txt
done <<EODEAMONS
jqd
MonitorNode
php
php-cgi
scd
watchdog
zdd
zdpack
zmd
zsd
EODEAMONS
# Manifest end

while read command; do
	$command >> $ZEND_DATA_TMPDIR/zs_$(echo "$command" | cut -d " " -f 1).txt
done <<EOCMD
lsof +D $ZCE_PREFIX
du -sh $ZCE_PREFIX/
du -sh $ZCE_PREFIX/*
du -sh $ZCE_PREFIX/var/*
du -sh $ZCE_PREFIX/tmp/*
EOCMD


# MD5 sums start
echo "bin :" > $ZEND_DATA_TMPDIR/zs_md5.txt
md5sum $ZCE_PREFIX/bin/* >> $ZEND_DATA_TMPDIR/zs_md5.txt
echo "lib :" >> $ZEND_DATA_TMPDIR/zs_md5.txt
find $ZCE_PREFIX/lib -type f -exec md5sum {} \; >> $ZEND_DATA_TMPDIR/zs_md5.txt
# MD5 sums end


ls -RAlF $ZCE_PREFIX > $ZEND_DATA_TMPDIR/zs_dir.txt

cp /etc/zce.rc $ZEND_DATA_TMPDIR/zs_rc.txt
cp -R $ZCE_PREFIX/etc $ZEND_DATA_TMPDIR/zend_etc
cp -R $ZCE_PREFIX/gui/lighttpd/etc $ZEND_DATA_TMPDIR/lighttpd_etc
cp -R $ZCE_PREFIX/gui/config $ZEND_DATA_TMPDIR/gui_config

# Databases are not accessible for a regular user
if [ ! $NONSU ]; then

. $ZCE_PREFIX/share/support_tool/db_params.lib

	if [ "$dbtype" == "SQLITE" ]; then
		mkdir $ZEND_DATA_TMPDIR/zs_sqlite
		for dbfile in $ZCE_PREFIX/var/db/*; do
			echo -n "$dbfile : " >> $ZEND_DATA_TMPDIR/zs_sqlite/integrity.txt
			sqlite3 $dbfile "PRAGMA integrity_check;" >> $ZEND_DATA_TMPDIR/zs_sqlite/integrity.txt
		done
		
		# codetracing.db and zsd.db don't have a version record
		while read db table; do
			echo "$db :" >> $ZEND_DATA_TMPDIR/zs_sqlite/schema_ver.txt
			sqlite3 -line $ZCE_PREFIX/var/db/$db "SELECT * from ${table};" >> $ZEND_DATA_TMPDIR/zs_sqlite/schema_ver.txt
			echo >> $ZEND_DATA_TMPDIR/zs_sqlite/schema_ver.txt
		done <<EODBS
deployment.db deployment_properties
gui.db gui_metadata
jobqueue.db version
monitor.db version
statistics.db schema_properties
EODBS

		if [ $GETSQLITE ]; then
			cp $ZCE_PREFIX/var/db/* $ZEND_DATA_TMPDIR/zs_sqlite/
		fi

	elif [ "$dbtype" == "MYSQL" ]; then
		mkdir $ZEND_DATA_TMPDIR/zs_mysql
		ZENDDB_MYSQL_COMMANDS="$(cat <<EOT
		
SELECT TABLE_SCHEMA AS 'Database', CONCAT(SUM(ROUND(((DATA_LENGTH + INDEX_LENGTH) / 1024 / 1024),2)),' MB') AS 'Size' FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_SCHEMA = '${dbname}';

SELECT TABLE_NAME AS 'Table', CONCAT(ROUND(((DATA_LENGTH + INDEX_LENGTH) / 1024 / 1024),2),' MB') AS 'Size', TABLE_ROWS AS 'Records' FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_SCHEMA = '${dbname}';

SELECT CONCAT(ROUND((DATA_FREE / 1024 / 1024),2),' MB') AS 'Allocated to InnoDB' FROM INFORMATION_SCHEMA.TABLES WHERE ENGINE = 'InnoDB' LIMIT 1;

SELECT property, property_value FROM ${dbname}.schema_properties;

SHOW PROCESSLIST;

EOT
)"

		$MYSQL_EXEC "SHOW VARIABLES;" > $ZEND_DATA_TMPDIR/zs_mysql/variables.txt
		$MYSQL_EXEC "$ZENDDB_MYSQL_COMMANDS" > $ZEND_DATA_TMPDIR/zs_mysql/zend_db_info.txt
		
	else
		echo  "Could not determine the database type" >> $ZEND_ERROR_LOG
	fi
fi


# Apache configuration
if [ -d /etc/httpd ]; then
	cp -R /etc/httpd $ZEND_DATA_TMPDIR/apache_config
elif [ -d /etc/apache ]; then
	cp -R /etc/apache $ZEND_DATA_TMPDIR/apache_config
elif [ -d /etc/apache2 ]; then
	cp -R /etc/apache2 $ZEND_DATA_TMPDIR/apache_config
else
	echo "Apache configuration not found" >> ${ZEND_ERROR_LOG}
fi
