<?php

date_default_timezone_set(@date_default_timezone_get()); // avoid getting date.timezone warnings

/**
 * Zend Server CLI Tools - main executable script
 * 
 * This script expects to run with a first parameter that specifies the tool
 * name. All other parameters are passed as is to the tool. 
 * 
 * Additionally, the PHP include path should be set to include the CLI tools
 * library and Zend Framework.
 * 
 * @author    Shahar Evron <shahar.e@zend.com>
 * @copyright (c) 2011 Zend Technologies LTD, all rights reserved
 * @license   ???
 */

/** Load Autoloader and register namespaces **/
require_once 'Zend/Loader/Autoloader.php';
Zend_Loader_Autoloader::getInstance()->registerNamespace(array('ZendX_', 'ZServer_'));

/** Create standard output stream **/
$outStream = new ZServer_Cli_OutputStream_Standard();

try {
    
    if ($_SERVER['argc'] < 2) {
        throw new ZServer_Cli_Tool_Exception("too few arguments", 1);
    }
    
    // Read the command argument and remove it from the arguments list
    $args = $_SERVER['argv'];
    array_shift($args); // First argument is the command, we don't need it
    $toolName = array_shift($args); // Second argument is the tool to use

    // Load the right tool class and parse command line arguments
    $tool = ZServer_Cli_Tool::toolFactory($toolName)
        ->setOutputStream($outStream)
        ->parseArguments($args);
    
    if (getenv('DEBUG')) { 
        $tool->enableDebug();
        $outStream->setExtraEolChars(PHP_EOL);
    }

    // Run!
    $tool->run();
        
} catch (ZServer_Cli_Tool_Exception $ex) {
    // Catch-all error handling here...
    $outStream->error("Error: " . $ex->getMessage() . PHP_EOL); 
    $outStream->debug((string) $ex . PHP_EOL);
    
    // Exit with exception error code
    exit($ex->getCode());
}
