<?php

class ZServer_Cli_OutputStream_Standard implements ZServer_Cli_OutputStream_Interface
{
    protected $_stdout;
    
    protected $_stdoutStreamName;
    
    protected $_stderr;
    
    protected $_stderrStreamName;
    
    protected $_debug = false;
    
    protected $_eolChar = '';
    
    public function __construct($stdout = 'php://stdout', $stderr = 'php://stderr', $append = true)
    {
        $mode = ($append ? 'a' : 'w');
        
        $this->_stdout = fopen($stdout, $mode);
        $this->_stderr = fopen($stderr, $mode);
        
        $this->_stdoutStreamName = $stdout;
        $this->_stderrStreamName = $stderr;
    }
    
    public function enableDebug($on = true)
    {
        $this->_debug = (bool) $on;
    }
    
    public function write($message)
    {
        fprintf($this->_stdout, "%s", $message . $this->_eolChar);
    }
    
    public function error($message)
    {
        fprintf($this->_stderr, "%s", $message . $this->_eolChar);
    }
    
    public function debug($message)
    {
        if ($this->_debug) { 
            fprintf($this->_stderr, "DEBUG: %s", $message . $this->_eolChar);
        }
    }
    
    public function getStdOutStreamName()
    {
        return $this->_stdoutStreamName;
    }
    
    public function getStdErrStreamName()
    {
        return $this->_stderrStreamName;
    }
    
    /**
     * Set extra characters to add ad the end of each line written to output.
     *  
     * Can be useful for debugging purposes.
     *  
     * @param  string $chars
     * @return ZServer_Cli_OutputStream_Standard
     */
    public function setExtraEolChars($chars)
    {
        $this->_eolChar = $chars;
        return $this;
    }
}