<?php

class ZServer_Cli_Tool_Command_Manage_ClusterEnableServer extends ZServer_Cli_Tool_Command_Manage_Abstract
{
    protected $_description = 'enable a disabled Zend Server Cluster Manager cluster member';
    
    protected $_usageMessage = '<server ID> [options]';
    
    protected $_args = array(
    	'r|retry=i' => 'retry action <value> times if server is locked; default is 3',
        'w|wait=i'  => 'wait <value> seconds between retires; default is 5',
        's|sync'    => 'run synchronously; wait for enablement process to complete',
    );
    
    protected function _run()
    {
        $retry = $this->_tool->getArguments()->r;
        $wait  = $this->_tool->getArguments()->w;
        $sync  = $this->_tool->getArguments()->s;
        
        $args = $this->_tool->getArguments()->getRemainingArgs();
        $serverId = (int) array_shift($args);
        
        if (! $serverId) { 
            // Error: missing server ID
            throw new ZServer_Cli_Tool_Command_Exception("missing or invalid server ID", ZServer_Cli_Tool::EXIT_INVALID_ARG);
        }
        
        if (! empty($args)) {
            // Error: too many arguments
            throw new ZServer_Cli_Tool_Command_Exception("unexpected number of arguments, expecting exactly one argument", ZServer_Cli_Tool::EXIT_INVALID_ARG);
        }
        
	   	if ($retry < 1) $retry = 3;  
		if ($wait < 1) $wait = 5;        
        $this->_getZsApiClient()->setRetryIfLocked($retry, $wait);
        
        $serverInfo = $this->_getZsApiClient()->clusterEnableServer($serverId);
        
        if ($sync && $serverInfo->getStatus() == ZendX_Service_ZendServer_ServerInfo::STARTING_UP) { 
            $serverInfo = $this->_waitUntilServerStatusChange($serverInfo);
        }
        
        $this->_printServerInfo($serverInfo);
    }
}