<?php

class ZServer_Cli_Tool_Command_Manage_ChangePassword extends ZServer_Cli_Tool_Command_Manage_Abstract
{
    protected $_description = 'Change a current user password';
    
    protected $_usageMessage = '<Old password> <New Password> ';
    
    protected $_args = array(
    	'o|old=i' => 'retry action <value> times if server is locked; default is 3',
        'n|new=i'  => 'wait <value> seconds between retires; default is 5',
    );
    
    protected function _run()
    {
        $old = $this->_tool->getArguments()->o;
        $new  = $this->_tool->getArguments()->n;
        
        $args = $this->_tool->getArguments()->getRemainingArgs();
        $serverId = (int) array_shift($args);
        
        if (! $serverId) { 
            // Error: missing server ID
            throw new ZServer_Cli_Tool_Command_Exception("missing or invalid server ID", ZServer_Cli_Tool::EXIT_INVALID_ARG);
        }
        
        if (! empty($args)) {
            // Error: too many arguments
            throw new ZServer_Cli_Tool_Command_Exception("unexpected number of arguments, expecting exactly one argument", ZServer_Cli_Tool::EXIT_INVALID_ARG);
        }
        
        $serverInfo = $this->_getZsApiClient()->clusterEnableServer($serverId);
        
        if ($sync && $serverInfo->getStatus() == ZendX_Service_ZendServer_ServerInfo::STARTING_UP) { 
            $serverInfo = $this->_waitUntilServerStatusChange($serverInfo);
        }
        
        $this->_printServerInfo($serverInfo);
    }
}