<?php

class ZServer_Cli_Tool_Command_Manage_SystemInfo extends ZServer_Cli_Tool_Command_Manage_Abstract
{
    protected $_description = 'get information about the remote Zend Server system';
    
    protected $_usageMessage = '';
    
    protected function _run()
    {
        $systemInfo = $this->_getZsApiClient()->getSystemInfo();
        $this->_printSystemInfoOutput($systemInfo);
    }
    
    /**
	 * Print system information
	 * 
	 * @param ZendX_Service_ZendServer_SystemInfo $systemInfo
     */
    protected function _printSystemInfoOutput(ZendX_Service_ZendServer_SystemInfo $systemInfo)
    {
        $output = "SYSTEMINFO\t{$this->_getServerUrl()}\t" . 
             "{$systemInfo->getOperatingSystem()}\t" .
        	 "{$systemInfo->getStatus()}\t" . 
             "{$systemInfo->getEdition()} {$systemInfo->getZendServerVersion()}\t" .
             "PHP {$systemInfo->getPHPVersion()}" . PHP_EOL;

        $output .= $this->_getLicenseInfoOutput($systemInfo->getServerLicenseInfo(), 'SERVER');
        $output .= $this->_getLicenseInfoOutput($systemInfo->getManagerLicenseInfo(), 'MANAGER');
        
        $this->_tool->output()->write($output);
    }
    
    protected function _getLicenseInfoOutput(ZendX_Service_ZendServer_LicenseInfo $licenseInfo, $edition)
    {
        if ($licenseInfo->getValidUntil()) { 
            $validUntil = date('Y-m-d', $licenseInfo->getValidUntil());
        } else {
            $validUntil = '';
        }
        
        return "\tLICENSEINFO $edition\t" . 
            "{$licenseInfo->getStatus()}\t" . 
            "{$licenseInfo->getOrderNumber()}\t" . 
            "{$validUntil}\t" . 
            "{$licenseInfo->getNodeLimit()}" . PHP_EOL;
    }
}