<?php

class ZServer_Cli_Tool_Command_Manage_applicationDeploy extends ZServer_Cli_Tool_Command_Manage_Abstract
{
    protected $_description = 'deploy an application';

	protected $_usageMessage = '-p <pkgFile> -b <baseUrl> [options]';
    
	protected $_args = array(
		'p|pkgFile=s'     	=> 'full path to the package file (required)',
		'b|baseUrl=s'    	=> 'Base URL to deploy the application to. If only path given, assuming deployment to default Vhost (required)',
		'a|appName-s'     	=> 'user defined application identifier - if not specified, the baseUrl parameter will be used',
		'u|userParams-s'    => 'Set values for user parameters defined in the package - space delimited (i.e.: "a=1 b=2")',
		'd|defaultServer'   => 'set to TRUE to Deploy the application on the default server (vhost); the base URL host provided will be ignored and replaced with <default-server>. In case of a conjunction of this parameter and createVhost, the latter will be ignored. Default is FALSE',
		'c|createVhost'     => 'Create a virtual host based on the base URL if such a virtual host was not already created by Zend Server (default is NOT to create one)',	
		'g|ignoreFailures'	=> 'Ignore failures during staging if only some servers reported failures (default is NOT to ignore errors)',	
    );    
    
    
    protected function _run()
    {
		$pkgFile  = $this->_tool->getArguments()->p;
		$baseUrl  = $this->_tool->getArguments()->b;
		$this->_tool->getArguments()->a ? $appName = $this->_tool->getArguments()->a : $appName = null;
		$this->_tool->getArguments()->u ? $userParams = $this->_getUsersParams($this->_tool->getArguments()->u) : $userParams = array();		
		$this->_tool->getArguments()->c ? $createVhost = $this->_tool->getArguments()->c : $createVhost = false;		
		$this->_tool->getArguments()->g ? $ignoreFailures = $this->_tool->getArguments()->g : $ignoreFailures = false;
		$this->_tool->getArguments()->d ? $defaultServer = $this->_tool->getArguments()->d : $defaultServer = false;
		
		if (!$pkgFile) { 
			throw new ZServer_Cli_Tool_Command_Exception("pkgFile not specified", ZServer_Cli_Tool::EXIT_INVALID_ARG);
		}
    	
		if (!is_file($pkgFile)) { 
			throw new ZServer_Cli_Tool_Command_Exception("pkgFile [$pkgFile] is not a valid file", ZServer_Cli_Tool::EXIT_INVALID_ARG);
		}
		    		
		if (!$baseUrl) { 
			throw new ZServer_Cli_Tool_Command_Exception("baseUrl not specified", ZServer_Cli_Tool::EXIT_INVALID_ARG);
		}
        
		$serverInfo = $this->_getZsApiClient()->applicationDeploy($pkgFile, $baseUrl, $appName, $userParams, $createVhost, $ignoreFailures, $defaultServer);
		$this->_printApplicationInfo($serverInfo);
    }
    
    protected function getUsersParams($params) {
    	$user_params = array();
    	$param_couples = explode(' ', $params);
    	
    	foreach ($param_couples as $couple) {
    		@list($key, $value) = explode('=', $couple);
    		if ($value === null) throw new ZServer_Cli_Tool_Command_Exception('userParams not passed properly - should be of the form of "a=1 b=2"', ZServer_Cli_Tool::EXIT_INVALID_ARG);
    		$user_params[$key] = $value;
    	}
    	
		return $user_params;
    }
}