<?php

class ZServer_Cli_Tool_Command_Manage_applicationUpdate extends ZServer_Cli_Tool_Command_Manage_Abstract
{
	protected $_description = 'update an application';

	protected $_usageMessage = '-i <appId> -p <pkgFile> [options]';
	
	protected $_args = array(
		'i|id=i'	 		=> 'Application ID to update (required)',
		'p|pkgFile=s'	 	=> 'full path to the package file (required)',
		'u|userParams-s'	=> 'Set values for user parameters defined in the package - space delimited (i.e.: "a=1 b=2")',
		'g|ignoreFailures'	=> 'Ignore failures during staging if only some servers reported failures (default is NOT to ignore errors)',	
	);	  
	
	protected function _run()
	{
		$pkgFile  = $this->_tool->getArguments()->p;
		
		if (!$pkgFile) { 
			throw new ZServer_Cli_Tool_Command_Exception("pkgFile not specified", ZServer_Cli_Tool::EXIT_INVALID_ARG);
		}
		
		if (!is_file($pkgFile)) { 
			throw new ZServer_Cli_Tool_Command_Exception("pkgFile [$pkgFile] is not a valid file", ZServer_Cli_Tool::EXIT_INVALID_ARG);
		}		
			
		if (!isset($this->_tool->getArguments()->i)) {			
			throw new ZServer_Cli_Tool_Command_Exception("appId not specified", ZServer_Cli_Tool::EXIT_INVALID_ARG);
		}
				
		$appId = (integer) $this->_tool->getArguments()->i;
		$this->_tool->getArguments()->u ? $userParams = $this->_getUsersParams($this->_tool->getArguments()->u) : $userParams = array();		
		$this->_tool->getArguments()->g ? $ignoreFailures = $this->_tool->getArguments()->g : $ignoreFailures = false;		
		
		$serverInfo = $this->_getZsApiClient()->applicationUpdate($appId, $pkgFile, $userParams, $ignoreFailures);
		$this->_printApplicationInfo($serverInfo);
	}
}