<?php

class ZServer_Cli_Tool_Command_Manage_bootstrapSingleServer extends ZServer_Cli_Tool_Command_Manage_Abstract
{
	protected $_description = 'Bootstrap a server for standalone usage in production or development environment.';
	
	protected $_usageMessage = '-p <password> -o <license order number> -l <license key> -a <accept eula TRUE/FALSE> [options]';
	
	protected $_args = array(
		'p|adminPassword=a'	 => 'The new administrator password to store for ZS\'s gui authentication (required)',
		'o|orderNumber-s'	  => 'License order number to store in the server\'s configuration. This license can be obtained from zend.com', 
		'l|licenseKey-s'	  => 'License key to store in the server\'s configuration. This license can be obtained from zend.com',
		'a|acceptEula=s'	  => 'Must be set to "TRUE" to accept ZS6\'s EULA (required)',
		'r|production-s'	  => 'Bootstrap this server using the factory \'production\' usage profile. Default value: "TRUE"',
		'u|appUrl-s'	      => 'The default application URL to use when displaying and handling deployed application URLs in the UI. Default: empty',
		'e|adminEmail-s'	  => 'The default Email to use when sending notifications about events, audit entries and other features',
		'd|devPassword-s'	  => 'The new developer user password to be stored for authentication. If no password is supplied, the developer user will not be created',
		);
	
	
	protected function _run()
	{
		$adminPassword  = $this->_tool->getArguments()->p;
		$orderNumber   	= $this->_tool->getArguments()->o;
		$licenseKey		= $this->_tool->getArguments()->l;
		$acceptEula		= $this->_tool->getArguments()->a;
		$production		= $this->_tool->getArguments()->r;
		$applicationUrl = $this->_tool->getArguments()->u;
		$adminEmail		= $this->_tool->getArguments()->e;
		$developerPassword= $this->_tool->getArguments()->d;
		
		if (! $adminPassword) {
			throw new ZServer_Cli_Tool_Command_Exception("Admin password not specified", ZServer_Cli_Tool::EXIT_INVALID_ARG);
		}
		
		if ( $acceptEula ==! "TRUE") {
			throw new ZServer_Cli_Tool_Command_Exception("Accept Eula  not 'TRUE'", ZServer_Cli_Tool::EXIT_INVALID_ARG);
		}
		
		$serverBootstrapInfo = $this->_getZsApiClient()->bootstrapSingleServer(
				$adminPassword, 
				$orderNumber, 
				$licenseKey, 
				$acceptEula, 
				$production, 
				$applicationUrl, 
				$adminEmail, 
				$developerPassword);
		
		 $output = //$serverBootstrapInfo->getSuccess() . "\t" .
				$serverBootstrapInfo->getApiKeyName() . "\t" .
				$serverBootstrapInfo->getApiKeyHash() . PHP_EOL; 
		$this->_tool->output()->write($output);
	}
}