<?php

class ZServer_Cli_Tool_Command_Manage_serverAddToCluster extends ZServer_Cli_Tool_Command_Manage_Abstract
{
	protected $_description = 'add a new (bootstrapped) Zend server to Cluster. Connects directly to MySQL';
	
	protected $_usageMessage = '-n <name> -i <node IP> -o <DB Host> -u <DB user name> -p <password>  [options]';
	
	protected $_args = array(
		'n|name=a'	 => 'name to give new server (required)',
		//'u|url=a'	  => 'new server\'s full ZS GUI URL (required)',
		'i|nodeIP=a'	  => 'new server\'s IP (required)',
		'o|dbHost=a'	  => 'Database host address to join the cluster (required)',
		'u|dbUsername=a'  => 'Database credentials - username (required)',
		'p|dbPassword=a'  => 'Database credentials - password (required)',
		'd|dbName=a'  	  => 'Database (Schema) name to connect or create. Default "ZendServer"',
		's|sync'    => 'run synchronously; wait till server completed all related actions like deploying all apps',
		'r|retry=i' => 'retry action <value> times if server is locked; default is 3',
		'w|wait=i'  => 'wait <value> seconds between retires; default is 5'
		);
	
	
	protected function _run()
	{
		$serverName  = $this->_tool->getArguments()->n;
		//$serverUrl   = $this->_tool->getArguments()->u;
		$nodeIp  = $this->_tool->getArguments()->i;
		$dbHost  = $this->_tool->getArguments()->o;
		$dbUsername = $this->_tool->getArguments()->u;
		$dbPassword = $this->_tool->getArguments()->p;
		$dbName = $this->_tool->getArguments()->d;
		$sync  = $this->_tool->getArguments()->s;
		$retry = $this->_tool->getArguments()->r;
		$wait  = $this->_tool->getArguments()->w;		
		
		if (! $serverName) {
			throw new ZServer_Cli_Tool_Command_Exception("server name not specified", ZServer_Cli_Tool::EXIT_INVALID_ARG);
		}
		
// 		if (! $serverUrl) { 
// 			throw new ZServer_Cli_Tool_Command_Exception("server URL not specified", ZServer_Cli_Tool::EXIT_INVALID_ARG);
// 		}
		if (! $nodeIp) {
			throw new ZServer_Cli_Tool_Command_Exception("Node IP not specified", ZServer_Cli_Tool::EXIT_INVALID_ARG);
		}
		if (! $dbHost) {
			throw new ZServer_Cli_Tool_Command_Exception("DB Host not specified", ZServer_Cli_Tool::EXIT_INVALID_ARG);
		}
		if (! $dbUsername) {
			throw new ZServer_Cli_Tool_Command_Exception("DB user name not specified", ZServer_Cli_Tool::EXIT_INVALID_ARG);
		}
		if (! $dbPassword) {
			throw new ZServer_Cli_Tool_Command_Exception("DB Password not specified", ZServer_Cli_Tool::EXIT_INVALID_ARG);
		}
		if (! $dbName) {
			$dbName="ZendServer";
		}
	   	if ($retry < 1) $retry = 4;  
		if ($wait < 1) $wait = 4;		
		$this->_getZsApiClient()->setRetryIfLocked($retry, $wait);
				
		$serverAddToClusterResponse = $this->_getZsApiClient ()->serverAddToCluster ( $serverName, $dbHost, $dbUsername, $dbPassword, $nodeIp, $dbName );
		$serverInfo=$serverAddToClusterResponse['serverInfo'];
		$newWebApiKeyName=$serverAddToClusterResponse['clusterAdminKey']['apiKeyName'];
		$newWebApiKey=$serverAddToClusterResponse['clusterAdminKey']['apiKeyHash'];
		$serverId = $serverInfo->getId ();
		$status=$serverInfo->getStatus();
	if ($sync) {
			$c = 0;
			for($i = 0; $i < 300; $i ++) {
				try{
					$newServerInfo = $this->_getZsApiClient ($newWebApiKey,$newWebApiKeyName)->clusterGetServerStatus ( array (
							$serverId 
					) );
					$info = $newServerInfo [$serverId];
					$status = $info->getStatus ();
					//echo "nodeId is $serverId Status is $status \n";
					if ($status === "OK" || $status === "Error") {
						$c++;
						if ($c == 13)
							break;
					} else
						$c = 0;
				}catch (ZendX_Service_ZendServer_Exception $ex) {
				}
				sleep ( 1 );
			}
		}
		
				$this->_tool->output ()->write ( "NODE_ID = $serverId\nWEB_API_KEY = $newWebApiKeyName\nWEB_API_KEY_HASH = $newWebApiKey\n");
	}
}