<?php

class ZServer_Cli_Tool_Command_Setup_AddApiKey extends ZServer_Cli_Tool_Command_Abstract
{
    const EXIT_INVALID_KEY = 51;
    
    protected $_description = 'create a new Zend Server API key';
    
    protected $_usageMessage = '<key name> [options]';
    
    protected $_args = array(
        'k|key=w' => 'secret key value (by default a random key will be generated)',
        'a|admin' => 'create a full access key (default is read-only key)'
    );
    
    public function run()
    {
        $args = $this->_tool->getArguments()->getRemainingArgs();
        $keyName = array_shift($args);
        
        if (! $keyName) { 
            // Error: missing order number
            throw new ZServer_Cli_Tool_Command_Exception("key name not specified. Run with --help for usage information", ZServer_Cli_Tool::EXIT_INVALID_ARG);
        }
        
        if (! empty($args)) {
            // Error: too many arguments
            throw new ZServer_Cli_Tool_Command_Exception("unexpected number of arguments, expecting only one argument", ZServer_Cli_Tool::EXIT_INVALID_ARG);
        }
        
        $keyHash = null;
        if ($this->_tool->getArguments()->k) { 
            $keyHash = $this->_tool->getArguments()->k;
            // test does not use ctype_xdigit because it may not be availble in PHP CLI
            if (! preg_match("/^[[:xdigit:]]{64}$/", $keyHash)) { 
                throw new ZServer_Cli_Tool_Command_Exception("Secret key is expected to be a 64-digit long hexadecimal number", ZServer_Cli_Tool::EXIT_INVALID_ARG);
            }
        }
        // Set the new password
        $this->_addKey(
            $keyName, 
            (bool) $this->_tool->getArguments()->a,
            $keyHash
        );
    }
    
    protected function _addKey($keyName, $adminKey, $keyHash = '')
    {
        $keyRing = new Api_Auth_Keyring();
        
        try {
            $key = $keyRing->addKey(
                $keyName, 
                ($adminKey ? Api_Auth_Acl_Abstract::ROLE_FULL_ACCESS : Api_Auth_Acl_Abstract::ROLE_READ_ONLY),
                $keyHash
            );
            
        } catch (Zwas_Exception $ex) {             
            $this->_tool->output()->debug($ex->getMessageObject()->__toString());
            throw new ZServer_Cli_Tool_Command_Exception("unable to add api key: " . $ex->getMessageObject()->__toString(), self::EXIT_INVALID_KEY);
       }
        
        $this->_tool->output()->write($key->getHash() . PHP_EOL);
    }
}