<?php

class ZServer_Cli_Tool_Command_Setup_SetLicense extends ZServer_Cli_Tool_Command_Abstract
{
    protected $_description = 'set the Zend Server or ZSCM license key';
    
    protected $_usageMessage = '<order number> <license key>';
    
    public function run()
    {
        $args = $this->_tool->getArguments()->getRemainingArgs();
        $orderNumber = array_shift($args);
        $licenseKey = array_shift($args);
        
        if (! ($orderNumber && $licenseKey)) { 
            // Error: missing order number
            throw new ZServer_Cli_Tool_Command_Exception("missing required arguments. Run with --help for usage information", ZServer_Cli_Tool::EXIT_INVALID_ARG);
        }
        
        if (! empty($args)) {
            // Error: too many arguments
            throw new ZServer_Cli_Tool_Command_Exception("unexpected number of arguments, expecting exactly two arguments", ZServer_Cli_Tool::EXIT_INVALID_ARG);
        }
        
        // Set the new license
        $this->_setLicense($orderNumber, $licenseKey);
    }
    
    /**
     * Set the new licnense key
     * 
     * @param string $orderNumber
     * @param string $licenseKey
     */
    protected function _setLicense($orderNumber, $licenseKey)
    {
        try {
            // This needs to be silenced to avoid Zend Utils warnings :(
            @Context_Version::getLicenseModel(array())->setLicense($orderNumber, $licenseKey);
        } catch (Zwas_Exception $ex) { 
            $this->_tool->output()->debug($ex->getMessageObject()->__toString());
            throw new ZServer_Cli_Tool_Command_Exception("unable to set license key: " . $ex->getMessageObject()->__toString(), ZServer_Cli_Tool::EXIT_INVALID_ARG);
        }
    }
}