<?php

class ZServer_Cli_Tool_Command_Setup_SetNodesLicense extends ZServer_Cli_Tool_Command_Abstract
{
    protected $_description = 'set the license to be used by ZSCM cluster members';
    
    protected $_usageMessage = '<order number> <license key>';

    public function run()
    {
        if (ZEND_SERVER_APP_VERSION != 'EE') { 
            throw new ZServer_Cli_Tool_Command_Exception("this command can only be used on a ZSCM installation", ZServer_Cli_Tool::EXIT_UNKNOWN);
        }
        
        $args = $this->_tool->getArguments()->getRemainingArgs();
        $orderNumber = array_shift($args);
        $licenseKey = array_shift($args);
        
        if (! ($orderNumber && $licenseKey)) { 
            // Error: missing order number
            throw new ZServer_Cli_Tool_Command_Exception("missing required arguments. Run with --help for usage information", ZServer_Cli_Tool::EXIT_INVALID_ARG);
        }
        
        if (! empty($args)) {
            // Error: too many arguments
            throw new ZServer_Cli_Tool_Command_Exception("unexpected number of arguments, expecting exactly two arguments", ZServer_Cli_Tool::EXIT_INVALID_ARG);
        }
        
        // Set the new license
        $this->_setNodesLicense($orderNumber, $licenseKey);
    }
    
    /**
     * Set the new nodes licnense key
     * 
     * @param string $orderNumber
     * @param string $licenseKey
     * 
     * @todo this is broken due to problematic Zend_Utils integration
     */
    protected function _setNodesLicense($orderNumber, $licenseKey)
    {
        try {
             ApplicationModel::setNodesLicense($orderNumber, $licenseKey);
            
        } catch (Zwas_Exception $ex) {             
            $this->_tool->output()->debug($ex->getMessageObject()->__toString());
            throw new ZServer_Cli_Tool_Command_Exception("unable to set node license key: " . $ex->getMessageObject()->__toString(), ZServer_Cli_Tool::EXIT_UNKNOWN);
        }
    }
}