<?php

class ZServer_Cli_Tool_Command_Setup_ShowApiKey extends ZServer_Cli_Tool_Command_Abstract
{
    const EXIT_UNKNOWN_KEY = 51;
    
    protected $_description = 'show information about a specific API key';
    
    protected $_usageMessage = '<key name> [-s]';
    
    protected $_args = array(
        's|showkey' => 'show secret key; if not specified secret key will not be displayed'
    );
    
    public function run()
    {
        $args = $this->_tool->getArguments()->getRemainingArgs();
        $keyName = array_shift($args);
        
        if (! $keyName) { 
            // Error: missing order number
            throw new ZServer_Cli_Tool_Command_Exception("key name not specified. Run with --help for usage information", ZServer_Cli_Tool::EXIT_INVALID_ARG);
        }
        
        if (! empty($args)) {
            // Error: too many arguments
            throw new ZServer_Cli_Tool_Command_Exception("unexpected number of arguments, expecting only one argument", ZServer_Cli_Tool::EXIT_INVALID_ARG);
        }
        
        $this->_showKey($keyName, (bool) $this->_tool->getArguments()->s);
    }
    
    protected function _showKey($keyName, $showSecret)
    {
        $keyRing = new Api_Auth_Keyring();
        
        try {
            $key = $keyRing->getKey($keyName);
        } catch (Zwas_Exception $ex) {
            $this->_tool->output()->debug($ex->getMessageObject()->__toString());
            throw new ZServer_Cli_Tool_Command_Exception($ex->getMessageObject()->__toString(), self::EXIT_UNKNOWN_KEY);
        }
        
        if ($key)  {
            $this->_tool->output()->write(self::printApiKeyDetails($key, $showSecret));
        }            
    }
    
    /**
     * Return output-ready information about the provided key
     *
     * @param  Api_Auth_Key $key
     * @param  boolean      $showSecret
     * @return string
     */
    static public function printApiKeyDetails(Api_Auth_Key $key, $showSecret = false)
    {
        $role = ($key->getRole() == Api_Auth_Acl_Abstract::ROLE_FULL_ACCESS ? 'fullaccess' : 'readonly');
        $output = $key->getName() . "\t" . 
                  $role . "\t" .
                  date("Y-m-d H:i:s", $key->getCreationTime()) . "\t" . 
                  ($showSecret ? $key->getHash() : '') . PHP_EOL;
        
        return $output;
    }
}