<?php

class ZServer_Cli_Tool_Setup extends ZServer_Cli_Tool 
{
    const EXIT_ZSERROR = 31;
    
    protected $_toolName = 'zs-setup';
    
    protected $_toolDescription = 'Configure basic Zend Server options';
    
    protected $_commandClasses = array(
        'set-password'      => 'ZServer_Cli_Tool_Command_Setup_SetPassword',
        'set-license'       => 'ZServer_Cli_Tool_Command_Setup_SetLicense',
        'show-eula'         => 'ZServer_Cli_Tool_Command_Setup_ShowEula',
        'accept-eula'       => 'ZServer_Cli_Tool_Command_Setup_AcceptEula',
        'add-api-key'       => 'ZServer_Cli_Tool_Command_Setup_AddApiKey',
        'list-api-keys'     => 'ZServer_Cli_Tool_Command_Setup_ListApiKeys',
        'show-api-key'      => 'ZServer_Cli_Tool_Command_Setup_ShowApiKey',
        'revoke-api-key'    => 'ZServer_Cli_Tool_Command_Setup_RevokeApiKey',
        'set-nodes-license' => 'ZServer_Cli_Tool_Command_Setup_SetNodesLicense',
        'create-cluster-db' => 'ZServer_Cli_Tool_Command_Setup_CreateClusterDb',
    );
    
    /**
     * Pre-run actions: bootstrap the Zend Server GUI model environment so that
     * we can re-use GUI code
     * 
     * @see ZServer_Cli_Tool::_preRun()
     */
    protected function _preRun()
    {
        if (getenv('ZEND_SERVER_GUI_PATH')) { 
            define('ZEND_SERVER_GUI_PATH', getenv('ZEND_SERVER_GUI_PATH'));
        } else {
            throw new ZServer_Cli_Tool_Exception("the ZEND_SERVER_GUI_PATH environment variable is not set", 1);
        }
        
        define('ZEND_SERVER_APP_PATH',     ZEND_SERVER_GUI_PATH . '/application');
        define('ZEND_SERVER_INSTALL_PATH', dirname(ZEND_SERVER_GUI_PATH));

        require_once ZEND_SERVER_APP_PATH . '/application.php'; // Define the ZEND_SERVER_APP_PATH

        define('ZEND_SERVER_SKU_PATH',  ZEND_SERVER_APP_PATH . DIRECTORY_SEPARATOR . ZEND_SERVER_APP_VERSION);
        define('ZEND_SERVER_DATA_PATH', ZEND_SERVER_APP_PATH . '/data');
        define('ZEND_SERVER_CONFIGURATION_FILE', ZEND_SERVER_DATA_PATH . '/zend-server.ini');
        define('ZEND_SERVER_USER_CONFIGURATION_FILE', ZEND_SERVER_DATA_PATH . '/zend-server-user.ini');
        
        // Set the include path
        $includePath = ZEND_SERVER_GUI_PATH . '/library';
        
        // This switch has no break on purpose (fall through all editionis)
        switch(ZEND_SERVER_APP_VERSION) {
            case 'EE': $includePath .= PATH_SEPARATOR . ZEND_SERVER_APP_PATH . '/EE/models';
            case 'PE': $includePath .= PATH_SEPARATOR . ZEND_SERVER_APP_PATH . '/PE/models';
            case 'CE': $includePath .= PATH_SEPARATOR . ZEND_SERVER_APP_PATH . '/CE/models';
        }

        set_include_path($includePath . PATH_SEPARATOR . get_include_path());

        Zend_Loader_Autoloader::getInstance()->setFallbackAutoloader(true);

		/**
 		 * Disable Zend Framework locale cache
 		 * Bug in PHP 5.3/Windows, function: is_writable() return flase on writable directory
 		 */
        Zend_Locale::disableCache(true);

        /**
         * In order to use date() style formatting also in Zend_Date
         */
        Zend_Date::setOptions(array('format_type' => 'php'));
        
        // Load application configuration
        ApplicationModel::loadConfiguration();
        
        define('ZEND_SERVER_TEMP_PATH', ApplicationModel::getGuiTempDir());
        
        // Set default DB adapter
        Context_Db_DefaultAdapter::init();
        
        // Enable debug logging
        if ($this->_debugEnabled && $this->_outStream instanceof ZServer_Cli_OutputStream_Standard) { 
            Zwas_Log::setLoggerWriter(new Zend_Log_Writer_Stream($this->_outStream->getStdErrStreamName()));
        }
    }
}