<?php

require_once 'ZendX/Service/ZendServer/libraryVersion.php';
require_once 'ZendX/Service/ZendServer/Message.php';

class ZendX_Service_ZendServer_LibraryInfo
{
    /**
     * Application Status Codes
     */
    const OK               = 'deployed';
    const STAGING          = 'staging';
    const ACTIVATING       = 'activating';
    const DEACTIVATING     = 'deactivating';
    const UNSTAGING        = 'unstaging';
    const UNKNOWN          = 'unknown';
    const INCONSISTENT     = 'inconsistent';
    const UPLOAD_ERROR     = 'uploadError';
    const STAGE_ERROR      = 'stageError';
    const ACTIVAE_ERROR    = 'activeError';
    const DEACTIVATE_ERROR = 'deactivateError';
    const UNSTAGE_ERROR    = 'unstageError';
    const NOT_EXISTS       = 'notExists';

    /**
     * Library ID
     *
     * @var integer
     */
    protected $_id = null;

    /**
     * Library name
     *
     * @var string
     */
    protected $_libName = null;

    /**
     * Installed Location
     *
     * @var string
     */
    protected $_installedLocation = null;

    /**
     * Library Status. Will be equal to one of ths status constants.
     *
     * @var string
     */
    protected $_status = null;

    /**
     * List of versions and their status
     *
     * @var array of Libraryversion objects
     */
    protected $_versions = array();

    /**
     * Deployed versions
     *
     * @var array of deployed versions
     */
    protected $_deployedVersions = array();

    /**
     * Array of messages
     *
     * @var array of Message objects
     */
    protected $_messages = array();

	/**
     * Create a LibraryInfo object from a response XML element
     *
     * @param SimpleXMLElement $responseXml
     */
    public function __construct(SimpleXMLElement $xml)
    {
        if ($xml->getName() != 'libraryInfo') {
            require_once 'ZendX/Service/ZendServer/Exception.php';
            throw new ZendX_Service_ZendServer_Exception("Unexpected base XML element:  {$xml->getName()}");
        }

        $this->_id = (int) $xml->libraryId;
        if (! $this->_id) {
            require_once 'ZendX/Service/ZendServer/Exception.php';
            throw new ZendX_Service_ZendServer_Exception("Missing required applicationInfo element: id");
        }

        // Set all required string elements
        $elements = array(
            'libraryName',
            'status',
        );

        foreach($elements as $element) {
            $this->{"_$element"} = (string) $xml->{$element};
            if (! $this->{"_$element"}) {
                require_once 'ZendX/Service/ZendServer/Exception.php';
                throw new ZendX_Service_ZendServer_Exception("Missing required libraryInfo element: $element");
            }
        }

        

         foreach($xml->libraryVersions->children() as $libraryVersion) {
             $this->_versions[] = new ZendX_Service_ZendServer_LibraryVersion($libraryVersion);
         }

//         foreach((array)$xml->deployedVersions->deployedVersion as $version) {
//             $this->_deployedVersions[] = (string) $version;
//         }

//         foreach($xml->messageList->children() as $message) {
//             $this->_messages[] = new ZendX_Service_ZendServer_Message($message);
//         }
    }

	/**
	 * Get the application ID
	 *
     * @return integer
     */
    public function getId()
    {
        return $this->_id;
    }


	/**
	 * Get the application name as defined in the app package
	 *
     * @return string
     */
    public function getAppName()
    {
        return $this->_libraryName;
    }


	/**
     * Get the application Status
     *
     * This will be equal to one of the status constants
     *
	 * @return string
     */
    public function getStatus()
    {
        return $this->_status;
    }

	/**
     * Get the list of servers on which the app is deployed
     *
     * @return array
     */
    public function getServers()
    {
        return $this->_servers;
    }

	/**
	 * Get the list of deployed versions
	 *
     * @return array
     */
    public function getVersions()
    {
        return $this->_versions;
    }

    /**
     * Get the list of messages, if any
     *
     * @return array Array of Message objects
     */
    public function getMessages()
    {
        return $this->_messages;
    }
}