<?php

class ZendX_Service_ZendServer_LibraryVersion
{
    /**
     * Library Version  ID
     * 
     * @var integer
     */
    protected $_libraryVersionId = null;
    
    /**
     * Deployed version
     * 
     * @var string
     */
    protected $_version = null;
    
    /**
     * Library Status
     * 
     * @var string
     */
    protected $_status = null;
    
    /**
     * Installed location
     *
     * @var string
     */
    protected $_installedLocation;
    
    public function __construct(SimpleXMLElement $xml)
    {
        if ($xml->getName() != 'libraryVersion') {
            require_once 'ZendX/Service/ZendServer/Exception.php'; 
            throw new ZendX_Service_ZendServer_Exception("Unexpected base XML element: {$xml->getName()}");
        }
        
        if (strlen((string) $xml->libraryVersionId)) { // <id> may be integer 0, this is valid
            $this->_libraryVersionId = (int) $xml->libraryVersionId;
        } else {
            require_once 'ZendX/Service/ZendServer/Exception.php';
            throw new ZendX_Service_ZendServer_Exception("Missing required library Version element: libraryVersionId");
        }
        
        // Set all required string elements
        $elements = array(
            //'libraryVersionId', 
            'version', 
        	'status',
        	'installedLocation',
        );
        
        foreach($elements as $element) { 
            $this->{"_$element"} = (string) $xml->{$element};
        }
    }
    
	/**
	 * Get the server ID
	 * 
     * @return integer
     */
    public function getId()
    {
        return $this->_libraryVersionId;
    }

	/**
	 * Get the  library version Id
	 * 
     * @return string
     */
    public function getLibraryVersionId()
    {
        return $this->_libraryVersionId;
    }

    /**
     * Get the  library version
     *
     * @return string
     */
    public function getVersion()
    {
    	return $this->_version;
    }
    
	/**
	 * Get the application's status on this particular server
	 * 
	 * This is one of the status constants defined in ZendX_Service_ZendServer_ApplicationInfo
	 * 
     * @return string
     */
    public function getStatus()
    {
        return $this->_status;
    }
	
    /**
     * Get the Library's installed location
     *
     * 
     * @return string
     */
    public function getInstalledLocation()
    {
    	return $this->_installedLocation;
    }
}